

Public Class KeyFrameCollection
    Implements IEnumerable, IDisposable

    Private mobjItems As System.Collections.ArrayList

    Public Sub New()
        mobjItems = New System.Collections.ArrayList()
    End Sub

    Public Sub SwapItem(ByVal Index1 As Integer, ByVal Index2 As Integer)
        Dim intFirst, intLast As Integer
        Dim objFirst, objLast As Object

        ' do err chech
        If Index1 < 0 Or Index1 > mobjItems.Count - 1 Then Exit Sub
        If Index2 < 0 Or Index2 > mobjItems.Count - 1 Then Exit Sub
        If Index1 = Index2 Then Exit Sub

        intFirst = CInt(IIf(Index1 < Index2, Index1, Index2))
        intLast = CInt(IIf(Index1 > Index2, Index1, Index2))

        ' store items
        objFirst = mobjItems(intFirst)
        objLast = mobjItems(intLast)

        ' remove from list
        mobjItems.RemoveAt(intLast)
        mobjItems.RemoveAt(intFirst)
        ' reinsert
        mobjItems.Insert(intFirst, objFirst)
        mobjItems.Insert(intLast, objLast)
    End Sub

    Public Function GetEnumerator() As System.Collections.IEnumerator Implements System.Collections.IEnumerable.GetEnumerator
        Return New KeyFrameCollectionEnumerator(Me)
    End Function

    Public Sub Dispose() Implements System.IDisposable.Dispose
        GC.SuppressFinalize(Me)
    End Sub

    Public Function Add(ByVal Item As KeyFrame) As Integer
        Return mobjItems.Add(Item)
    End Function

    Public Sub Clear()
        mobjItems.Clear()
    End Sub

    Public ReadOnly Property Count() As Integer
        Get
            Return mobjItems.Count
        End Get
    End Property

    Public Function IndexOf(ByVal Item As KeyFrame) As Integer
        Return mobjItems.IndexOf(Item)
    End Function

    Public Sub Insert(ByVal Index As Integer, ByVal Item As KeyFrame)
        mobjItems.Insert(Index, Item)
    End Sub

    Default Public Property Item(ByVal Index As Integer) As KeyFrame
        Get
            Return CType(mobjItems.Item(Index), KeyFrame)
        End Get
        Set(ByVal Value As KeyFrame)
            mobjItems(Index) = Value
        End Set
    End Property

    Public Sub Remove(ByVal Item As KeyFrame)
        mobjItems.Remove(Item)
    End Sub

    Public Sub Remove(ByVal Index As Integer)
        mobjItems.RemoveAt(Index)
    End Sub

    Public Sub Remove(ByVal Index As Integer, ByVal Count As Integer)
        mobjItems.RemoveRange(Index, Count)
    End Sub

    Class KeyFrameCollectionEnumerator
        Implements IEnumerator

        Private mobjParent As KeyFrameCollection
        Private CurrentEntry As KeyFrame
        Private mintCurrentIndex As Integer = 0

        ' Create a reference back to the object instance that 
        ' owns this Enumberator.
        Public Sub New(ByVal xParent As KeyFrameCollection)
            mobjParent = xParent
        End Sub

        Public ReadOnly Property Current() As Object Implements System.Collections.IEnumerator.Current
            Get
                Return CurrentEntry
            End Get
        End Property

        Public Function MoveNext() As Boolean Implements System.Collections.IEnumerator.MoveNext
            ' If we've itterated until we've reached the last one,
            ' then stop!
            If mintCurrentIndex < mobjParent.Count Then
                CurrentEntry = mobjParent.Item(mintCurrentIndex)
                mintCurrentIndex += 1
                Return True
            Else
                mobjParent.Dispose()
                Return False
            End If
        End Function

        Public Sub Reset() Implements System.Collections.IEnumerator.Reset
            mintCurrentIndex = 0
        End Sub
    End Class
End Class