'Created by: X
'Copyright: Created by: X
'E-Mail: mailto:createdbyx@yahoo.com
'Web: http://www.createdbyx.com/
'Date: November 23, 2003
'=============================================
Friend Interface IBindable

End Interface

Friend Interface IBindings
    Function GetListOfBoundObjects() As IBindable()
End Interface

Public Enum TrackPlayStyle
    ''' <summary>
    ''' Specifies that the track only be played once.
    ''' </summary>
    Once = 1
    ''' <summary>
    ''' Specifies that the track be played over and over again. When the track reaches the end it will restart from the start.
    ''' </summary>
    Looping = 2
    Linear = 4
    KeyLock = 8
End Enum

''' <summary>
''' Contains types relates to animation.
''' </summary>
'Namespace Animation

''' <summary>
''' Contais a collection of interfaces.ikey types.
''' </summary>
Friend Class Track
    Inherits CollectionBase
    Implements IDisposable

    ' Friend Name As String = ""
    Protected Friend mintCurrentKey As Integer
    Protected Friend mlngBaseTime As Long
    Protected Friend mblnEnabled As Boolean = True
    Protected Friend mintPlayStyle As TrackPlayStyle = TrackPlayStyle.Looping Or TrackPlayStyle.Linear
    ' Protected Friend mobjTrackRef As Track


    Public Overridable Property PlayType() As TrackPlayStyle
        Get
            Return mintPlayStyle
        End Get
        Set(ByVal Value As TrackPlayStyle)
            mintPlayStyle = Value
        End Set
    End Property

    ''' <summary>
    ''' Gets/sets weather this track is enabled or not.
    ''' </summary>
    ''' <returns>Returns True is thrack is enabled.</returns>
    Public Overridable Property Enabled() As Boolean
        Get
            Return mblnEnabled
        End Get
        Set(ByVal Value As Boolean)
            mblnEnabled = Value
        End Set
    End Property

    ''' <summary>
    ''' Gets/Sets the base time for the track. Base time represents the root time for the track. 
    ''' </summary>
    ''' <returns>Returns the base time for the track.</returns>
    ''' <remarks>
    ''' The falowing example better explains the purpose for the BaseTime property ...        
    ''' <example> 
    ''' <code>
    '''Dim lngCurrentTime As Long
    '''
    '''If AnimationTrack.Count = 0 Then Exit Function
    '''' get current time
    '''lngCurrentTime = GetTheCurrentTime()
    '''
    '''If lngCurrentTime > AnimationTrack.BaseTime + AnimationTrack.Item(AnimationTrack.CurrentKeyIndex).Time Then AnimationTrack.CurrentKeyIndex += 1
    '''
    '''If AnimationTrack.CurrentKeyIndex > AnimationTrack.Count - 1 Then
    '''    AnimationTrack.BaseTime = lngCurrentTime
    '''    AnimationTrack.CurrentKeyIndex = 0
    '''    Exit Function
    '''End If 
    '''
    '''' calc ammount of transition
    '''Dim S As Single
    '''
    '''S = CSng((lngCurrentTime - AnimationTrack.BaseTime) / AnimationTrack.Item(AnimationTrack.CurrentKeyIndex).Time)
    '''If S > 1 Then S = 1
    ''' </code>
    ''' </example> 
    ''' </remarks>
    Public Overridable Property BaseTime() As Long
        Get
            Return mlngBaseTime
        End Get
        Set(ByVal Value As Long)
            mlngBaseTime = Value
        End Set
    End Property

    ''' <summary>
    ''' Gets/Sets teh current index to a key  in the track.
    ''' </summary>
    ''' <returns></returns>
    ''' <exception cref="ArgumentOutOfRangeException">
    '''     Will throw a ArgumentOutOfRangeException Exception if the index is set outside the bounds of the collection or if there are no items in the collection.
    ''' </exception>
    Public Overridable Property CurrentKeyIndex() As Integer
        Get
            Return mintCurrentKey
        End Get
        Set(ByVal Value As Integer)
            mintCurrentKey = Value
            'If mintCurrentKey < 0 Then mintCurrentKey = 0
            'If mintCurrentKey > Me.Count - 1 Then mintCurrentKey = Me.Count - 1
        End Set
    End Property

    Public Overridable Function GetCurrentKey() As Key
        If Me.Count = 0 Then Return Nothing
        Return Me.DoItem(mintCurrentKey)
    End Function

    Public Overridable Function GetPreviousKey() As Key
        If Me.Count = 0 Then Return Nothing
        If mintCurrentKey = 0 Then
            Return Me.DoItem(mintCurrentKey)
        Else
            Return Me.DoItem(mintCurrentKey - 1)
        End If
    End Function

    Public Overridable Function GetNextKey() As Key
        If Me.Count = 0 Then Return Nothing
        If mintCurrentKey = Me.Count - 1 Then
            Return Me.DoItem(0)
        Else
            Return Me.DoItem(mintCurrentKey + 1)
        End If
    End Function

    Public Overridable Function GetLastKey() As Key
        If Me.Count = 0 Then Return Nothing
        Return Me.DoItem(Me.Count - 1)
    End Function

    Public ReadOnly Property EndOfTrack() As Boolean
        Get
            Return mintCurrentKey = Me.Count - 1
        End Get
    End Property

    Public ReadOnly Property StartOfTrack() As Boolean
        Get
            Return mintCurrentKey = 0
        End Get
    End Property

    '''' <summary>
    '''' Swaps the location of two objects in the collection.
    '''' </summary>
    '''' <param name="Item1">The first reference to a interfaces.ikey type previously added to the collection.</param>
    '''' <param name="Item2">The second reference to a interfaces.ikey type previously added to the collection.</param>
    'Protected Friend Sub DoSwap(ByVal Item1 As Key, ByVal Item2 As Key)
    '    Dim Index1, Index2 As Integer

    '    Index1 = Me.DoIndexOf(Item1)
    '    Index2 = Me.DoIndexOf(Item2)

    '    ' dont need to do anything
    '    If Index1 = Index2 Then Exit Sub
    '    ' do err chech
    '    If Index1 < 0 Or Index1 > Me.Count - 1 Then Throw New ArgumentOutOfRangeException("Item1", "The item was not previosly added to this collection.")
    '    If Index2 < 0 Or Index2 > Me.Count - 1 Then Throw New ArgumentOutOfRangeException("Item2", "The item was not previosly added to this collection.")

    '    Swap(Index1, Index2)
    'End Sub

    '''' <summary>
    '''' Swaps the location of two items in the collection.
    '''' </summary>
    '''' <param name="Index1">The index of the first item.</param>
    '''' <param name="Index2">The index of the second item.</param>
    'Public Sub Swap(ByVal Index1 As Integer, ByVal Index2 As Integer)
    '    Dim intFirst, intLast As Integer
    '    Dim objFirst, objLast As Key

    '    ' dont need to do anything
    '    If Index1 = Index2 Then Exit Sub
    '    ' do err chech
    '    If Index1 < 0 Or Index1 > Me.Count - 1 Then Throw New ArgumentOutOfRangeException("Index1")
    '    If Index2 < 0 Or Index2 > Me.Count - 1 Then Throw New ArgumentOutOfRangeException("Index2")

    '    intFirst = CType(IIf(Index1 < Index2, Index1, Index2), Integer)
    '    intLast = CType(IIf(Index1 > Index2, Index1, Index2), Integer)

    '    ' store items
    '    objFirst = Me.DoItem(intFirst)
    '    objLast = Me.DoItem(intLast)

    '    ' remove from list
    '    Me.Remove(intLast)
    '    Me.Remove(intFirst)
    '    ' reinsert
    '    Me.DoInsert(intFirst, objFirst)
    '    Me.DoInsert(intLast, objLast)
    'End Sub

    ''' <summary>
    ''' Disposes of all items in the collection by calling the Clear method.
    ''' </summary>
    Public Sub Dispose() Implements System.IDisposable.Dispose
        Me.Clear()
    End Sub

    ''' <summary>
    ''' Adds a interfaces.ikey type to the collection.
    ''' </summary>
    ''' <param name="Item">A reference to a interfaces.ikey type.</param>
    ''' <returns>Returns the index that the item was added to.</returns>
    Protected Friend Function DoAdd(ByVal Item As Key) As Integer
        Return Me.InnerList.Add(Item)
    End Function

    ''' <summary>
    ''' Returns the index of a interfaces.ikey type previously added to the collection.
    ''' </summary>
    ''' <param name="Item">A reference to a interfaces.ikey type that was previously added.</param>
    Protected Friend Function DoIndexOf(ByVal Item As Key) As Integer
        Return Me.InnerList.IndexOf(Item)
    End Function

    ''' <summary>
    ''' Inserts a interfaces.ikey type into the collection.
    ''' </summary>
    ''' <param name="Index">The index of the item to be retrieved.</param>
    ''' <param name="Item">A reference to a interfaces.ikey type.</param>
    Protected Friend Sub DoInsert(ByVal Index As Integer, ByVal Item As Key)
        Me.InnerList.Insert(Index, Item)
    End Sub

    ''' <summary>
    ''' Gets/Sets a item in the collection.
    ''' </summary>
    ''' <param name="Index">The index of the item to be retrieved.</param>
    ''' <returns>Returns a reference to a interfaces.ikey type.</returns>
    Protected Friend Property DoItem(ByVal Index As Integer) As Key
        Get
            Return DirectCast(Me.InnerList(Index), Key)
        End Get
        Set(ByVal Value As Key)
            Me.InnerList(Index) = Value
        End Set
    End Property

    ''' <summary>
    ''' Removes an item.
    ''' </summary>
    ''' <param name="Item">A reference to a interfaces.ikey type previously added to the collection.</param>
    Protected Friend Sub DoRemove(ByVal Item As Key)
        Me.InnerList.Remove(Item)
    End Sub

    ''' <summary>
    ''' Removes an item.
    ''' </summary>
    ''' <param name="Index">The index of the item to be removed.</param>
    Public Sub Remove(ByVal Index As Integer)
        Me.InnerList.RemoveAt(Index)
    End Sub

    ''' <summary>
    ''' Removes a number of items.
    ''' </summary>
    ''' <param name="Index">The index of the first item to be removed.</param>
    ''' <param name="Count">The number of items to removed.</param>
    Public Sub Remove(ByVal Index As Integer, ByVal Count As Integer)
        Me.InnerList.RemoveRange(Index, Count)
    End Sub

    ''' <summary>
    ''' Removes a number of items from the collection.
    ''' </summary>
    ''' <param name="Indexes">The index(s) of the items to be removed.</param>
    Public Sub Remove(ByVal Indexes() As Integer)
        Dim idx As Integer = Indexes.GetUpperBound(0)

        While idx > Indexes.GetLowerBound(0)
            Me.Remove(Indexes(idx))
            idx -= 1
        End While
    End Sub
End Class
'End Namespace

