'Created by: X
'Copyright: Created by: X
'Web: http://www.createdbyx.com/
'Date: May 17, 2004
'=============================================

Public Class AnimatedStrip
    Inherits DXTTesterExt.BaseExample
    Implements DXTTesterExt.IExample

    Private mobjTimer As HiResolutionTimer

    Protected Friend Overrides Sub DoCleanUp()
        mobjTimer = Nothing
        RemoveHandler mobjForm.Click, AddressOf Form_Click
        RemoveHandler mobjForm.KeyDown, AddressOf Form_KeyDown
        RemoveAnimation("AnimatedStrip")
        MyBase.DoCleanUp()
    End Sub

    Protected Overrides Sub DoInit(Optional ByVal Windowed As Boolean = True)
        MyBase.DoInit(Windowed)

        mobjForm.Text = "Animated Strip Example"

        AddTexture("Sprite", LoadResourceImage("Turkey.png"))

        EnableLighting(False)
        SetCullMode(Direct3D.Cull.None)

        AddAnimation("AnimatedStrip")
        AddTrack("AnimatedStrip", New String() {"Left", "Top", "Width", "Height"})
        ' default playstyle is linear so we need to specify keylock otherwise animation will not be frame based
        SetTrackPlayStyle("AnimatedStrip", New String() {"Left", "Top", "Width", "Height"}, _
                          TrackPlayStyle.Looping Or TrackPlayStyle.KeyLock)

        mobjTimer = New HiResolutionTimer()
        Dim Times() As Long
        Times = BuildTimeKeys(0, 12, CLng(mobjTimer.GetTimeInfo(TimeInformation.Frequency) / 30))  ' 30 fps
        AddTrackKey("AnimatedStrip", "Left", Times, New Single() {0, 84, 168, 252, 336, 420, 0, 84, 168, 252, 336, 420})
        AddTrackKey("AnimatedStrip", "Top", Times, New Single() {0, 0, 0, 0, 0, 0, 105, 105, 105, 105, 105, 105})
        AddTrackKey("AnimatedStrip", "Width", Times, New Single() {84, 84, 84, 84, 84, 84, 84, 84, 84, 84, 84, 84})
        AddTrackKey("AnimatedStrip", "Height", Times, New Single() {105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105})

        AddHandler mobjForm.Click, AddressOf Form_Click
        AddHandler mobjForm.KeyDown, AddressOf Form_KeyDown
    End Sub

    Public Function BuildTimeKeys(ByVal StartTime As Long, ByVal KeyCount As Integer, ByVal TimeBetweenKeys As Long) As Long()
        Dim Times() As Long
        If KeyCount <= 0 Then Return Nothing
        ReDim Times(KeyCount - 1)
        Dim idx As Integer
        For idx = 0 To Times.Length - 1
            Times(idx) = StartTime + (idx * TimeBetweenKeys)
        Next
        Return Times
    End Function

    Private Sub Form_Click(ByVal sender As Object, ByVal e As System.EventArgs)
        mobjForm.Close()
    End Sub

    Private Sub Form_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs)
        Select Case e.KeyCode
            Case Keys.Escape
                mobjForm.Close()

            Case Keys.Space
                RestartAnimation("AnimatedStrip", mobjTimer.GetTimeInfo(TimeInformation.SystemTime))

            Case Keys.O
                SetTrackPlayStyle("AnimatedStrip", New String() {"Left", "Top", "Width", "Height"}, TrackPlayStyle.Once Or TrackPlayStyle.KeyLock)

            Case Keys.L
                SetTrackPlayStyle("AnimatedStrip", New String() {"Left", "Top", "Width", "Height"}, TrackPlayStyle.Looping Or TrackPlayStyle.KeyLock)

        End Select

    End Sub

    Public Sub Run() Implements DXTTesterExt.IExample.Run
        Dim R As RectangleF

        DoInit()

        Dim Msg As String
        Msg = "Press Escape or click to exit" & vbCrLf
        Msg &= "Spacebar resets animation to begining." & vbCrLf
        Msg &= "O Key sets play style to play once" & vbCrLf
        Msg &= "L Key sets play style to loop" & vbCrLf

        While mobjForm.Created
            BeginScene()

            Dim Values() As Single
            GetTrackValue("AnimatedStrip", New String() {"Left", "Top", "Width", "Height"}, _
                          mobjTimer.GetTimeInfo(TimeInformation.SystemTime), Values)
            R = New RectangleF(Values(0), Values(1), Values(2), Values(3))

            BeginSpriteRender()
            DrawSprite("Sprite", R, 0, 0)
            EndSpriteRender()

            DrawText(Msg, 0, 120)

            EndScene()
            Application.DoEvents()
        End While

        DoCleanUp()
    End Sub

    Public Function GetDescription() As String Implements DXTTesterExt.IExample.GetDescription
        Return "Demonstrates code that uses the Animation features provided by DXTools."
    End Function

    Public Function GetImage() As System.Drawing.Image Implements DXTTesterExt.IExample.GetImage
        Return New Bitmap(Me.GetType, "AniStrip.png")
    End Function

    Public Function GetName() As String Implements DXTTesterExt.IExample.GetName
        Return "Animated Strip"
    End Function
End Class
