'Created by: X
'Copyright: Created by: X
'E-Mail: mailto:createdbyx@yahoo.com
'Web: http://www.createdbyx.com/
'Date: May 16, 2004
'=============================================

Imports Microsoft.DirectX.Direct3D.DrawTextFormat
Imports CBX.Libs.DX9Tools.GeneralHelpers
Imports CBX.Libs
Imports Microsoft.DirectX

Public Class BuildRectangleStrips
    Inherits BaseExample
    Implements IExample

    Private Geo As DX9Tools.PCTGeometry
    Private Rect As RectangleF
    Private Showing As ShowWhat = ShowWhat.Horiz
    Private TheCount As Integer = 10
    Private mobjTextures As DX9Tools.Collections.TextureCollection
    Private mobjCameras As DX9Tools.Collections.CameraCollection
    Private mobjFont As Direct3D.Font

    Private Enum ShowWhat
        Horiz = 0
        Vert = 1
        LinePoints = 2
    End Enum

    Public Sub Run() Implements DX9Tools_Tester.IExample.Run
        Dim Msg As String

        DoInit()

        While mobjForm.Created
            mobjGraphics.Device.SetTransform(DirectX.Direct3D.TransformType.Projection, mobjCameras(0).Projection)

            mobjGraphics.BeginScene()

            ' render the generated rectangle strip
            If Showing <> ShowWhat.LinePoints Then
                mobjGraphics.Device.RenderState.FillMode = DirectX.Direct3D.FillMode.Solid
                RenderGeometry(mobjGraphics.Device, Geo, mobjTextures)
            End If

            mobjGraphics.Device.RenderState.FillMode = DirectX.Direct3D.FillMode.WireFrame
            RenderGeometry(mobjGraphics.Device, Geo)

            ' render info
            Msg = "Press 1 for horizontial strips" & vbCrLf
            Msg &= "Press 2 for vertical strips" & vbCrLf
            Msg &= "Press 3 for line points" & vbCrLf
            Msg &= "Use + - to add and remove strips" & vbCrLf
            Msg &= "Left/Right mouse buttons manipulate the rectangle" & vbCrLf
            Msg &= "---------------------------------" & vbCrLf
            Msg &= TheCount.ToString & " - " & Showing.ToString

            mobjFont.DrawText(Msg, mobjForm.ClientRectangle, Left Or Top, Color.Yellow)

            mobjGraphics.EndScene()

            Application.DoEvents()
        End While

        DoCleanUp()
    End Sub

    Protected Overrides Sub DoCleanUp()
        mobjFont.Dispose()
        mobjFont = Nothing
        Geo = Nothing
        mobjTextures.Clear()
        mobjTextures = Nothing
        mobjCameras.Clear()
        mobjCameras = Nothing
        RemoveHandler mobjForm.MouseMove, AddressOf Form_MouseMove
        RemoveHandler mobjForm.KeyDown, AddressOf Form_KeyDown

        MyBase.DoCleanUp()
    End Sub

    Protected Overrides Sub DoInit()
        MyBase.DoInit()

        mobjForm.Text = "Build Rectangle Strips Example"
        AddHandler mobjForm.MouseMove, AddressOf Form_MouseMove
        AddHandler mobjForm.KeyDown, AddressOf Form_KeyDown

        TheCount = 10

        ' create graphics object
        mobjCameras = New DX9Tools.Collections.CameraCollection()
        mobjTextures = New DX9Tools.Collections.TextureCollection()
        ' create one camera and stoer it in a camera collection
        mobjCameras.Add(New DX9Tools.Camera(mobjGraphics))
        ' extract an embeded image resource and load it as a D3D texture
        mobjTextures.Add(LoadResourceImage(Me.GetType.Namespace & ".CBX.png"))

        ' create fonst
        mobjFont = New Direct3D.Font(mobjGraphics.Device, New Font(mobjForm.Font.FontFamily, 16, FontStyle.Bold, GraphicsUnit.Pixel))

        ' setup render states
        mobjGraphics.Device.RenderState.Lighting = False
        mobjGraphics.Device.RenderState.CullMode = DirectX.Direct3D.Cull.None


        mobjCameras(0).Projection = Matrix.OrthoOffCenterLH(0, mobjForm.ClientSize.Width, mobjForm.ClientSize.Height, 0, mobjCameras(0).NearPlane, mobjCameras(0).FarPlane)
        Rect = New RectangleF(0, 0, mobjForm.ClientSize.Width, mobjForm.ClientSize.Height)
        Geo = New DX9Tools.PCTGeometry(New DX9Tools.IndexedTextureBlendingItem(mobjGraphics.Device.TextureState(0), 0))

        SetupGeo()
    End Sub

    Private Sub SetupGeo()
        Dim Verts() As Vector3
        Dim tVerts() As Vector2

        Select Case Showing
            Case ShowWhat.Horiz, ShowWhat.Vert
                If Showing = ShowWhat.Horiz Then
                    Verts = DX9Tools.GeoHelpers.BuildHorizRectangleStrip(Rect, TheCount, tVerts)
                Else
                    Verts = DX9Tools.GeoHelpers.BuildVertRectangleStrip(Rect, TheCount, tVerts)
                End If
                Geo.VertexCount = Verts.Length
                Geo.PrimitiveType = DirectX.Direct3D.PrimitiveType.TriangleStrip
                Dim idx As Integer
                For idx = 0 To Verts.Length - 1
                    Geo.SetVertex(idx, Verts(idx), Color.White)
                    Geo.SetVertex(idx, tVerts(idx))
                Next

            Case ShowWhat.LinePoints
                Verts = DX9Tools.GeoHelpers.BuildLinePoints(New Vector3(Rect.Left, Rect.Top, 0), _
                                                            New Vector3(Rect.Right, Rect.Bottom, 0), TheCount)
                Geo.VertexCount = Verts.Length
                Geo.PrimitiveType = DirectX.Direct3D.PrimitiveType.PointList
                Dim idx As Integer

                For idx = 0 To Verts.Length - 1
                    Geo.SetVertex(idx, Verts(idx), Color.White)
                Next
        End Select
    End Sub

    Private Sub Form_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs)
        Select Case e.Button
            Case MouseButtons.Left
                Rect.Location = New PointF(e.X, e.Y)
                SetupGeo()

            Case MouseButtons.Right
                If e.X < Rect.Left Then Exit Select
                If e.Y < Rect.Top Then Exit Select
                Rect.Size = New SizeF(e.X - Rect.Left, e.Y - Rect.Top)
                SetupGeo()

        End Select
    End Sub

    Private Sub Form_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs)
        Select Case e.KeyCode
            Case Keys.D1
                Showing = ShowWhat.Horiz

            Case Keys.D2
                Showing = ShowWhat.Vert

            Case Keys.D3
                Showing = ShowWhat.LinePoints
                If TheCount < 2 Then TheCount = 2

            Case Keys.Add
                If e.Control Then TheCount += 10 Else TheCount += 1

            Case Keys.Subtract
                If e.Control Then TheCount -= 10 Else TheCount -= 1
                If Showing <> ShowWhat.LinePoints Then
                    If TheCount < 1 Then TheCount = 1
                Else
                    If TheCount < 2 Then TheCount = 2
                End If
        End Select

        SetupGeo()
    End Sub

    Public Function GetDescription() As String Implements DX9Tools_Tester.IExample.GetDescription
        Return "Demonstrates the code to build a rectangle strip."
    End Function

    Public Function GetImage() As System.Drawing.Image Implements DX9Tools_Tester.IExample.GetImage
        Return New Bitmap(Me.GetType, "RectStrip.png")
    End Function

    Public Function GetName() As String Implements DX9Tools_Tester.IExample.GetName
        Return "Build Rectangle Strips"
    End Function
End Class