'Created by: X
'Copyright: Created by: X
'E-Mail: mailto:createdbyx@yahoo.com
'Web: http://www.createdbyx.com/
'Date: January 6, 2004
'=============================================

Imports Microsoft.DirectX.Direct3D.Geometry
Imports Microsoft.DirectX.Direct3D.DrawTextFormat
Imports CBX.Libs.DX9Tools.GeneralHelpers
Imports CBX.Libs
Imports Microsoft.DirectX

Public Class GenerateEllipse
    Inherits BaseExample
    Implements IExample

    Private Verts As DX9Tools.PCTGeometry
    Private Rotation As Integer = 12
    Private mobjTextures As DX9Tools.Collections.TextureCollection
    Private mobjFont As Direct3D.Font
    Private Rect As Rectangle
    Private RectLine As Direct3D.Line
    Private Outline() As Vector2

    Public Sub Run() Implements DX9Tools_Tester.IExample.Run
        Dim Msg As String

        Me.DoInit()

        While mobjForm.Created
            ' setup 2D projection matrix
            Dim Proj As Matrix
            Proj = Matrix.OrthoOffCenterLH(0, mobjForm.ClientSize.Width, mobjForm.ClientSize.Height, 0, 1, 1000)
            mobjGraphics.Device.SetTransform(DirectX.Direct3D.TransformType.Projection, Proj)

            mobjGraphics.BeginScene(Color.LightBlue)

            SetupRotation()

            ' draw textured ellipse
            mobjGraphics.Device.RenderState.FillMode = DirectX.Direct3D.FillMode.Solid
            RenderGeometry(mobjGraphics.Device, Verts, mobjTextures)

            ' draw ellipse bounds
            mobjGraphics.Device.RenderState.FillMode = DirectX.Direct3D.FillMode.WireFrame
            RenderGeometry(mobjGraphics.Device, Verts)

            ' draw rect bounds
            RectLine.Draw(DX9Tools.GeoHelpers.Vector2FromRectangle(Rect, Direct3D.PrimitiveType.LineStrip), Color.Green)

            ' draw ellipse bounds
            RectLine.Draw(Outline, Color.Red)

            Msg = "Left/Right mouse buttons manipulate the rectangle" & vbCrLf
            Msg &= "Middle mouse button sets number of points in ellipse" & vbCrLf
            Msg &= "Number of points: " & Rotation.ToString

            mobjFont.DrawText(Msg, mobjForm.ClientRectangle, Left Or Top, Color.Yellow)

            mobjGraphics.EndScene()
            Application.DoEvents()
        End While

        Me.DoCleanUp()
    End Sub

    Protected Overrides Sub DoCleanUp()
        mobjFont.Dispose()
        mobjFont = Nothing
        Verts = Nothing
        mobjTextures.Dispose()
        mobjTextures = Nothing
        RectLine.Dispose()
        RectLine = Nothing
        RemoveHandler mobjForm.MouseMove, AddressOf Form_MouseMove
        MyBase.DoCleanUp()
    End Sub

    Protected Overrides Sub DoInit()
        MyBase.DoInit()

        Rotation = 12
        Rect = New Rectangle(50, 50, 250, 250)
        mobjForm.Text = "Generate Ellipse Example"
        AddHandler mobjForm.MouseMove, AddressOf Form_MouseMove

        mobjTextures = New DX9Tools.Collections.TextureCollection()
        mobjTextures.DisposeOnClear = True
        mobjTextures.Add(LoadResourceImage(Me.GetType.Namespace & ".CBX.png"))

        ' create fonst
        mobjFont = New Direct3D.Font(mobjGraphics.Device, New Font(mobjForm.Font.FontFamily, 16, FontStyle.Bold, GraphicsUnit.Pixel))

        mobjGraphics.Device.RenderState.Lighting = False
        mobjGraphics.Device.RenderState.CullMode = DirectX.Direct3D.Cull.None

        Verts = Nothing
        Verts = New DX9Tools.PCTGeometry(Direct3D.PrimitiveType.TriangleFan, _
                                         New DX9Tools.IndexedTextureBlendingItem(mobjGraphics.Device.TextureState(0), 0))

        RectLine = New Direct3D.Line(mobjGraphics.Device)

        ' setup geometry
        SetupGeo()
    End Sub

    Private Sub SetupGeo()
        Dim Pts() As Vector3
        Dim idx As Integer
        Dim HalfSize As SizeF
        Dim R As RectangleF

        HalfSize = New SizeF(Rect.Width \ 2, Rect.Height \ 2)
        R = New RectangleF(New PointF(), HalfSize)  '-HalfSize.Width, -HalfSize.Height, Rect.Width, Rect.Height)

        ' generate ellipse points for geometry
        ' NOTE: If you don't specify a floting point divide csng(360 / Rotation) you may get rotation anamolies.
        DX9Tools.GeoHelpers.GenerateEllipsePoints(R, DX9Tools.Orientation.XY, Math.Abs(Rotation), _
                                                  DegreeToRadian(360.0F / Rotation), DegreeToRadian(0), Pts)

        ' build point list for when we draw the outling of the ellipse
        DX9Tools.GeoHelpers.GenerateEllipsePoints(R, Math.Abs(359), DegreeToRadian(360 / 359), DegreeToRadian(0), Outline)

        For idx = 0 To Outline.Length - 1
            Outline(idx).Add(New Vector2(Rect.Left + HalfSize.Width, Rect.Top + HalfSize.Height))
        Next

        ' add geometry ellipse points
        Verts.VertexCount = Pts.Length
        For idx = 0 To Pts.Length - 1
            Verts.SetVertex(idx, New Direct3D.CustomVertex.PositionColoredTextured(Pts(idx), Color.White.ToArgb, 0, 0))
        Next

        'generate textured ellipse points for texture mapping. 
        R = New RectangleF(0, 0, 2, 2)
        ' NOTE: If you don't specify a floting point divide csng(360 / Rotation) you may get rotation anamolies.
        DX9Tools.GeoHelpers.GenerateEllipsePoints(R, DX9Tools.Orientation.XY, Math.Abs(Rotation), _
                                                  DegreeToRadian(360.0F / Rotation), DegreeToRadian(0), Pts)

        ' set texture U/V for ellipse points
        For idx = 0 To Pts.Length - 1
            Verts.SetVertex(idx, Pts(idx).X, Pts(idx).Y)
        Next
    End Sub

    Private Function SetupRotation() As Matrix
        Dim M As Matrix = Matrix.Identity
        Dim HalfSize, Size, Center As Vector3

        Size = New Vector3(Rect.Width, Rect.Height, 0)
        HalfSize = New Vector3(Size.X / 2, Size.Y / 2, Size.Z / 2)
        Center = Vector3.Add(New Vector3(Rect.Left, Rect.Top, 0), HalfSize)

        M.Multiply(Matrix.Translation(Center))
        mobjGraphics.Device.SetTransform(DirectX.Direct3D.TransformType.World, M)
        Return M
    End Function

    Private Sub Form_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs)
        Select Case e.Button
            Case MouseButtons.Left
                Rect.Location = New Point(e.X, e.Y)
                ' Rotation = 0
                SetupGeo()

            Case MouseButtons.Right
                Rect.Width = e.X - Rect.Left
                If Rect.Width < 1 Then Rect.Width = 1
                '  Rect.Width = Rect.Width \ 2

                Rect.Height = e.Y - Rect.Top
                If Rect.Height < 1 Then Rect.Height = 1
                ' Rect.Height = Rect.Height \ 2
                'Rotation = 4
                SetupGeo()

            Case MouseButtons.Middle
                Rotation = e.X
                If Rotation < 3 Then Rotation = 3
                If Rotation > 359 Then Rotation = 359
                SetupGeo()
        End Select
    End Sub


    Public Function GetDescription() As String Implements DX9Tools_Tester.IExample.GetDescription
        Return "Demonstrates code that uses a OrthoOffCenterLH projection matrix."
    End Function

    Public Function GetImage() As System.Drawing.Image Implements DX9Tools_Tester.IExample.GetImage
        Return New Bitmap(Me.GetType, "GenerateEllipse.png")
    End Function

    Public Function GetName() As String Implements DX9Tools_Tester.IExample.GetName
        Return "Generate Ellipse"
    End Function
End Class
