'Created by: X
'Copyright: Created by: X
'E-Mail: mailto:createdbyx@yahoo.com
'Web: http://www.createdbyx.com/
'Date: May 25, 2004
'=============================================

Imports CBX.Libs
Imports Microsoft.DirectX

Public Class MultiProjection
    Inherits BaseExample
    Implements IExample


    Private xLine(1), yLine(1), zLine(1) As Direct3D.CustomVertex.PositionColored
    Private mobjMesh As DX9Tools.MeshEx
    Private mobjCameras As DX9Tools.Collections.CameraCollection
    Private Rot As Vector3
    Private mobjFont As Direct3D.Font


    Public Sub Run() Implements DX9Tools_Tester.IExample.Run
        Dim WorldMatrix As Matrix = Matrix.Identity

        Me.DoInit()

        While mobjForm.Created

            mobjGraphics.BeginScene(Color.LightBlue)

            ' updates the View and projection matricies
            ' the projection matrix used is a standard 
            ' PerspectiveFovLH matrix
            mobjCameras(0).UpdateVP()

            'render box
            '------------------
            mobjGraphics.Device.RenderState.Lighting = True ' enable lighting
            ' WorldMatrix.RotateYawPitchRoll(Rot.X, Rot.Y, Rot.Z)   ' rotate the world objects
            WorldMatrix = Matrix.Identity
            WorldMatrix.Multiply(Matrix.RotationZ(Rot.Z))
            WorldMatrix.Multiply(Matrix.RotationY(Rot.Y))
            WorldMatrix.Multiply(Matrix.RotationX(Rot.X))
            mobjGraphics.Device.SetTransform(DirectX.Direct3D.TransformType.World, WorldMatrix)

            ' draw cube
            mobjGraphics.Device.VertexFormat = mobjMesh.Mesh.VertexFormat
            DX9Tools.GeneralHelpers.RenderMeshBruteForce(mobjMesh)

            ' render lines
            '---------------
            ' world matrix is already rotated so we only need to translate it to position the axis widget
            WorldMatrix.Multiply(Matrix.RotationAxis(Rot, Rot.X))
            WorldMatrix.Multiply(Matrix.Translation(55, mobjForm.ClientSize.Height - 55, 0))

            ' with a OrthoOffCenterLH matrix we can specify exact positioning on screen
            ' for example we translated the worldmatrix by (55, Me.ClientSize.Height - 55, 0)
            ' meaning we want our axis widget to be rendered in the lower left of the screen.
            ' We simply pass in 2D screen co-ordanants to where we want to render our geometry at.
            mobjGraphics.Device.SetTransform(DirectX.Direct3D.TransformType.World, WorldMatrix)

            ' we setup a OrthoOffCenterLH matrix using camera 2
            mobjCameras(1).UpdateVP()

            ' disable lighting
            mobjGraphics.Device.RenderState.Lighting = False

            ' render axis widget
            mobjGraphics.Device.VertexFormat = xLine(0).Format
            mobjGraphics.Device.DrawUserPrimitives(DirectX.Direct3D.PrimitiveType.LineList, 1, xLine)
            mobjGraphics.Device.DrawUserPrimitives(DirectX.Direct3D.PrimitiveType.LineList, 1, yLine)
            mobjGraphics.Device.DrawUserPrimitives(DirectX.Direct3D.PrimitiveType.LineList, 1, zLine)

            ' end and present scene
            mobjGraphics.EndScene()

            Application.DoEvents()
        End While

        Me.DoCleanUp()
    End Sub

    Private Sub SetupGeo()
        mobjMesh = New DX9Tools.MeshEx()
        mobjMesh.Mesh = Direct3D.Mesh.Box(mobjGraphics.Device, 50, 50, 50)
        Dim Mat As Direct3D.Material
        Mat.Diffuse = Color.Blue ' material for cube
        mobjMesh.AddMaterial(Mat)

        xLine(0) = New Direct3D.CustomVertex.PositionColored(0, 0, 0, Color.Red.ToArgb)
        xLine(1) = New Direct3D.CustomVertex.PositionColored(50, 0, 0, Color.red.ToArgb)

        yLine(0) = New Direct3D.CustomVertex.PositionColored(0, 0, 0, Color.Green.ToArgb)
        yLine(1) = New Direct3D.CustomVertex.PositionColored(0, -50, 0, Color.Green.ToArgb)

        zLine(0) = New Direct3D.CustomVertex.PositionColored(0, 0, 0, Color.blue.ToArgb)
        zLine(1) = New Direct3D.CustomVertex.PositionColored(0, 0, -50, Color.blue.ToArgb)
    End Sub

    Protected Overrides Sub DoCleanUp()
        mobjFont.Dispose()
        mobjFont = Nothing
        mobjCameras.Dispose()
        mobjCameras = Nothing
        mobjMesh.Dispose()
        mobjMesh = Nothing
        Erase xLine
        Erase yLine
        Erase zLine
        xLine = Nothing
        yLine = Nothing
        zLine = Nothing
        RemoveHandler mobjForm.MouseMove, AddressOf Form_MouseMove
        MyBase.DoCleanUp()
    End Sub

    Protected Overrides Sub DoInit()
        MyBase.DoInit()

        mobjForm.Text = "Multi Projection Example"
        AddHandler mobjForm.MouseMove, AddressOf Form_MouseMove

        mobjGraphics.Device.VertexFormat = Direct3D.CustomVertex.PositionOnly.Format
        With mobjGraphics.Device.RenderState
            .Lighting = True
            .Ambient = Color.FromArgb(255, 128, 128, 128)
            .CullMode = DirectX.Direct3D.Cull.None
        End With
        With mobjGraphics.Device
            .Lights(0).FromLight(DX9Tools.LightHelpers.CreateDirectionalLight(Color.White, DX9Tools.Direction.Z))
            .Lights(0).Enabled = True
            .Lights(0).Commit()
        End With

        ' add standard 3D projection camera
        mobjCameras = New DX9Tools.Collections.CameraCollection()
        ' we create a regular 3D camera
        mobjCameras.Add(New DX9Tools.Camera(mobjGraphics))
        ' we create a 2d camera
        mobjCameras.Add(New DX9Tools.Camera(mobjGraphics))
        mobjCameras(1).CameraType = DX9Tools.CameraType.Camera2D
        mobjCameras(1).ProjectionRectangle = DX9Tools.Conversion.RectangleToRectangleF(mobjForm.ClientRectangle)

        ' create fonst
        mobjFont = New Direct3D.Font(mobjGraphics.Device, New Font(mobjForm.Font.FontFamily, 16, FontStyle.Bold, GraphicsUnit.Pixel))

        ' setup geometry
        SetupGeo()
    End Sub

    Private Sub Form_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs)
        Dim Value As Integer

        ' restrict rotations to 0 to 359
        Value = e.X
        If Value < 0 Then Value = 0
        If Value > 359 Then Value = 359

        Select Case e.Button
            Case MouseButtons.Left
                Rot.X = Direct3D.Geometry.DegreeToRadian(Value)

            Case MouseButtons.Right
                Rot.Y = Direct3D.Geometry.DegreeToRadian(Value)

            Case MouseButtons.Middle
                Rot.Z = Direct3D.Geometry.DegreeToRadian(Value)
        End Select
    End Sub

    Public Function GetDescription() As String Implements DX9Tools_Tester.IExample.GetDescription
        Return "Still a work in progress..."
    End Function

    Public Function GetImage() As System.Drawing.Image Implements DX9Tools_Tester.IExample.GetImage
        Return Nothing 'New Bitmap(Me.GetType, "OrthoProject.png")
    End Function

    Public Function GetName() As String Implements DX9Tools_Tester.IExample.GetName
        Return "Multi Projection"
    End Function
End Class
