'Created by: X
'Copyright: Created by: X
'E-Mail: mailto:createdbyx@yahoo.com
'Web: http://www.createdbyx.com/
'Date: May 16, 2004
'=============================================
Imports Microsoft.DirectX
Imports Microsoft.DirectX.Direct3D.DrawTextFormat
Imports CBX.Libs.DX9Tools.GeneralHelpers
Imports CBX.Libs

Public Class OrthoProjection
    Inherits BaseExample
    Implements IExample

    Private mobjTextures As DX9Tools.Collections.TextureCollection
    Private Verts As DX9Tools.PCTGeometry
    Private Rotation As Integer
    Private Rect As Rectangle
    Private mobjFont As Direct3D.Font

    Public Sub Run() Implements DX9Tools_Tester.IExample.Run
        Dim Msg As String

        DoInit()

        While mobjForm.Created
            mobjGraphics.BeginScene()

            ' setup 2D projection matrix
            Dim Proj As Matrix
            Proj = Matrix.OrthoOffCenterLH(0, mobjForm.ClientSize.Width, mobjForm.ClientSize.Height, 0, 1, 1000)
            mobjGraphics.Device.SetTransform(DirectX.Direct3D.TransformType.Projection, Proj)

            ' setup rotation matrix
            SetupRotation()

            ' render rect
            mobjGraphics.Device.RenderState.FillMode = DirectX.Direct3D.FillMode.Solid
            RenderGeometry(mobjGraphics.Device, Verts, mobjTextures)
            
            ' render wireframe
            mobjGraphics.Device.RenderState.FillMode = DirectX.Direct3D.FillMode.WireFrame
            RenderGeometry(mobjGraphics.Device, Verts, Nothing)


            Msg = "Left/Right mouse buttons manipulate the rectangle" & vbCrLf
            Msg &= "Middle mouse button rotates the rectangle"
           
            mobjFont.DrawText(Msg, mobjForm.ClientRectangle, Left Or Top, Color.Yellow)

            mobjGraphics.EndScene()

            Application.DoEvents()
        End While

        DoCleanUp()
    End Sub

    Private Sub SetupRotation()
        Dim M As Matrix = Matrix.Identity
        Dim HalfSize, Center As Vector3

        HalfSize = New Vector3(Rect.Width \ 2, Rect.Height \ 2, 0)
        Center = Vector3.Add(New Vector3(Rect.Left, Rect.Top, 0), HalfSize)

        M.RotateZ(Direct3D.Geometry.DegreeToRadian(Rotation))
        M.Multiply(Matrix.Translation(Center))
        mobjGraphics.Device.SetTransform(DirectX.Direct3D.TransformType.World, M)
    End Sub

    Protected Overrides Sub DoCleanUp()
        mobjFont.Dispose()
        mobjFont = Nothing
        Verts = Nothing
        mobjTextures.Clear()
        mobjTextures = Nothing
        RemoveHandler mobjForm.MouseMove, AddressOf Form_MouseMove
        MyBase.DoCleanUp()
    End Sub

    Protected Overrides Sub DoInit()
        MyBase.DoInit()

        Rotation = 0
        Rect = New Rectangle(50, 50, 150, 150)
        mobjForm.Text = "Orthographic Projection Example"
        AddHandler mobjForm.MouseMove, AddressOf Form_MouseMove

        mobjTextures = New DX9Tools.Collections.TextureCollection()
        mobjTextures.DisposeOnClear = True
        mobjTextures.Add(LoadResourceImage(Me.GetType.Namespace & ".CBX.png"))

        ' create fonst
        mobjFont = New Direct3D.Font(mobjGraphics.Device, New Font(mobjForm.Font.FontFamily, 16, FontStyle.Bold, GraphicsUnit.Pixel))

        mobjGraphics.Device.RenderState.Lighting = False
        mobjGraphics.Device.RenderState.CullMode = DirectX.Direct3D.Cull.None

        ' setup vertex buffer
        Verts = Nothing
        Verts = New DX9Tools.PCTGeometry(Direct3D.PrimitiveType.TriangleFan, 4, _
                                         New DX9Tools.IndexedTextureBlendingItem(mobjGraphics.Device.TextureState(0), 0))

        ' setup geometry
        SetupGeo()
    End Sub

    Private Sub SetupGeo()
        Dim HalfSize As Size

        HalfSize = New Size(Rect.Width \ 2, Rect.Height \ 2)
        Verts.SetVertex(0, New Direct3D.CustomVertex.PositionColoredTextured(-HalfSize.Width, -HalfSize.Height, 0, Color.White.ToArgb, 0, 0))
        Verts.SetVertex(1, New Direct3D.CustomVertex.PositionColoredTextured(HalfSize.Width, -HalfSize.Height, 0, Color.White.ToArgb, 1, 0))
        Verts.SetVertex(2, New Direct3D.CustomVertex.PositionColoredTextured(HalfSize.Width, HalfSize.Height, 0, Color.White.ToArgb, 1, 1))
        Verts.SetVertex(3, New Direct3D.CustomVertex.PositionColoredTextured(-HalfSize.Width, HalfSize.Height, 0, Color.White.ToArgb, 0, 1))
    End Sub

    Private Sub Form_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs)
        Select Case e.Button
            Case MouseButtons.Left
                Rect.Location = New Point(e.X, e.Y)
                Rotation = 0
                SetupGeo()

            Case MouseButtons.Right
                Rect.Width = e.X - Rect.Left
                If Rect.Width < 1 Then Rect.Width = 1

                Rect.Height = e.Y - Rect.Top
                If Rect.Height < 1 Then Rect.Height = 1
                Rotation = 0
                SetupGeo()

            Case MouseButtons.Middle
                Rotation = e.X
                If Rotation < 0 Then Rotation = 0
                If Rotation > 359 Then Rotation = 359
        End Select
    End Sub

    Public Function GetDescription() As String Implements DX9Tools_Tester.IExample.GetDescription
        Return "Demonstrates code that uses a OrthoOffCenterLH projection matrix."
    End Function

    Public Function GetImage() As System.Drawing.Image Implements DX9Tools_Tester.IExample.GetImage
        Return New Bitmap(Me.GetType, "OrthoProject.png")
    End Function

    Public Function GetName() As String Implements DX9Tools_Tester.IExample.GetName
        Return "Orthographic Projection"
    End Function
End Class