'Created by: X
'Copyright: Created by: X
'E-Mail: mailto:createdbyx@yahoo.com
'Web: http://www.createdbyx.com/
'Date: May 20, 2004
'=============================================

Imports CBX.Libs.DX9Tools.GeneralHelpers
Imports CBX.Libs.DX9Tools.GeoHelpers
Imports Microsoft.DirectX.Direct3D.DrawTextFormat
Imports CBX.Libs
Imports Microsoft.DirectX

Public Class Parallax
    Inherits BaseExample
    Implements IExample

    Private mobjTextures As DX9Tools.Collections.TextureCollection
    Private mobjMesh As DX9Tools.PCTGeometry
    Private mIntRotationSpeed As Integer = 3
    Private msngZoomSpeed As Single = 10
    Private mobjCameras As DX9Tools.Collections.CameraCollection
    Private mobjFont As Direct3D.Font
    Private mobjGameTimer As DX9Tools.GameTools.GameTimer


    Protected Overrides Sub DoInit()
        MyBase.DoInit()
        mobjForm.Text = "Parallax Effect Example"
        AddHandler mobjForm.KeyDown, AddressOf Form_KeyDown

        mIntRotationSpeed = 3
        msngZoomSpeed = 10

        mobjCameras = New DX9Tools.Collections.CameraCollection()
        mobjCameras.Add(New DX9Tools.Camera(mobjGraphics))
        mobjCameras(0).SetPosition(0, 0, -100)
        mobjCameras(0).FarPlane = 10000

        mobjTextures = New DX9Tools.Collections.TextureCollection()
        mobjTextures.Add(LoadResourceImage(Me.GetType.Namespace & ".CBX.png"))

        mobjGraphics.Device.RenderState.Lighting = False
        mobjGraphics.Device.RenderState.CullMode = Direct3D.Cull.None
        mobjGameTimer = New DX9Tools.GameTools.GameTimer()

        ' create fonst
        mobjFont = New Direct3D.Font(mobjGraphics.Device, New Font(mobjForm.Font.FontFamily, 16, FontStyle.Bold, GraphicsUnit.Pixel))

        SetupGeo()
    End Sub

    Protected Overrides Sub DoCleanUp()
        mobjFont.Dispose()
        mobjFont = Nothing
        mobjMesh.Dispose()
        mobjMesh = Nothing
        mobjCameras.Dispose()
        mobjCameras = Nothing
        mobjTextures.DisposeOnClear = True
        mobjTextures.Dispose()
        mobjTextures = Nothing
        mobjGameTimer = Nothing
        RemoveHandler mobjForm.KeyDown, AddressOf Form_KeyDown
        MyBase.DoCleanUp()
    End Sub

    Public Sub Run() Implements DX9Tools_Tester.IExample.Run
        Dim M As Matrix = Matrix.Identity
        Dim blnZoomingOut As Boolean = True
        Dim intRotation As Integer = 0
        Dim Msg As String

        DoInit()
        While mobjForm.Created

            mobjGraphics.BeginScene()

            mobjCameras(0).UpdateVP()

            ' restrict animation to 30 moves per second (1000 milli sec in 1 sec divided by 30 equals 33.3333333)
            If mobjGameTimer.GetCurrentTime > mobjGameTimer.LastCapturedTime + MilliToTicks(33) Then
                Dim sngZoom As Single

                With mobjCameras(0)
                    If blnZoomingOut Then sngZoom = .Position.Z - msngZoomSpeed Else sngZoom = .Position.Z + msngZoomSpeed
                    .SetPosition(DX9Tools.SetVector.Z, sngZoom)

                    If blnZoomingOut = True And .Position.Z < -1000 Then blnZoomingOut = False
                    If blnZoomingOut = False And .Position.Z > -msngZoomSpeed Then blnZoomingOut = True

                End With

                intRotation += mIntRotationSpeed
                If intRotation > 360 Then intRotation = 360 - intRotation

                M = Matrix.Identity
                M.RotateZ(Direct3D.Geometry.DegreeToRadian(intRotation))
                mobjGameTimer.CaptureTime()
            End If

            mobjGraphics.Device.Transform.World = M
            RenderGeometry(mobjGraphics.Device, mobjMesh, mobjTextures)


            Msg = "Left/Right keys change rotation speed" & vbCrLf
            Msg &= "Up/Down keys change zoom speed" & vbCrLf & vbCrLf
            Msg &= "Rotation Speed: " & mIntRotationSpeed.ToString & vbCrLf
            Msg &= "Zoom Speed: " & msngZoomSpeed.ToString

            mobjFont.DrawText(Msg, mobjForm.ClientRectangle, Left Or Top, Color.Red)


            mobjGraphics.EndScene()
            Application.DoEvents()

        End While

        DoCleanUp()
    End Sub

    Private Sub SetupGeo()
        Dim V() As Vector3
        Dim tUV() As Vector2
        Dim idx As Integer

        ' we use the Vector3FromRectangle to generate an array of 4 vector3 points
        V = Vector3FromRectangle(New RectangleF(-2500, -2500, 5000, 5000), DirectX.Direct3D.PrimitiveType.TriangleStrip)
        ' get texture size info
        Dim Size As Size = GetTextureSize(mobjTextures(0))
        ' we use the UVFromRectangle method to generate an array of 4 vector2 points
        tUV = UVFromRectangle(Size.Width, Size.Height, 0, 0, Size.Width * 100, Size.Height * 100)

        ' create and setup mesh
        mobjMesh = New DX9Tools.PCTGeometry(DirectX.Direct3D.PrimitiveType.TriangleStrip, V.Length, _
                                            New DX9Tools.IndexedTextureBlendingItem(mobjGraphics.Device.TextureState(0), 0))
         ' combine verts and textures
        For idx = 0 To V.Length - 1
            mobjMesh.SetVertex(idx, New Direct3D.CustomVertex.PositionColoredTextured(V(idx), Color.White.ToArgb, tUV(idx).X, tUV(idx).Y))
        Next
    End Sub

    Private Sub Form_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs)
        Select Case e.KeyCode
            Case Keys.Left
                mIntRotationSpeed -= 1

            Case Keys.Right
                mIntRotationSpeed += 1

            Case Keys.Up
                msngZoomSpeed += 1

            Case Keys.Down
                msngZoomSpeed -= 1
        End Select
        ' restrict zoom and rotation speeds
        If msngZoomSpeed < 0 Then msngZoomSpeed = 0
        If msngZoomSpeed > 100 Then msngZoomSpeed = 100
        If mIntRotationSpeed < -359 Then mIntRotationSpeed = -359
        If mIntRotationSpeed > 359 Then mIntRotationSpeed = 359
    End Sub

    Public Function GetDescription() As String Implements DX9Tools_Tester.IExample.GetDescription
        Return "Demonstrates code that uses blaw."
    End Function

    Public Function GetImage() As System.Drawing.Image Implements DX9Tools_Tester.IExample.GetImage
        Return New Bitmap(Me.GetType, "Parallax.png")
    End Function

    Public Function GetName() As String Implements DX9Tools_Tester.IExample.GetName
        Return "Parallax Effect"
    End Function
End Class
