'Created by: X
'Copyright: Created by: X
'E-Mail: mailto:createdbyx@yahoo.com
'Web: http://www.createdbyx.com/
'Date: May 16, 2004
'=============================================

Imports Microsoft.DirectX.Direct3D.Geometry
Imports Microsoft.DirectX.Direct3D.DrawTextFormat
Imports Microsoft.DirectX

Public Class Quaternions
    Inherits BaseExample
    Implements IExample

    Private Verts() As Direct3D.CustomVertex.PositionOnly
    Private Rotation As Integer
    Private mobjPosition As Vector3
    Private mobjRAA As Vector3
    Private mobjAxis As Vector3
    Private mobjFont As Direct3D.Font

    Public Sub Run() Implements DX9Tools_Tester.IExample.Run
        Dim Msg As String

        DoInit()
        Dim Dev As Direct3D.Device = GetD3DDevice()

        While mobjForm.Created
            BeginScene()

            ' setup 2D projection matrix
            Dev.Transform.Projection = Matrix.OrthoOffCenterLH(0, mobjForm.ClientSize.Width, mobjForm.ClientSize.Height, 0, 1, 1000)


            ' set vertex format
            Dev.VertexFormat = Verts(0).Format

            RenderCircle(Color.Red, mobjAxis)
            RenderCircle(Color.Orange, mobjPosition)
            RenderCircle(Color.Cyan, mobjRAA)

            ' redner info
            Msg = "Left mouse button sets position" & vbCrLf
            Msg &= "RIght mouse button sets axis" & vbCrLf
            Msg &= "Middle mouse button sets the rotation" & vbCrLf
            Msg &= "Rotation: " & Rotation.ToString & vbCrLf
            ' Msg &= "GetRotation: " & DX9Tools.MathHelpers.GetRotation(DX9Tools.Orientation.XY, mobjPosition, mobjAxis)

            mobjFont.DrawText(Msg, mobjForm.ClientRectangle, Left Or Top, Color.Yellow)

            Msg = "Position: " & Color.Orange.ToString
            mobjFont.DrawText(Msg, mobjForm.ClientRectangle, Left Or Bottom, Color.Orange)

            Msg = "Axis: " & Color.Red.ToString
            mobjFont.DrawText(Msg, mobjForm.ClientRectangle, Center Or Bottom, Color.Red)

            Msg = "Rotated Point: " & Color.Cyan.ToString
            mobjFont.DrawText(Msg, mobjForm.ClientRectangle, Right Or Bottom, Color.Cyan)

            EndScene()

            Application.DoEvents()
        End While
        Dev = Nothing
        DoCleanUp()
    End Sub

    Private Sub RenderCircle(ByVal Color As Color, ByVal Pos As Vector3)
        ' calc primitive count
        Dim pCount As Integer
        pCount = GetPrimitiveCount(Direct3D.PrimitiveType.TriangleList, Verts.Length)

        Dim Dev As Direct3D.Device = GetD3DDevice()

        ' setup material
        Dim Mat As Direct3D.Material
        Mat.Ambient = Color
        Mat.Diffuse = Mat.Ambient
        Dev.Material = Mat

        ' setup translation matrix for position
        Dim M As Matrix = Matrix.Identity
        M.Multiply(Matrix.Translation(Pos))
        Dev.SetTransform(Direct3D.TransformType.World, M)

        ' render geometry
        Dev.DrawUserPrimitives(Direct3D.PrimitiveType.TriangleList, pCount, Verts)
    End Sub

    Protected Overrides Sub DoCleanUp()
        mobjFont.Dispose()
        mobjFont = Nothing
        Erase Verts
        Verts = Nothing
        RemoveHandler mobjForm.MouseMove, AddressOf Form_MouseMove
        MyBase.DoCleanUp()
    End Sub

    Protected Overrides Sub DoInit()
        MyBase.DoInit()

        Rotation = 0
        mobjPosition = New Vector3(mobjForm.ClientSize.Width \ 2, mobjForm.ClientSize.Width \ 2, 0)
        mobjAxis = Vector3.Add(mobjPosition, New Vector3(25, 25, 0))
        mobjForm.Text = "Rotate Around Axis Example"
        AddHandler mobjForm.MouseMove, AddressOf Form_MouseMove

        ' create fonst
        mobjFont = New Direct3D.Font(mobjGraphics.Device, New Font(mobjForm.Font.FontFamily, 16, FontStyle.Bold, GraphicsUnit.Pixel))

        mobjGraphics.Device.RenderState.Lighting = True
        mobjGraphics.Device.RenderState.Ambient = Color.White
        mobjGraphics.Device.RenderState.CullMode = DirectX.Direct3D.Cull.None

        ' setup geometry
        SetupGeo()
    End Sub

    Private Sub SetupGeo()
        Dim R As RectangleF
        Dim V() As Vector3
        Dim idx As Integer

        R = New RectangleF(0, 0, 50, 50)
        DX9Tools.GeoHelpers.GenerateEllipsePoints(R, DX9Tools.Orientation.XY, 18, DegreeToRadian(360 / 18), 0, V)
        ReDim Verts(V.Length)
        Verts(Verts.Length - 1).SetPosition(V(0))
        For idx = 0 To V.Length - 1
            Verts(idx).SetPosition(V(idx))
        Next
    End Sub

    Private Sub Form_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs)
        Select Case e.Button
            Case MouseButtons.Left
                mobjPosition = New Vector3(e.X, e.Y, 0)

            Case MouseButtons.Right
                mobjAxis = New Vector3(e.X, e.Y, 0)

            Case MouseButtons.Middle
                Rotation = e.X
                If Rotation < 0 Then Rotation = 0
                If Rotation > 359 Then Rotation = 359
        End Select

        ' mobjRAA = DX9Tools.MathHelpers.RotateAroundAxis(mobjAxis, mobjPosition, New Vector3(0, 0, DegreeToRadian(Rotation)))
        'mobjRAA = New Quaternion(mobjAxis.X, mobjAxis.Y, mobjAxis.Z, DegreeToRadian(Rotation))
        Dim Tmp As Quaternion = Quaternion.RotationYawPitchRoll(DegreeToRadian(Rotation), 0, 0)
        ' Tmp.RotateAxis(New Vector3(1, 0.5, 0.25), DegreeToRadian(Rotation))
        '  Tmp = Tmp.Identity
        ' Tmp.RotationYawPitchRoll(0, 0, DegreeToRadian(Rotation))
        Dim M As Matrix = Matrix.Identity
        M.Multiply(Matrix.RotationQuaternion(Tmp))
        ' M.Multiply(Matrix.Translation(mobjPosition))
        mobjRAA = New Vector3(M.M41, M.M42, M.M43)
    End Sub

    Public Function GetDescription() As String Implements DX9Tools_Tester.IExample.GetDescription
        Return "Demonstrates code that uses a Quaternion to rotate a point around an axis."
    End Function

    Public Function GetImage() As System.Drawing.Image Implements DX9Tools_Tester.IExample.GetImage
        Return New Bitmap(Me.GetType, "RotateAroundAxis.png")
    End Function

    Public Function GetName() As String Implements DX9Tools_Tester.IExample.GetName
        Return "Quaternions"
    End Function
End Class