'Created by: X
'Copyright: Created by: X
'E-Mail: mailto:createdbyx@yahoo.com
'Web: http://www.createdbyx.com/
'Date: May 17, 2004
'=============================================

' We import this name space so we can call MilliToTicks
Imports CBX.Libs.DX9Tools.GeneralHelpers
Imports CBX.Libs
Imports Microsoft.DirectX

Public Class StateBlockSpeedTest
    Inherits BaseExample
    Implements IExample

    Private Const Iterations As Integer = 10000
    Private mobjState1 As Direct3D.StateBlock
    Private mobjState2 As Direct3D.StateBlock
    Private mobjDefultState(2) As Direct3D.TextureState

    Protected Overrides Sub DoInit()
        MyBase.DoInit()

        ' create and capture default states
        mobjState1 = New Direct3D.StateBlock(mobjGraphics.Device, DirectX.Direct3D.StateBlockType.All)
        mobjState1.Capture()
        mobjDefultState(0) = New Direct3D.TextureState()
        mobjDefultState(0).FromTextureState(mobjGraphics.Device.TextureState(0))
        mobjDefultState(1) = New Direct3D.TextureState()
        mobjDefultState(1).FromTextureState(mobjGraphics.Device.TextureState(1))
        mobjDefultState(2) = New Direct3D.TextureState()
        mobjDefultState(2).FromTextureState(mobjGraphics.Device.TextureState(2))

        ' create and capture detail add state
        DoManualSetState()
        mobjState2 = New Direct3D.StateBlock(mobjGraphics.Device, DirectX.Direct3D.StateBlockType.All)
        mobjState2.Capture()

        ' setup form
        With mobjForm
            .Text = "StateBlock Speed Test Example"
            .ControlBox = False
            .MinimizeBox = False
            .MaximizeBox = False

            Dim L As New Label()
            L.Name = "lblDesc"
            L.Text = "Please wait while test is running..."
            L.AutoSize = True
            .Controls.Add(L)
        End With

        Application.DoEvents()
    End Sub

    Private Sub DoManualSetState()
        ' Detail add code retrieved from MFCTex.exe tool.
        mobjGraphics.Device.TextureState(0).ColorOperation = DirectX.Direct3D.TextureOperation.Modulate
        mobjGraphics.Device.TextureState(0).ColorArgument1 = DirectX.Direct3D.TextureArgument.TextureColor
        mobjGraphics.Device.TextureState(0).ColorArgument2 = DirectX.Direct3D.TextureArgument.Diffuse
        mobjGraphics.Device.TextureState(0).AlphaOperation = DirectX.Direct3D.TextureOperation.SelectArg2
        mobjGraphics.Device.TextureState(0).AlphaArgument2 = DirectX.Direct3D.TextureArgument.Diffuse

        mobjGraphics.Device.TextureState(1).ColorOperation = DirectX.Direct3D.TextureOperation.AddSigned
        mobjGraphics.Device.TextureState(1).ColorArgument1 = DirectX.Direct3D.TextureArgument.TextureColor
        mobjGraphics.Device.TextureState(1).ColorArgument2 = DirectX.Direct3D.TextureArgument.Current
        mobjGraphics.Device.TextureState(1).AlphaOperation = DirectX.Direct3D.TextureOperation.SelectArg1
        mobjGraphics.Device.TextureState(1).AlphaArgument1 = DirectX.Direct3D.TextureArgument.TextureColor

        mobjGraphics.Device.TextureState(2).ColorOperation = DirectX.Direct3D.TextureOperation.Disable
        mobjGraphics.Device.TextureState(2).AlphaOperation = DirectX.Direct3D.TextureOperation.Disable
    End Sub

    Private Sub DoStateBlockSetState()
        mobjState2.Apply()
    End Sub

    Private Sub DoManualRestoreState()
        mobjGraphics.Device.TextureState(0).FromTextureState(mobjDefultState(0))
        mobjGraphics.Device.TextureState(1).FromTextureState(mobjDefultState(1))
        mobjGraphics.Device.TextureState(2).FromTextureState(mobjDefultState(2))
    End Sub

    Private Sub DoStateBlockRestoreState()
        mobjState1.Apply()
    End Sub

    Public Sub Run() Implements DX9Tools_Tester.IExample.Run
        Dim ManualTime, StateBlockTime, StartTime, EndTime, TotalTime As Long
        Dim idx As Integer
       
        DoInit()

RunTestAgain:
        ' do manual test
        StartTime = Now.Ticks
        For idx = 1 To Iterations
            DoManualSetState()      ' set the states
            DoManualRestoreState()  ' restore the states
        Next
        EndTime = Now.Ticks
        TotalTime = EndTime - StartTime
        ManualTime = TotalTime

        ' store info
        Dim Msg As String
        Msg = "Manual Start time: " & TicksToMilli(StartTime).ToString & " - " & StartTime.ToString & vbCrLf
        Msg &= "Manual End time: " & TicksToMilli(EndTime).ToString & " - " & EndTime.ToString & vbCrLf
        Msg &= "Manual Total time: " & TicksToMilli(TotalTime).ToString & " - " & TotalTime.ToString & vbCrLf


        ' do stateblock test
        StartTime = Now.Ticks
        For idx = 1 To Iterations
            DoStateBlockSetState()     ' set the states
            DoStateBlockRestoreState() ' restore the states
        Next
        EndTime = Now.Ticks
        TotalTime = EndTime - StartTime
        StateBlockTime = TotalTime

        'store the rest of the info and display it
        Msg &= "StateBlock Start time: " & TicksToMilli(StartTime).ToString & " - " & StartTime.ToString & vbCrLf
        Msg &= "StateBlock End time: " & TicksToMilli(EndTime).ToString & " - " & EndTime.ToString & vbCrLf
        Msg &= "StateBlock Total time: " & TicksToMilli(TotalTime).ToString & " - " & TotalTime.ToString & vbCrLf
        Msg &= "Total Iterations: " & Iterations.ToString & vbCrLf
        Msg &= "--------------------------" & vbCrLf
        Msg &= "Using StateBlocks is " & Str(ManualTime / StateBlockTime) & " times faster then setting device states manualy." & vbCrLf
        Msg &= "Would you like to run the test again?"
        If MsgBox(Msg, MsgBoxStyle.YesNo Or MsgBoxStyle.Information) = MsgBoxResult.Yes Then GoTo RunTestAgain

        DoCleanUp()
    End Sub

    Public Function GetDescription() As String Implements DX9Tools_Tester.IExample.GetDescription
        Return "Calculates how long it takes manually set device states verses how long it takes using state blocks. Will run the loop " & Iterations.ToString & " times."
    End Function

    Public Function GetImage() As System.Drawing.Image Implements DX9Tools_Tester.IExample.GetImage
        Return New Bitmap(Me.GetType, "StateBlock.png")
    End Function

    Public Function GetName() As String Implements DX9Tools_Tester.IExample.GetName
        Return "StateBlock Speed Test"
    End Function

    Protected Overrides Sub DoCleanUp()
        mobjState1.Dispose()
        mobjState1 = Nothing
        mobjState2.Dispose()
        mobjState2 = Nothing
        mobjDefultState(0) = Nothing
        mobjDefultState(1) = Nothing
        mobjDefultState(2) = Nothing
        Erase mobjDefultState
        MyBase.DoCleanUp()
    End Sub
End Class

