'Created by: X
'Copyright: Created by: X
'E-Mail: mailto:createdbyx@yahoo.com
'Web: http://www.createdbyx.com/
'Date: May 16, 2004
'=============================================

Imports Microsoft.DirectX.Direct3D.DrawTextFormat
Imports Microsoft.DirectX.Direct3D.Geometry
Imports CBX.Libs.DX9Tools.GeneralHelpers
Imports CBX.Libs
Imports Microsoft.DirectX

Public Class TextureMatrix
    Inherits BaseExample
    Implements IExample

    Private mobjTextures As DX9Tools.Collections.TextureCollection
    Private Verts As DX9Tools.PCTGeometry
    Private mobjFont As Direct3D.Font
    Private TRMatrix As Matrix = Matrix.Identity
    Private TSMatrix As Matrix = Matrix.Identity
    Private TTMatrix As Matrix = Matrix.Identity
    Private TMatrix As Matrix = Matrix.Identity

    Public Sub Run() Implements DX9Tools_Tester.IExample.Run
        Dim Msg As String

        DoInit()

        While mobjForm.Created
            mobjGraphics.BeginScene()

            ' setup 2D projection matrix
            Dim Proj As Matrix
            Proj = Matrix.OrthoOffCenterLH(0, mobjForm.ClientSize.Width, mobjForm.ClientSize.Height, 0, 1, 1000)
            mobjGraphics.Device.SetTransform(DirectX.Direct3D.TransformType.Projection, Proj)

            ' render rect
            mobjGraphics.Device.RenderState.FillMode = DirectX.Direct3D.FillMode.Solid
            mobjGraphics.Device.SetTransform(DirectX.Direct3D.TransformType.Texture0, TMatrix)
            Dim WM As Matrix = Matrix.Identity
            WM.Translate(mobjForm.ClientSize.Width \ 2, mobjForm.ClientSize.Height \ 2, 0)
            mobjGraphics.Device.SetTransform(DirectX.Direct3D.TransformType.World, WM)
            RenderGeometry(mobjGraphics.Device, Verts, mobjTextures)

            ' render wireframe
            mobjGraphics.Device.RenderState.FillMode = DirectX.Direct3D.FillMode.WireFrame
            RenderGeometry(mobjGraphics.Device, Verts, Nothing)


            Msg = "Left mouse button scales texture" & vbCrLf
            Msg &= "Right mouse button rotates texture" & vbCrLf
            Msg &= "Middle mouse button translates texture"

            mobjFont.DrawText(Msg, mobjForm.ClientRectangle, Left Or Top, Color.Yellow)

            mobjGraphics.EndScene()

            Application.DoEvents()
        End While

        DoCleanUp()
    End Sub

    Protected Overrides Sub DoCleanUp()
        mobjFont.Dispose()
        mobjFont = Nothing
        Verts = Nothing
        mobjTextures.Clear()
        mobjTextures = Nothing
        RemoveHandler mobjForm.MouseMove, AddressOf Form_MouseMove
        MyBase.DoCleanUp()
    End Sub

    Protected Overrides Sub DoInit()
        MyBase.DoInit()

        mobjForm.Text = "Texture Matrix Example"
        AddHandler mobjForm.MouseMove, AddressOf Form_MouseMove

        mobjTextures = New DX9Tools.Collections.TextureCollection()
        mobjTextures.Add(LoadResourceImage(Me.GetType.Namespace & ".CBX.png"))

        ' create fonst
        mobjFont = New Direct3D.Font(mobjGraphics.Device, New Font(mobjForm.Font.FontFamily, 16, FontStyle.Bold, GraphicsUnit.Pixel))

        mobjGraphics.Device.RenderState.Lighting = False
        mobjGraphics.Device.RenderState.CullMode = DirectX.Direct3D.Cull.None

        TRMatrix = Matrix.Identity
        TSMatrix = Matrix.Identity
        TTMatrix = Matrix.Identity
        TMatrix = Matrix.Identity

        ' setup geometry
        SetupGeo()
    End Sub

    Private Sub SetupGeo()
        Dim HalfSize As Size
        Dim Rect As Rectangle = New Rectangle((mobjForm.ClientSize.Width \ 2) - 256, 0, 512, 512)

        Verts = Nothing

        Dim State As New DX9Tools.IndexedTextureBlendingItem(mobjGraphics.Device.TextureState(0), 0)
        State.State.TextureTransform = Direct3D.TextureTransform.Count2
        Verts = New DX9Tools.PCTGeometry(Direct3D.PrimitiveType.TriangleFan, 4, State)

        HalfSize = New Size(Rect.Width \ 2, Rect.Height \ 2)
        Verts.SetVertex(0, New Direct3D.CustomVertex.PositionColoredTextured(-HalfSize.Width, -HalfSize.Height, 0, Color.White.ToArgb, 0, 0))
        Verts.SetVertex(1, New Direct3D.CustomVertex.PositionColoredTextured(HalfSize.Width, -HalfSize.Height, 0, Color.White.ToArgb, 1, 0))
        Verts.SetVertex(2, New Direct3D.CustomVertex.PositionColoredTextured(HalfSize.Width, HalfSize.Height, 0, Color.White.ToArgb, 1, 1))
        Verts.SetVertex(3, New Direct3D.CustomVertex.PositionColoredTextured(-HalfSize.Width, HalfSize.Height, 0, Color.White.ToArgb, 0, 1))
    End Sub

    Private Sub Form_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs)
        Select Case e.Button
            Case MouseButtons.Left
                If e.X <> 0 And e.Y <> 0 Then
                    TSMatrix.Scale(CSng(e.X / mobjForm.ClientSize.Width), CSng(e.Y / mobjForm.ClientSize.Height), 0)
                End If

            Case MouseButtons.Right
                TRMatrix.RotateZ(DegreeToRadian(e.X))

            Case MouseButtons.Middle
                ' typically for translations we change the m41,m42,m43 values of a matrix to represent a translation
                ' MS Documentation for translations http://msdn.microsoft.com/archive/en-us/dx8_vb/directx_vb/graphics_intro_54xf.asp
                ' But for translating a texture matrix we use the m31 and m32 values.
                If e.X <> 0 Then TTMatrix.M31 = -CSng(e.X / mobjForm.ClientSize.Width)
                If e.Y <> 0 Then TTMatrix.M32 = -CSng(e.Y / mobjForm.ClientSize.Height)

        End Select

        TMatrix = Matrix.Identity
        TMatrix.Multiply(TTMatrix)
        TMatrix.Multiply(TRMatrix)
        TMatrix.Multiply(TSMatrix)
    End Sub

    Public Function GetDescription() As String Implements DX9Tools_Tester.IExample.GetDescription
        Return "Demonstrates code that manupulates the texture by manipulating the Texture0 matrix."
    End Function

    Public Function GetImage() As System.Drawing.Image Implements DX9Tools_Tester.IExample.GetImage
        Return New Bitmap(Me.GetType, "TextureMatrix.png")
    End Function

    Public Function GetName() As String Implements DX9Tools_Tester.IExample.GetName
        Return "Texture Matrix"
    End Function
End Class