'Created by: X
'Copyright: Created by: X
'E-Mail: mailto:createdbyx@gmail.com
'Web: http://www.createdbyx.com/
'Date: Monday, January 24, 2005
'=============================================


Public Class ScreenShot
    Inherits DXTTesterExt.BaseExample
    Implements DXTTesterExt.IExample


    Private BackgroundSize As Size = New Size(4096, 256)
    Private CharacterSize As Size = New Size(64, 64)
    Private MousePos As Vector2
    Private mView As Vector2
    Private mChar As Vector2
    Private Const BorderSize As Integer = 100
     Private Format As Direct3D.ImageFileFormat = Direct3D.ImageFileFormat.Bmp
    Private WithEvents Timer As New IntervalUpdate(New HiResolutionTimer(), 10)


    Protected Overrides Sub DoInit(Optional ByVal Windowed As Boolean = True)
        MyBase.DoInit()
        mobjForm.Text = "Ortho Panning with Screen Shot example"
        AddHandler mobjForm.MouseMove, AddressOf Form_MouseMove
        AddHandler mobjForm.KeyDown, AddressOf Form_KeyDown

        AddTexture("Back", LoadResourceImage("Back.png"))
        AddTexture("Char", LoadResourceImage("Char.png"))

        EnableLighting(False)

        MousePos = New Vector2(mobjForm.ClientSize.Width / 2.0F, mobjForm.ClientSize.Height / 2.0F)

        Format = Direct3D.ImageFileFormat.Bmp
        SetupGeo()

        mView.Y = (mobjForm.ClientSize.Height / 2.0F) - (BackgroundSize.Height / 2.0F)
        mView.X = -(BackgroundSize.Width / 2.0F)
    End Sub

    Protected Friend Overrides Sub DoCleanUp()
        RemoveHandler mobjForm.MouseMove, AddressOf Form_MouseMove
        RemoveHandler mobjForm.KeyDown, AddressOf Form_KeyDown

        MyBase.DoCleanUp()
    End Sub

    Public Sub Run() Implements DXTTesterExt.IExample.Run
        DoInit()

        While mobjForm.Created
            SetProjectionMatrix(Matrix.OrthoOffCenterLH(0, mobjForm.ClientSize.Width, mobjForm.ClientSize.Height, 0, 0, 1000))

            mChar = MousePos
            Me.Timer.Update()
            BeginScene()

            Dim Dev As Direct3D.Device = GetD3DDevice()

            ' draw background
            SetTextureTransform(0, Direct3D.TextureTransform.Count2)
            SetTextureAddress(0, Direct3D.TextureAddress.Mirror)
            SetTransform(Direct3D.TransformType.Texture0, Matrix.Scaling(BackgroundSize.Width / 256.0F, 1, 1))
            SetWorldMatrix(BuildMatrix(BackgroundSize.Width, BackgroundSize.Height, 1, mView.X, mView.Y, 0, 0, 0, 0))
            'SetTexture(0, "Back")
            DrawVertexBuffer("Brush", "Back")

            'draw character
            SetTextureAddress(0, Direct3D.TextureAddress.Wrap)
            SetTransform(Direct3D.TransformType.Texture0, Matrix.Identity)
            SetWorldMatrix(BuildMatrix(CharacterSize.Width, CharacterSize.Height, 1, mChar.X - (CharacterSize.Width / 2.0F), mChar.Y - (CharacterSize.Height / 2.0F), 0, 0, 0, 0))
            ' SetTexture(0, "Char")
            DrawVertexBuffer("Brush", "Char")

            'draw pan border
            SetWorldMatrix(Matrix.Identity)
            SetDrawColor(Color.Green)
            BeginLineDrawing()
            DrawRectangle(BorderSize, BorderSize, mobjForm.ClientSize.Width - BorderSize, mobjForm.ClientSize.Height - BorderSize)
            EndLineDrawing()

            Dim Msg As String
            Msg = "Press R to reset the screen" & vbCrLf
            Msg &= "Press Spacebar to take a screen shot" & vbCrLf
            Msg &= "Press F to change image save format" & vbCrLf
            Msg &= "Use Left mouse button move the character." & vbCrLf
            Msg &= "Current image save format: " & Format.ToString & vbCrLf

            SetDrawColor(Color.Yellow)
            DrawText(Msg, 0, 0)

            EndScene()

            Application.DoEvents()
        End While

        DoCleanUp()
    End Sub

    Private Sub SetupGeo()

        Dim VB As New VertexBuffer()
        VB.PrimitiveCount = 2
        VB.PrimitiveType = Direct3D.PrimitiveType.TriangleFan

        Dim mobjVerts(3) As Direct3D.CustomVertex.PositionTextured
        mobjVerts(0) = New Direct3D.CustomVertex.PositionTextured(0, 0, 0, 0, 0)
        mobjVerts(1) = New Direct3D.CustomVertex.PositionTextured(1, 0, 0, 1, 0)
        mobjVerts(2) = New Direct3D.CustomVertex.PositionTextured(1, 1, 0, 1, 1)
        mobjVerts(3) = New Direct3D.CustomVertex.PositionTextured(0, 1, 0, 0, 1)
        VB.Buffer = New Direct3D.VertexBuffer(mobjVerts(0).GetType, 4, GetD3DDevice, 0, mobjVerts(0).Format, Direct3D.Pool.Managed)
        VB.Buffer.SetData(mobjVerts, 0, Direct3D.LockFlags.None)
        Erase mobjVerts

        AddVertexBuffer("Brush", VB)
    End Sub

    Private Sub Form_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs)
        Select Case e.Button
            Case MouseButtons.Left
                MousePos = New Vector2(e.X, e.Y)
        End Select
    End Sub

    Private Sub Form_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs)

        Select Case e.KeyCode
            Case Keys.Escape
                mobjForm.Close()

            Case Keys.R
                mView.Y = (mobjForm.ClientSize.Height / 2.0F) - (BackgroundSize.Height / 2.0F)
                mView.X = -(BackgroundSize.Width / 2.0F)

            Case Keys.F
                Dim I As Integer = Format
                I += 1
                If I > 6 Then I = 0
                Format = CType(I, Direct3D.ImageFileFormat)

            Case Keys.Space
                Static LastCaptureTime As Long
                ' only allow screen capture every 1/4 second.
                If Now.Ticks - LastCaptureTime > TimeSpan.FromMilliseconds(250).Ticks Then
                    Dim File As String = IO.Path.Combine(IO.Path.GetDirectoryName(Application.ExecutablePath), "Capture." & Format.ToString)
                    If IO.File.Exists(File) Then IO.File.Delete(File)
                    ' Capturing a screen show is a easy as calling one method. Note that the 
                    ' CaptureScreenshot method saves the contents of the devices backbuffer, so make sure you dont
                    ' do a clear before capturing a screen shot.
                    ' The fallowing formats have been tested...
                    ' 1: Jpg works but part of the image may be corrupted
                    ' 2: Dds works
                    ' 3: bmp works
                    ' 4: dib works
                    ' 5: tga does not save
                    ' 6: ppm does not save
                    ' 7: png does not save
                    If CaptureScreenshot(File, Format) Then
                        ' Success!
                    Else
                        ' Capturing screen shot went bad!
                    End If
                End If
                LastCaptureTime = Now.Ticks
        End Select
    End Sub

    Public Function GetDescription() As String Implements DXTTesterExt.IExample.GetDescription
        Return "Demonstrates an example of a 2D side scroller using 3D geometry and a OrthoOffCenterLH projection matrix. " & _
               "Also demonstrates the use of the CaptureScreenshot method provided by DX9Tools."
    End Function

    Public Function GetImage() As System.Drawing.Image Implements DXTTesterExt.IExample.GetImage
        Return New Bitmap(Me.GetType, "ScreenShot.png")
    End Function

    Public Function GetName() As String Implements DXTTesterExt.IExample.GetName
        Return "Ortho Panning with Screen Shot"
    End Function

    Private Sub Timer_IntervalEllapsed() Handles Timer.IntervalEllapsed
        If MousePos.X < BorderSize Then
            mView.X += BorderSize - MousePos.X
        End If
        If MousePos.X > mobjForm.ClientSize.Width - BorderSize Then
            mView.X += -(MousePos.X - (mobjForm.ClientSize.Width - BorderSize))
        End If
    End Sub
End Class
