'Created by: X
'Copyright: Created by: X
'E-Mail: mailto:createdbyx@yahoo.com
'Web: http://www.createdbyx.com/
'Date: June 6, 2004
'=============================================

Imports CBX.Libs
Imports Microsoft.DirectX
Imports Microsoft.DirectX.Direct3D.Geometry

Public Class SkyBox
    Implements IDisposable

    Private msngSize As Single = 1000
    Private mobjCenter As Vector3 = New Vector3()
    Private mintTextureIndex(5) As Integer
    Private mobjVerts(5) As DX9Tools.PCTGeometry


    Public Sub Render(ByVal Dev As Direct3D.Device, _
    Optional ByVal Textures As DX9Tools.Collections.TextureCollection = Nothing, Optional ByVal Callback As DX9Tools.RequestTexture = Nothing)
        Dim idx As Integer
        Dim OldFormat As Direct3D.VertexFormats
        OldFormat = Dev.VertexFormat
        Dev.VertexFormat = mobjVerts(0).Format
        For idx = 0 To mobjVerts.Length - 1
            DX9Tools.GeneralHelpers.RenderGeometry(Dev, mobjVerts(idx), Textures, Callback)
        Next
        Dev.VertexFormat = OldFormat
    End Sub

    Private Sub BuildBox()
        Dim TexV(), V() As Vector3
        Dim idx As Integer


        ' create front plane
        V = DX9Tools.GeoHelpers.CreatePlane(New RectangleF(-(msngSize / 2), -(msngSize / 2), msngSize, msngSize), 1, 1, msngSize / 2)
        TexV = DX9Tools.GeoHelpers.CreatePlane(New RectangleF(0, 0, 1, -1), 1, 1)
        For idx = 0 To V.Length - 1
            mobjVerts(idx).VertexCount = V.Length
        Next

        For idx = 0 To V.Length - 1
            mobjVerts(Direct3D.CubeMapFace.PositiveZ).SetVertex(idx, Vector3.Add(V(idx), mobjCenter))
            mobjVerts(Direct3D.CubeMapFace.PositiveZ).SetVertex(idx, TexV(idx).X, TexV(idx).Y)
        Next

        ' create right plane
        V = DX9Tools.GeoHelpers.RotateVertexes(New Vector3(), New Vector3(0, DegreeToRadian(90), 0), V, False)
        For idx = 0 To V.Length - 1
            mobjVerts(Direct3D.CubeMapFace.PositiveX).SetVertex(idx, Vector3.Add(V(idx), mobjCenter))
            mobjVerts(Direct3D.CubeMapFace.PositiveX).SetVertex(idx, TexV(idx).X, TexV(idx).Y)
        Next

        ' create back plane
        V = DX9Tools.GeoHelpers.RotateVertexes(New Vector3(), New Vector3(0, DegreeToRadian(90), 0), V, False)
        For idx = 0 To V.Length - 1
            mobjVerts(Direct3D.CubeMapFace.NegativeZ).SetVertex(idx, Vector3.Add(V(idx), mobjCenter))
            mobjVerts(Direct3D.CubeMapFace.NegativeZ).SetVertex(idx, TexV(idx).X, TexV(idx).Y)
        Next

        ' create left plane
        V = DX9Tools.GeoHelpers.RotateVertexes(New Vector3(), New Vector3(0, DegreeToRadian(90), 0), V, False)
        For idx = 0 To V.Length - 1
            mobjVerts(Direct3D.CubeMapFace.NegativeX).SetVertex(idx, Vector3.Add(V(idx), mobjCenter))
            mobjVerts(Direct3D.CubeMapFace.NegativeX).SetVertex(idx, TexV(idx).X, TexV(idx).Y)
        Next

        ' create bottom plane
        V = DX9Tools.GeoHelpers.RotateVertexes(New Vector3(), New Vector3(0, DegreeToRadian(90), 0), V, False)
        V = DX9Tools.GeoHelpers.RotateVertexes(New Vector3(), New Vector3(DegreeToRadian(90), DegreeToRadian(90), 0), V, False)
        For idx = 0 To V.Length - 1
            mobjVerts(Direct3D.CubeMapFace.NegativeY).SetVertex(idx, Vector3.Add(V(idx), mobjCenter))
            mobjVerts(Direct3D.CubeMapFace.NegativeY).SetVertex(idx, TexV(idx).X, TexV(idx).Y)
        Next

        ' create top plane
        V = DX9Tools.GeoHelpers.RotateVertexes(New Vector3(), New Vector3(DegreeToRadian(180), DegreeToRadian(90), 0), V, False)
        For idx = 0 To V.Length - 1
            mobjVerts(Direct3D.CubeMapFace.PositiveY).SetVertex(idx, Vector3.Add(V(idx), mobjCenter))
            mobjVerts(Direct3D.CubeMapFace.PositiveY).SetVertex(idx, TexV(idx).X, TexV(idx).Y)
        Next

        ' set all vertex colors to white
        For idx = 0 To mobjVerts.Length - 1
            mobjVerts(idx).SetVertex(Color.White.ToArgb)
        Next
    End Sub

    Public Property Size() As Single
        Get
            Return msngSize
        End Get
        Set(ByVal Value As Single)
            msngSize = Value
            BuildBox()
        End Set
    End Property

    Public Property Center() As Vector3
        Get
            Return mobjCenter
        End Get
        Set(ByVal Value As Vector3)
            mobjCenter = Value
            BuildBox()
        End Set
    End Property

#Region "  Contructor overloads ... "
    Public Sub New()
        MyBase.new()
        Dim idx As Integer
        For idx = 0 To mobjVerts.Length - 1
            mobjVerts(idx) = New DX9Tools.PCTGeometry()
            mobjVerts(idx).PrimitiveType = DirectX.Direct3D.PrimitiveType.TriangleList
        Next
    End Sub

    Public Sub New(ByVal Size As Single)
        Me.new()
        msngSize = Size
        BuildBox()
    End Sub

    Public Sub New(ByVal Center As Vector3)
        Me.new()
        msngSize = Size
        mobjCenter = Center
        BuildBox()
    End Sub

    Public Sub New(ByVal Center As Vector3, ByVal Size As Single)
        Me.new()
        msngSize = Size
        mobjCenter = Center
        BuildBox()
    End Sub

    Public Sub New(ByVal Blending As DX9Tools.BaseTextureBlendingItem)
        Me.new()
        Dim idx As Integer
        For idx = 0 To mobjVerts.Length - 1
            mobjVerts(idx).Blending = Blending
        Next
    End Sub

    Public Sub New(ByVal Blending As DX9Tools.BaseTextureBlendingItem, ByVal Size As Single)
        Me.new()
        msngSize = Size
        Dim idx As Integer
        For idx = 0 To mobjVerts.Length - 1
            mobjVerts(idx).Blending = Blending
        Next
        BuildBox()
    End Sub

    Public Sub New(ByVal Blending As DX9Tools.BaseTextureBlendingItem, ByVal Center As Vector3)
        Me.new()
        msngSize = Size
        mobjCenter = Center
        Dim idx As Integer
        For idx = 0 To mobjVerts.Length - 1
            mobjVerts(idx).Blending = Blending
        Next
        BuildBox()
    End Sub

    Public Sub New(ByVal Blending As DX9Tools.BaseTextureBlendingItem, ByVal Center As Vector3, ByVal Size As Single)
        Me.new()
        msngSize = Size
        mobjCenter = Center
        Dim idx As Integer
        For idx = 0 To mobjVerts.Length - 1
            mobjVerts(idx).Blending = Blending
        Next

        BuildBox()
    End Sub
#End Region

    Public Property TextureIndex(ByVal Index As Direct3D.CubeMapFace) As Integer
        Get
            Return Me.TextureIndex(Index)
        End Get
        Set(ByVal Value As Integer)
            Me.TextureIndex(Index) = Value
        End Set
    End Property

    Public Property TextureIndex(ByVal Index As Integer) As Integer
        Get
            Return mintTextureIndex(Index)
        End Get
        Set(ByVal Value As Integer)
            mintTextureIndex(Index) = Value
              If TypeOf mobjVerts(Index).Blending Is DX9Tools.IndexedTextureBlendingItem Then
                DirectCast(mobjVerts(Index).Blending, DX9Tools.IndexedTextureBlendingItem).TextureIndex = Value
            End If
         End Set
    End Property

    Public Sub Dispose() Implements System.IDisposable.Dispose
        Dim idx As Integer
        For idx = 0 To mobjVerts.Length - 1
            mobjVerts(idx).Dispose()
            mobjVerts(idx) = Nothing
        Next
    End Sub

    Public ReadOnly Property Vertexes(ByVal Index As Integer) As DX9Tools.PCTGeometry
        Get
            Return mobjVerts(Index)
        End Get
    End Property

    Public ReadOnly Property Vertexes(ByVal Index As Direct3D.CubeMapFace) As DX9Tools.PCTGeometry
        Get
            Return Me.Vertexes(Index)
        End Get
    End Property
End Class