Public Module DrawingHelpers

#Region " DrawText overloads ... "
    Public Function DrawText(ByVal Text As String, ByVal X As Single, ByVal Y As Single) As Boolean
        If mblnBeginScene = False Then Return False
        Return DrawText(mstrCurrentFont, Text, X, Y)
    End Function

    Public Function DrawText(ByVal Name As String, ByVal Text As String, ByVal X As Single, ByVal Y As Single) As Boolean
        ' If mblnHasBeenInit = False Then Return False
        If mblnBeginScene = False Then Return False
        If Name Is Nothing Then Return False
        If mobjFonts.ContainsKey(Name) = False Then Return False
        Dim Fnt As Font
        Fnt = DirectCast(mobjFonts(Name), Font)
        If Fnt.D3DFont Is Nothing Then Return False
        Dim R As Rectangle = New Rectangle(CInt(X), CInt(Y), 1, 1)
        Fnt.D3DFont.DrawText(Text, R, mobjCurrentFontTextFormat, mobjDrawColor)
        'R = Fnt.D3DFont.MeasureString(Nothing, Text, mobjCurrentFontTextFormat, mobjDrawColor)
        'R.X = CInt(X)
        'R.Y = CInt(Y)
        'Fnt.D3DFont.DrawText(Nothing, Text, R, mobjCurrentFontTextFormat, mobjDrawColor)
        Return True
    End Function

    Public Function DrawText(ByVal Text As String, ByVal Boundary As Rectangle) As Boolean
        Return DrawText(DEFAULTFONT, Text, Boundary)
    End Function

    Public Function DrawText(ByVal Name As String, ByVal Text As String, ByVal Boundary As Rectangle) As Boolean
        '  If mblnHasBeenInit = False Then Return False
        If mblnBeginScene = False Then Return False
        If Name Is Nothing Then Return False
        If mobjFonts.ContainsKey(Name) = False Then Return False
        Dim Fnt As Font
        Fnt = DirectCast(mobjFonts(Name), Font)
        If Fnt.D3DFont Is Nothing Then Return False
        Fnt.D3DFont.DrawText(Text, Boundary, mobjCurrentFontTextFormat, mobjDrawColor)
        Return True
    End Function
#End Region

    Public Function DrawMesh(ByVal Name As String) As Boolean
        If mblnBeginScene = False Then Return False
        ' If Name Is Nothing Then Return False
        If mobjMeshes.ContainsKey(Name) = False Then Return False

        Dim MX As Mesh = DirectCast(mobjMeshes(Name), Mesh)
        With mobjDevice
            .VertexFormat = MX.mobjMesh.VertexBuffer.Description.VertexFormat

            Dim idx As Integer
            For idx = 0 To MX.mobjMesh.AttributeTable.Length - 1
                Try
                    ApplyMaterial(MX.mobjMaterials(idx))
                Catch ex As Exception
                    SpecifyError("DXTools.DrawingHelpers.DrawMesh", ex.Message)
                    Return False
                End Try
                Try
                    ApplyTexture(0, MX.mobjTextures(idx))
                Catch ex As Exception
                    SpecifyError("DXTools.DrawingHelpers.DrawMesh", ex.Message)
                    Return False
                End Try
                MX.mobjMesh.DrawSubset(idx)
            Next
        End With

        Return True
    End Function

    Public Function DrawVertexBuffer(ByVal Name As String) As Boolean
        If mblnBeginScene = False Then Return False
        Dim VB As VertexBuffer = Nothing
        If Name Is Nothing Then Return False
        If mobjVertexBuffers.ContainsKey(Name) = False Then Return False
        VB = DirectCast(mobjVertexBuffers(Name), VertexBuffer)
        If VB.Buffer Is Nothing Then Return False
        Try
            With mobjDevice
                .VertexFormat = VB.Buffer.Description.VertexFormat
                .SetStreamSource(VB.StreamIndex, VB.Buffer, VB.StreamOffset)
                .DrawPrimitives(VB.PrimitiveType, VB.StartVertex, VB.PrimitiveCount)
            End With
        Catch ex As Exception
            Return False
        End Try

        Return True
    End Function

    Public Function DrawVertexBuffer(ByVal Name As String, ByVal Texture As String, Optional ByVal Stage As Integer = 0) As Boolean
        If mblnBeginScene = False Then Return False
        If mobjVertexBuffers.ContainsKey(Name) = False Then Return False
        If Texture = NOTEXTURE Then
            mobjDevice.SetTexture(Stage, Nothing)
            Return DrawVertexBuffer(Name)
        End If

        If mobjTextures.ContainsKey(Texture) = False Then Return False

        mobjDevice.SetTexture(Stage, DirectCast(mobjTextures(Texture), Direct3D.Texture))
        Return DrawVertexBuffer(Name)
    End Function

    Public Function DrawRectangle(ByVal X1 As Single, ByVal Y1 As Single, ByVal X2 As Single, ByVal Y2 As Single, ByVal Color As Color) As Boolean
        If mblnBeginLines = False Then Return False
        If DrawLine(X1, Y1, X2, Y1, Color) = False Then Return False
        If DrawLine(X1, Y2, X2, Y2, Color) = False Then Return False
        If DrawLine(X1, Y1, X1, Y2, Color) = False Then Return False
        If DrawLine(X2, Y1, X2, Y2, Color) = False Then Return False
        Return True
    End Function

    Public Function DrawRectangle(ByVal X1 As Single, ByVal Y1 As Single, ByVal X2 As Single, ByVal Y2 As Single) As Boolean
        If mblnBeginLines = False Then Return False
        Return DrawRectangle(X1, Y1, X2, Y2, mobjDrawColor)
    End Function

    Public Function DrawLine(ByVal X1 As Single, ByVal Y1 As Single, ByVal X2 As Single, ByVal Y2 As Single, ByVal Color As Color) As Boolean
        If mblnBeginLines = False Then Return False
        Static Pts() As Vector2 = {New Vector2(), New Vector2()}
        Pts(0).X = X1
        Pts(0).Y = Y1
        Pts(1).X = X2
        Pts(1).Y = Y2
        mobjLine.Draw(Pts, Color)
        Return True
    End Function

    Public Function DrawLine(ByVal X1 As Single, ByVal Y1 As Single, ByVal X2 As Single, ByVal Y2 As Single) As Boolean
        If mblnBeginLines = False Then Return False
        Return DrawLine(X1, Y1, X2, Y2, mobjDrawColor)
    End Function

    Public Function DrawLine(ByVal Values() As Vector2, ByVal Color As Color) As Boolean
        If mblnBeginLines = False Then Return False
        mobjLine.Draw(Values, Color)
        Return True
    End Function

    Public Function DrawLine(ByVal Values() As Vector2) As Boolean
        If mblnBeginLines = False Then Return False
        Return DrawLine(Values, mobjDrawColor)
    End Function

    Public Function BeginLineDrawing() As Boolean
        ' If mblnHasBeenInit = False Then Return False
        If mblnBeginScene = False Then Return False
        If mblnBeginLines Then Return False
        If mblnBeginSprites Then Return False
        mobjLine.Begin()
        mblnBeginLines = True
        Return True
    End Function

    Public Function EndLineDrawing() As Boolean
        If mblnBeginLines = False Then Return False
        mobjLine.End()
        mblnBeginLines = False
        Return True
    End Function

    Public Function BeginSpriteRender() As Boolean
        '  If mblnHasBeenInit = False Then Return False
        If mblnBeginScene = False Then Return False
        If mblnBeginSprites Then Return False
        If mblnBeginLines Then Return False
        mobjSprite.Begin() '( Direct3D.SpriteFlags.AlphaBlend Or Direct3D.SpriteFlags.SortDepthBackToFront)
        mblnBeginSprites = True
        Return True
    End Function

    Public Function DrawSprite(ByVal Texture As String, ByVal Source As Rectangle, ByVal X As Single, ByVal Y As Single) As Boolean
        If mblnBeginSprites = False Then Return False
        If mobjTextures.ContainsKey(Texture) = False Then Return False
        Static Scale As New Vector2(1, 1)
        Static EmptyVec As New Vector2()
        Static Pos As Vector2
        Pos.X = X
        Pos.Y = Y
        mobjSprite.Draw(DirectCast(mobjTextures(Texture), Direct3D.Texture), Source, Scale, EmptyVec, 0, Pos, Color.White)
        Return True
    End Function

    Public Function DrawSprite(ByVal Texture As String, ByVal Source As Rectangle, ByVal X As Single, ByVal Y As Single, ByVal ScaleX As Single, ByVal ScaleY As Single) As Boolean
        If mblnBeginSprites = False Then Return False
        If mobjTextures.ContainsKey(Texture) = False Then Return False
        Static Scale As New Vector2(1, 1)
        Static EmptyVec As New Vector2()
        Static Pos As Vector2
        Scale.X = ScaleX
        Scale.Y = ScaleY
        Pos.X = X
        Pos.Y = Y
        mobjSprite.Draw(DirectCast(mobjTextures(Texture), Direct3D.Texture), Source, Scale, EmptyVec, 0, Pos, Color.White)
        Return True
    End Function

    Public Function DrawSprite(ByVal Texture As String, ByVal Source As RectangleF, ByVal X As Single, ByVal Y As Single) As Boolean
        If mblnBeginSprites = False Then Return False
        If mobjTextures.ContainsKey(Texture) = False Then Return False
        Static Scale As New Vector2(1, 1)
        Static EmptyVec As New Vector2()
        Static Pos As Vector2
        Static Rect As Rectangle
        Pos.X = X
        Pos.Y = Y
        'Rect = Rectangle.FromLTRB(CInt(Source.Left), CInt(Source.Top), CInt(Source.Right), CInt(Source.Bottom))
        Rect.X = CInt(Source.Left)
        Rect.Y = CInt(Source.Top)
        Rect.Width = CInt(Source.Right)
        Rect.Height = CInt(Source.Bottom)

        mobjSprite.Draw(DirectCast(mobjTextures(Texture), Direct3D.Texture), Rect, Scale, EmptyVec, 0, Pos, Color.White)
        Return True
    End Function

    Public Function DrawSprite(ByVal Texture As String, ByVal Source As RectangleF, ByVal X As Single, ByVal Y As Single, ByVal RotationX As Single, ByVal RotationY As Single, ByVal Rotation As Single) As Boolean
        If mblnBeginSprites = False Then Return False
        If mobjTextures.ContainsKey(Texture) = False Then Return False
        Static Scale As New Vector2(1, 1)
        ' Static EmptyVec As New Vector2()
        Static Pos As Vector2
        Static Rect As Rectangle
        Pos.X = X
        Pos.Y = Y
        'Rect = Rectangle.FromLTRB(CInt(Source.Left), CInt(Source.Top), CInt(Source.Right), CInt(Source.Bottom))
        Rect.X = CInt(Source.Left)
        Rect.Y = CInt(Source.Top)
        Rect.Width = CInt(Source.Right)
        Rect.Height = CInt(Source.Bottom)

        mobjSprite.Draw(DirectCast(mobjTextures(Texture), Direct3D.Texture), Rect, Scale, New Vector2(RotationX, RotationY), Rotation, Pos, Color.White)
        Return True
    End Function

    Public Function DrawSprite(ByVal Texture As String, ByVal Source As Rectangle, ByVal X As Single, ByVal Y As Single, ByVal RotationX As Single, ByVal RotationY As Single, ByVal Rotation As Single, ByVal Color As Color) As Boolean
        If mblnBeginSprites = False Then Return False
        If mobjTextures.ContainsKey(Texture) = False Then Return False
        Static Scale As New Vector2(1, 1)
        ' Static EmptyVec As New Vector2()
        Static Pos As Vector2
        '  Static Rect As Rectangle
        Pos.X = X
        Pos.Y = Y
        'Rect = Rectangle.FromLTRB(CInt(Source.Left), CInt(Source.Top), CInt(Source.Right), CInt(Source.Bottom))
        'Rect.X = Source.Left
        'Rect.Y = Source.Top
        'Rect.Width = Source.Right
        'Rect.Height = Source.Bottom

        mobjSprite.Draw(DirectCast(mobjTextures(Texture), Direct3D.Texture), Source, Scale, New Vector2(RotationX, RotationY), Rotation, Pos, Color)
        Return True
    End Function

    Public Function DrawSprite(ByVal Sprite As Sprite2D) As Boolean
        If mblnBeginSprites = False Then Return False
        ' If Sprite.Visible = False Then Return False
        If mobjTextures.ContainsKey(Sprite.Texture) = False Then Return False
        'Dim R As Rectangle
        ' Dim S As Size
        ' S.Width = Sprite.Scale.Width * Sprite.Rectangle.Size.Width
        ' S.Height = Sprite.Scale.Height * Sprite.Rectangle.Size.Height
        ' R = New Rectangle(New Point(0, 0), S)
        Static Pos As Vector2
        Static Cen As Vector2
        Static Scale As Vector2

        Pos.X = Sprite.X
        Pos.Y = Sprite.Y
        Scale.X = Sprite.ScaleX
        Scale.Y = Sprite.ScaleY
        Cen.X = Sprite.CenterX * Scale.X
        Cen.Y = Sprite.CenterY * Scale.Y
        Pos.X -= Cen.X
        Pos.Y -= Cen.Y
        mobjSprite.Draw(DirectCast(mobjTextures(Sprite.Texture), Direct3D.Texture), Sprite.Rectangle, Scale, Cen, Sprite.Rotation, Pos, Sprite.Color)
        Return True
    End Function

    'Public Function DrawSprite(ByVal Sprite As Sprite3D) As Boolean
    '    If mblnBeginSprites = False Then Return False
    '    If Sprite.Visible = False Then Return False
    '    If mobjTextures.ContainsKey(Sprite.Texture) = False Then Return False
    '    mobjSprite.Draw(mobjTextures(Sprite.Texture), Sprite.Rectangle, Sprite.Center, Sprite.Position, Sprite.Color)
    '    Return True
    'End Function

    Public Function EndSpriteRender() As Boolean
        If mblnBeginSprites = False Then Return False
        mobjSprite.End()
        mblnBeginSprites = False
        Return True
    End Function
End Module
