﻿Public NotInheritable Class Font
    Implements IDisposable

    Friend mstrName As String
    Public D3DFont As Direct3D.Font

    ' Friend mobjFontsCollectionReference As Collections.Generic.Dictionary(Of String, Font)

    Public Property Name() As String
        Get
            Return mstrName
        End Get
        Set(ByVal value As String)
            If value Is Nothing Then Throw New ArgumentNullException("Value")
            If value.Trim = String.Empty Then Throw New ArgumentNullException("Value")
            If mobjFonts Is Nothing = False Then
                If mstrName Is Nothing = False Then
                    If mobjFonts.ContainsKey(mstrName) Then mobjFonts.Remove(mstrName)
                End If
            End If
            mstrName = value
            If mobjFonts Is Nothing = False Then
                mobjFonts.Add(mstrName, Me)
            End If
        End Set
    End Property

#Region " IDisposable Support "
    Private disposed As Boolean = False

    ' IDisposable
    Private Overloads Sub Dispose(ByVal disposing As Boolean)
        If Not Me.disposed Then
            If disposing Then
                If Me.D3DFont Is Nothing = False Then Me.D3DFont.Dispose()
                Me.D3DFont = Nothing
            End If

            ' TODO: put code to free unmanaged resources here
        End If
        Me.disposed = True
    End Sub

    ' This code added by Visual Basic to correctly implement the disposable pattern.
    Public Overloads Sub Dispose() Implements IDisposable.Dispose
        ' Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub

    Protected Overrides Sub Finalize()
        ' Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
        Dispose(False)
        MyBase.Finalize()
    End Sub
#End Region

End Class

Public Module Fonts

    Friend mobjFonts As New Collections.Hashtable()  ' .Generic.Dictionary(Of String, Font)
    Friend mstrCurrentFont As String = String.Empty
    Friend mobjCurrentFontTextFormat As Direct3D.DrawTextFormat = Direct3D.DrawTextFormat.Top Or Direct3D.DrawTextFormat.Left Or Direct3D.DrawTextFormat.NoClip
    Friend mobjCurrentFontBoundary As Rectangle
    Public Const DEFAULTFONT As String = "{AD5C68F3-F72C-499f-AA28-CA4E9F2C8C3B}"


    Public Function ChangeFontName(ByRef Name As String, ByVal Value As String) As Boolean
        If mobjFonts.ContainsKey(Name) Then Return False
        If mobjFonts.ContainsKey(Value) Then Return False
        Dim Item As Font
        Item = DirectCast(mobjFonts(Name), Font)
        mobjFonts.Remove(Name)
        mobjFonts.Add(Value, Item)

        Return True
    End Function

    Public Function GetFontNames(ByRef Names() As String) As Boolean
        Erase Names
        ReDim Names(mobjFonts.Count - 1)
        mobjFonts.Keys.CopyTo(Names, 0)

        Return True
    End Function

    Public Function CreateFont(ByVal Name As String, ByVal Font As Drawing.Font) As Boolean
        If mobjFonts.ContainsKey(Name) = False Then Return False
        Dim F As New Font()

        Try
            F.D3DFont = New Direct3D.Font(mobjDevice, Font)
        Catch ex As Exception
            Return False
        End Try

        Return AddFont(Name, F)
    End Function

    Public Function GetFontBoundary() As Rectangle
        Return mobjCurrentFontBoundary
    End Function

    Public Sub SetFontBoundary(ByVal Value As Rectangle)
        mobjCurrentFontBoundary = Value
    End Sub

    Public Function GetFontFormatting() As Direct3D.DrawTextFormat
        Return mobjCurrentFontTextFormat
    End Function

    Public Sub SetFontFormatting(ByVal Value As Direct3D.DrawTextFormat)
        mobjCurrentFontTextFormat = Value
    End Sub

    Public Function GetNumberOfFonts() As Integer
        Return mobjFonts.Count
    End Function

    Public Sub RemoveAllFonts(Optional ByVal CallDispose As Boolean = True)
        ' Dim Fnt As Font = Nothing
        ' If mobjFonts.ContainsKey(DEFAULTFONT) Then Fnt = mobjFonts(DEFAULTFONT)

        If CallDispose Then
            With mobjFonts.GetEnumerator
                .Reset()
                While .MoveNext
                    ' If .Current.Value.mstrName <> DEFAULTFONT Then
                    DirectCast(.Value, Font).Dispose()
                    ' End If
                End While
            End With
        End If

        mobjFonts.Clear()
        ' If Fnt IsNot Nothing And mblnHasBeenInit Then mobjFonts.Add(DEFAULTFONT, Fnt)
    End Sub

    Public Function GetCurrentFont() As Font
        If mobjFonts.ContainsKey(mstrCurrentFont) = False Then Return Nothing
        Return DirectCast(mobjFonts(mstrCurrentFont), Font)
    End Function

    Public Function GetCurrentFontName() As String
        Return mstrCurrentFont
    End Function

    Public Function SetCurrentFont(ByVal Name As String) As Boolean
        If Name Is Nothing Then Return False
        If mobjFonts.ContainsKey(Name) = False Then Return False
        mstrCurrentFont = Name
        Return True
    End Function

    Public Function RemoveFont(ByVal Name As String) As Boolean
        If Name Is Nothing Then Return False
        If mobjFonts.ContainsKey(Name) = False Then Return False
        ' mobjFonts(Name).mobjFontsCollectionReference = Nothing
        mobjFonts.Remove(Name)
        Return True
    End Function

    Public Function AddFont(ByVal Name As String, ByVal Value As Font) As Boolean
        If Name Is Nothing Then Return False
        If mobjFonts.ContainsKey(Name) Then Return False
        '  Value.mobjFontsCollectionReference = mobjFonts
        Value.mstrName = Name
        mobjFonts.Add(Name, Value)
        Return True
    End Function

    Public Function SetFont(ByVal Name As String, ByVal Value As Font) As Boolean
        If Name Is Nothing Then Return False
        If mobjFonts.ContainsKey(Name) = False Then Return False
        mobjFonts(Name) = Value
        Return True
    End Function

    Public Function GetFont(ByVal Name As String, ByRef Value As Font) As Boolean
        If Name Is Nothing Then Return False
        If mobjFonts.ContainsKey(Name) = False Then Return False
        Value = DirectCast(mobjFonts(Name), Font)
        Return True
    End Function
End Module
