'Created by: X
'Copyright: Created by: X
'E-Mail: mailto:createdbyx@yahoo.com
'Web: http://www.createdbyx.com/
'Date: November 25, 2003
'=============================================

''' <summary>
''' The Conversion module contains a number of ready made conversion functions to assist the developer.
''' </summary>
Public Class Conversion
    ''' <summary>
    ''' Converts a Rectangle type to a RectangleF type.
    ''' </summary>
    ''' <param name="R">The source rectangle to be converted.</param>
    ''' <returns>Returns the converted rectangle.</returns>
    Public Shared Function RectangleToRectangleF(ByVal R As Rectangle) As RectangleF
        Return New RectangleF(R.Left, R.Top, R.Width, R.Height)
    End Function

    ''' <summary>
    ''' Converts a RectangleF type to a Rectangle type.
    ''' </summary>
    ''' <param name="R">The source rectangle to be converted.</param>
    ''' <returns>Returns the converted rectangle.</returns>
    Public Shared Function RectangleFToRectangle(ByVal R As RectangleF) As Rectangle
        Return New Rectangle(CInt(R.Left), CInt(R.Top), CInt(R.Width), CInt(R.Height))
    End Function

    ''' <summary>
    ''' The Vector2ToPoint is used to convert a Vector3 structure to a Point structure.
    ''' </summary>
    ''' <param name="V">The vector that will be converted and returned as a point.</param>
    ''' <returns>Returns the converted vector as a point.</returns>
    Public Shared Function Vector2ToPoint(ByVal V As Vector2) As Point
        Return New Point(CInt(V.X), CInt(V.Y))
    End Function

    ''' <summary>
    ''' The Vector2ToPointF is used to convert a Vector3 structure to a PointF structure.
    ''' </summary>
    ''' <param name="V">The vector that will be converted and returned as a PointF.</param>
    ''' <returns>Returns the converted vector as a PointF.</returns>
    Public Shared Function Vector2ToPointF(ByVal V As Vector2) As PointF
        Return New PointF(V.X, V.Y)
    End Function

    ''' <summary>
    ''' The PointToVector2 is used to convert a Point structure to a Vector3 structure.
    ''' </summary>
    ''' <param name="P">The point that will be converted and returned as a vector2.</param>
    ''' <returns>Returns the converted point as a vector.</returns>
    Public Shared Function PointToVector2(ByVal P As Point) As Vector2
        Return New Vector2(P.X, P.Y)
    End Function

    ''' <summary>
    ''' The PointFToVector2 is used to convert a PointF structure to a Vector3 structure.
    ''' </summary>
    ''' <param name="P">The point that will be converted and returned as a vector2.</param>
    ''' <returns>Returns the converted point as a vector.</returns>
    Public Shared Function PointFToVector2(ByVal P As PointF) As Vector2
        Return New Vector2(P.X, P.Y)
    End Function
End Class