'Created by: X
'Copyright: Created by: X
'E-Mail: mailto:createdbyx@yahoo.com
'Web: http://www.createdbyx.com/
'Date: August 6, 2003
'=============================================

Imports Microsoft.DirectX.Direct3D.Geometry

''' <summary>
''' Contains a methods to assist in geometry related functionality.
''' </summary>
Public Module GeoHelpers
    ''' <summary>
    ''' Rotates an array of vector3 types.
    ''' </summary>
    ''' <param name="Axis">The axis upon which the vectors will be rotated around.</param>
    ''' <param name="Rotation">The rotation in radians.</param>
    ''' <param name="Vertexes">The array of vertexes to be rotated.</param>
    ''' <param name="YawPitchRoll">If true the XYZ values in the Rotation parameter will be treated as Yaw, Pitch, and Roll respectivly.</param>
    ''' <param name="Order">If the YawPitchRoll parameter is false this parameter allows yopu to specify in what order the vectors </param>
    ''' <returns>Returns an array of rotated vectors.</returns>
    Public Function RotateVertexes(ByVal Axis As Vector3, ByVal Rotation As Vector3, ByVal Vertexes() As Vector3, _
    Optional ByVal YawPitchRoll As Boolean = True) As Vector3()
        Dim idx As Integer
        Dim V() As Vector3

        ReDim V(Vertexes.Length - 1)
        For idx = 0 To Vertexes.Length - 1
            V(idx) = MathHelpers.RotateAroundAxis(Axis, Vertexes(idx), Rotation, YawPitchRoll)
        Next

        Return V
    End Function

#Region " Shape creation ... "
    Public Function CreateDonut(ByVal Rect As RectangleF, ByVal InnerRadius As Single, ByVal OuterRadius As Single, _
     ByVal StartDegree As Single, ByVal DegreePerRing As Single, ByVal Rings As Integer) As Vector3()
        Dim idx, index As Integer
        Dim Temp(), Inner(), Verts() As Vector3
        Dim Rotation As Single
        Dim Displacement As Vector2

        If Rings < 2 Then Throw New ArgumentOutOfRangeException("Rings", Rings, "Rings parameter must be 2 or heigher.")

        ReDim Verts((Rings * 2) * 3)
        Dim InnerV, OutterV As Vector3
        InnerV = New Vector3(0, InnerRadius, 0)
        OutterV = New Vector3(0, OuterRadius, 0)

        Rotation = StartDegree
        For idx = 0 To Rings - 1
            Dim InX, OutX As Vector3
            InX = MathHelpers.RotateAroundAxis(New Vector3(), InnerV, New Vector3(DegreeToRadian(0), DegreeToRadian(0), DegreeToRadian(Rotation)))
            OutX = MathHelpers.RotateAroundAxis(New Vector3(), OutterV, New Vector3(DegreeToRadian(0), DegreeToRadian(0), DegreeToRadian(Rotation)))
            Dim NextInX, NextOutX As Vector3
            NextInX = MathHelpers.RotateAroundAxis(New Vector3(), InnerV, New Vector3(DegreeToRadian(0), DegreeToRadian(0), DegreeToRadian(Rotation + DegreePerRing)))
            NextOutX = MathHelpers.RotateAroundAxis(New Vector3(), OutterV, New Vector3(DegreeToRadian(0), DegreeToRadian(0), DegreeToRadian(Rotation + DegreePerRing)))

            Verts(index) = InX
            Verts(index + 1) = OutX
            Verts(index + 2) = NextInX
            Verts(index + 3) = NextInX
            Verts(index + 4) = OutX
            Verts(index + 5) = NextOutX
            Rotation += DegreePerRing
            index += 6
        Next

        Return Verts
    End Function

    ''' <summary>
    ''' Creates a array of Vecotr3 type representing a box made out of a TriangleList.
    ''' </summary>
    ''' <param name="Center">Specifies the center of the box.</param>
    ''' <param name="Size">Specifies the diemention of the box.</param>
    ''' <param name="Rows">Specifies the number of rows the box will be created with.</param>
    ''' <param name="Columns">Specifies the number of columns the box will be created with.</param>
    ''' <param name="Stacks">Specifies the number of stacks the box will be created with.</param>
    ''' <returns>Returns an array of Vecotr3 types representing a box made out of a TriangleList.</returns>
    Public Function CreateBox(ByVal Center As Vector3, ByVal Size As Vector3, ByVal Rows As Integer, ByVal Columns As Integer, ByVal Stacks As Integer) As Vector3()
        Dim Temp(), Verts() As Vector3
        Dim R As RectangleF
        Dim HalfSize As Vector3
        Dim idx, index As Integer

        HalfSize = New Vector3(Size.X / 2, Size.Y / 2, Size.Z / 2)

        ' create front
        R = New RectangleF(-HalfSize.X, -HalfSize.Y, Size.X, Size.Y)
        Temp = CreatePlane(R, Rows, Columns, -HalfSize.Z)
        ReDim Verts((Temp.Length * 1) - 1)
        For idx = 0 To Temp.Length - 1
            Verts(index) = MathHelpers.RotateAroundAxis(New Vector3(), Temp(idx), New Vector3(DegreeToRadian(0), DegreeToRadian(0), DegreeToRadian(0)))
            Verts(index).Add(Center)
            index += 1
        Next

        'create back
        ReDim Preserve Verts((Verts.Length + Temp.Length) - 1)
        For idx = 0 To Temp.Length - 1
            Verts(index) = MathHelpers.RotateAroundAxis(New Vector3(), Temp(idx), New Vector3(DegreeToRadian(180), DegreeToRadian(0), DegreeToRadian(0)))
            Verts(index).Add(Center)
            index += 1
        Next

        'create left
        R = New RectangleF(-HalfSize.Z, -HalfSize.Y, Size.Z, Size.Y)
        Temp = CreatePlane(R, Rows, Stacks, -HalfSize.X)
        ReDim Preserve Verts((Verts.Length + Temp.Length) - 1)
        For idx = 0 To Temp.Length - 1
            Verts(index) = MathHelpers.RotateAroundAxis(New Vector3(), Temp(idx), New Vector3(DegreeToRadian(90), DegreeToRadian(0), DegreeToRadian(0)))
            Verts(index).Add(Center)
            index += 1
        Next

        'create right
        ReDim Preserve Verts((Verts.Length + Temp.Length) - 1)
        For idx = 0 To Temp.Length - 1
            Verts(index) = MathHelpers.RotateAroundAxis(New Vector3(), Temp(idx), New Vector3(DegreeToRadian(270), DegreeToRadian(0), DegreeToRadian(0)))
            Verts(index).Add(Center)
            index += 1
        Next

        'create top
        R = New RectangleF(-HalfSize.X, -HalfSize.Z, Size.X, Size.Z)
        Temp = CreatePlane(R, Stacks, Columns, -HalfSize.Y)
        ReDim Preserve Verts((Verts.Length + Temp.Length) - 1)
        For idx = 0 To Temp.Length - 1
            Verts(index) = MathHelpers.RotateAroundAxis(New Vector3(), Temp(idx), New Vector3(DegreeToRadian(0), DegreeToRadian(90), DegreeToRadian(0)))
            Verts(index).Add(Center)
            index += 1
        Next

        'create bottom
        ReDim Preserve Verts((Verts.Length + Temp.Length) - 1)
        For idx = 0 To Temp.Length - 1
            Verts(index) = MathHelpers.RotateAroundAxis(New Vector3(), Temp(idx), New Vector3(DegreeToRadian(0), DegreeToRadian(270), DegreeToRadian(0)))
            Verts(index).Add(Center)
            index += 1
        Next

        Return Verts
    End Function

    ''' <summary>
    ''' Creates a array of Vecotr3 type representing a plane made out of a TriangleList.
    ''' </summary>
    ''' <param name="Rect">Specifies the dimentions of the plane.</param>
    ''' <param name="Rows">Specifies the number of rows the plane will be created with.</param>
    ''' <param name="Columns">Specifies the number of columns the plane will be created with.</param>
    ''' <param name="Depth">Specifies teh Z depth that the plane will be created at.</param>
    ''' <returns>Returns an array of Vecotr3 types representing a plane made out of a TriangleList.</returns>
    Public Function CreatePlane(ByVal Rect As Rectangle, ByVal Rows As Integer, ByVal Columns As Integer, Optional ByVal Depth As Single = 0) As Vector3()
        Return CreatePlane(New RectangleF(Rect.Left, Rect.Top, Rect.Width, Rect.Height), Rows, Columns, Depth)
    End Function

    ''' <summary>
    ''' Creates a array of Vecotr3 type representing a plane made out of a TriangleList.
    ''' </summary>
    ''' <param name="Rect">Specifies the dimentions of the plane.</param>
    ''' <param name="Rows">Specifies the number of rows the plane will be created with.</param>
    ''' <param name="Columns">Specifies the number of columns the plane will be created with.</param>
    ''' <param name="Depth">Specifies teh Z depth that the plane will be created at.</param>
    ''' <returns>Returns an array of Vecotr3 types representing a plane made out of a TriangleList.</returns>
    Public Function CreatePlane(ByVal Rect As RectangleF, ByVal Rows As Integer, ByVal Columns As Integer, Optional ByVal Depth As Single = 0) As Vector3()
        Dim Verts() As Vector3
        Dim Index, idx, idy As Integer
        Dim HalfSize As SizeF = Rect.Size
        Dim CellSize As SizeF

        If Rows < 1 Then Throw New ArgumentOutOfRangeException("Rows", Rows, "Rows parameter must be 1 or heigher.")
        If Columns < 1 Then Throw New ArgumentOutOfRangeException("Columns", Columns, "Columns parameter must be 1 or heigher.")

        CellSize = New SizeF(Rect.Width / Columns, Rect.Height / Rows)
        ReDim Verts((((Rows * Columns) * 2) * 3) - 1)
        For idy = 0 To Rows - 1
            For idx = 0 To Columns - 1
                Verts(Index + 0) = New Vector3(idx * CellSize.Width, idy * CellSize.Height, Depth)
                Verts(Index + 1) = New Vector3(idx * CellSize.Width, (idy * CellSize.Height) + CellSize.Height, Depth)
                Verts(Index + 2) = New Vector3((idx * CellSize.Width) + CellSize.Width, (idy * CellSize.Height) + CellSize.Height, Depth)

                Verts(Index + 3) = New Vector3((idx * CellSize.Width) + CellSize.Width, (idy * CellSize.Height) + CellSize.Height, Depth)
                Verts(Index + 4) = New Vector3((idx * CellSize.Width) + CellSize.Width, idy * CellSize.Height, Depth)
                Verts(Index + 5) = New Vector3(idx * CellSize.Width, idy * CellSize.Height, Depth)

                Dim idz As Integer
                For idz = Index To Index + 5
                    Verts(idz).Add(New Vector3(Rect.Left, Rect.Top, 0))
                Next

                Index += 6
            Next
        Next

        Return Verts
    End Function
#End Region


    '#Region " BuildVBFromGeometry overloads  ... "
    '    ''' <summary>
    '    ''' Builds a VertexBuffer object from a PCTGeometry object.
    '    ''' </summary>
    '    ''' <param name="Dev">A reference to a Direct3D.Device object.</param>
    '    ''' <param name="G">A reference to a PCTGeometry object.</param>
    '    ''' <returns>Returns a reference to a VertexBuffer object.</returns>
    '    ''' <remarks>The vertex buffer will be created with the DirectX.Direct3D.Pool.Managed flag.</remarks>
    '    Public Function BuildVBFromGeometry(ByVal Dev As Direct3D.Device, ByVal G As PCTGeometry) As Direct3D.VertexBuffer
    '        Return BuildVBFromGeometry(Dev, G, DirectX.Direct3D.Pool.Managed)
    '    End Function

    '    ''' <summary>
    '    ''' Builds a VertexBuffer object from a PCTGeometry object.
    '    ''' </summary>
    '    ''' <param name="Dev">A reference to a Direct3D.Device object.</param>
    '    ''' <param name="G">A reference to a PCTGeometry object.</param>
    '    ''' <param name="Pool">Member of the Pool enumerated type that describes a valid memory class in which to place the resource.</param>
    '    ''' <returns>Returns a reference to a VertexBuffer object.</returns>
    '    Public Function BuildVBFromGeometry(ByVal Dev As Direct3D.Device, ByVal G As Interfaces.IGeometry, ByVal Pool As Direct3D.Pool) As Direct3D.VertexBuffer
    '        Dim VB As Direct3D.VertexBuffer
    '        'Dim verts() As Direct3D.CustomVertex.PositionColoredTextured

    '        ' create ring vertexbuffer 
    '        VB = New Direct3D.VertexBuffer(GetType(Direct3D.CustomVertex.PositionColoredTextured), G.VertexCount, _
    '                                       Dev, 0, G.Format, DirectX.Direct3D.Pool.Managed)
    '        VB.SetData(G.GetVertexes, 0, Direct3D.LockFlags.None)
    '        '' get vert array
    '        'verts = CType(VB.Lock(0, DirectX.Direct3D.LockFlags.None), Direct3D.CustomVertex.PositionColoredTextured())

    '        '' transfer vert data
    '        'Array.Copy(G.Vertexes, verts, G.VertexCount)

    '        '' unlock
    '        'VB.Unlock()

    '        ' return vertex buffer
    '        Return VB
    '    End Function
    '#End Region

#Region " LoadMesh overloads ... "
    ''' <summary>
    ''' Loads a mesh from a file.
    ''' </summary>
    ''' <param name="Filename">The mesh file to load.</param>
    ''' <param name="Dev">A reference to a Direct3D.Device type.</param>
    ''' <param name="Textures">A reference to a DX9Tools.SimpleTextureCollection type that any textures will be added to.</param>
    ''' <param name="MeshFlags">Defines flags used to specify creation options for a mesh.</param>
    ''' <param name="TextureCallback">Provides a reference to a LoadTextureFromFile delagate. If this is NOT 'nothing' then the LoadMesh method will rely on it to load the texture.</param>
    ''' <param name="TexturePath">Specifies the path to the location of the images that will be loaded.</param>
    ''' <returns>Returns a reference to a new MeshEx type.</returns>
    Public Function LoadMesh(ByVal Name As String, ByVal Stream As io.stream, _
                            Optional ByVal MeshFlags As Direct3D.MeshFlags = Direct3D.MeshFlags.Managed) As Boolean
        'If mblnHasBeenInit = False Then Return False
        'If mobjMeshes.ContainsKey(Name) Then Return False

        'Dim M, Mats() As Direct3D.ExtendedMaterial
        'Dim Mesh As Mesh

        'Mesh = New Mesh()
        'Mesh.Mesh = Direct3D.Mesh.FromStream(Stream, MeshFlags, mobjDevice, Mats)
        ''redim mesh.mobjMaterials (mats.Length-1)
        'Dim idx As Integer = 0
        'For Each M In Mats
        '    Dim MName As String
        '    MName = Name & "_Material" & idx.ToString
        '    AddMaterial(MName, M.Material3D)
        '    Mesh.mobjMaterials.Add(MName)

        '    If IsNothing(M.TextureFilename) = False Or M.TextureFilename.Length <> 0 Then
        '        Dim Tex As Direct3D.Texture
        '        Dim tName As String = NOTEXTURE
        '        gobjEvents.DoLoadMesh(Name, M.TextureFilename, Tex, tname)
        '        ' If Tex Is Nothing = False Then
        '        Mesh.mobjTextures.Add(tName)
        '        ' End  If
        '    End If

        '    idx += 1
        'Next
        'AddMesh(Name, Mesh)
        '' SetMeshMaterials(Mesh, Dev, Mats, Textures, TextureCallback, TexturePath)

        'Return True
        Dim Mesh As Direct3D.Mesh
        Dim Fmt As Direct3D.VertexFormats
        Dim Pos As Long
        Pos = Stream.Position
        ' Mesh = New Mesh()
        Mesh = Direct3D.Mesh.FromStream(Stream, MeshFlags, mobjDevice)
        Fmt = Mesh.VertexFormat
        Mesh.Dispose()
        Mesh = Nothing
        Stream.Position = Pos
        Return LoadMesh(Name, Stream, Fmt, MeshFlags)
    End Function

    Public Function LoadMesh(ByVal Name As String, ByVal Stream As IO.Stream, _
                             ByVal Format As Direct3D.VertexFormats, _
                             Optional ByVal MeshFlags As Direct3D.MeshFlags = Direct3D.MeshFlags.Managed) As Boolean
        If mblnHasBeenInit = False Then Return False
        If mobjMeshes.ContainsKey(Name) Then Return False

        Dim M, Mats() As Direct3D.ExtendedMaterial
        Dim Mesh As Mesh

        Mesh = New Mesh()
        Dim Tmp As Direct3D.Mesh
        Tmp = Direct3D.Mesh.FromStream(Stream, MeshFlags, mobjDevice, Mats)
        Mesh.Mesh = Tmp.Clone(MeshFlags, Format, mobjDevice)
        Tmp.Dispose()
        Tmp = Nothing

        'redim mesh.mobjMaterials (mats.Length-1)
        Dim idx As Integer = 0
        For Each M In Mats
            Dim MName As String
            MName = Name & "_Material" & idx.ToString
            AddMaterial(MName, M.Material3D)
            Mesh.mobjMaterials.Add(MName)

            If IsNothing(M.TextureFilename) = False Then
                If M.TextureFilename.Length <> 0 Then
                    Dim Tex As Direct3D.Texture
                    Dim tName As String = NOTEXTURE
                    gobjEvents.DoLoadMesh(Name, M.TextureFilename, Tex, tName)
                    ' If Tex Is Nothing = False Then
                    Mesh.mobjTextures.Add(tName)
                    ' End  If
                End If
            End If

            idx += 1
        Next
        AddMesh(Name, Mesh)
        ' SetMeshMaterials(Mesh, Dev, Mats, Textures, TextureCallback, TexturePath)

        Return True
    End Function

    '''' <summary>
    '''' Loads a mesh from a file.
    '''' </summary>
    '''' <param name="Filename">The mesh file to load.</param>
    '''' <param name="Dev">A reference to a Direct3D.Device type.</param>
    '''' <param name="Textures">A reference to a DX9Tools.SimpleTextureCollection type that any textures will be added to.</param>
    '''' <param name="MeshFlags">Defines flags used to specify creation options for a mesh.</param>
    '''' <param name="TextureCallback">Provides a reference to a LoadTextureFromFile delagate. If this is NOT 'nothing' then the LoadMesh method will rely on it to load the texture.</param>
    '''' <param name="TexturePath">Specifies the path to the location of the images that will be loaded.</param>
    '''' <returns>Returns a reference to a new MeshEx type.</returns>
    'Public Function LoadMesh(ByVal Filename As String, ByVal Dev As Direct3D.Device, _
    '                        ByVal Textures As DX9Tools.Collections.TextureCollection, _
    '                        Optional ByVal MeshFlags As Direct3D.MeshFlags = Direct3D.MeshFlags.Managed, _
    '                        Optional ByVal TextureCallback As LoadTextureFromFile = Nothing, Optional ByVal TexturePath As String = Nothing) As MeshEx
    '    Dim Mats() As Direct3D.ExtendedMaterial
    '    Dim Mesh As MeshEx

    '    Mesh = New DX9Tools.MeshEx()
    '    Mesh.Mesh = Direct3D.Mesh.FromFile(Filename, MeshFlags, Dev, Mats)
    '    SetMeshMaterials(Mesh, Dev, Mats, Textures, TextureCallback, TexturePath)

    '    Return Mesh
    'End Function

    '''' <summary>
    '''' Loads a mesh from a stream.
    '''' </summary>
    '''' <param name="DataStream">The stream to load the mesh from.</param>
    '''' <param name="Dev">A reference to a Direct3D.Device type.</param>
    '''' <param name="Textures">A reference to a DX9Tools.SimpleTextureCollection type that any textures will be added to.</param>
    '''' <param name="MeshFlags">Defines flags used to specify creation options for a mesh.</param>
    '''' <param name="TextureCallback">Provides a reference to a LoadTextureFromFile delagate. If this is NOT 'nothing' then the LoadMesh method will rely on it to load the texture.</param>
    '''' <param name="TexturePath">Specifies the path to the location of the images that will be loaded.</param>
    '''' <returns>Returns a reference to a new MeshEx type.</returns>
    'Public Function LoadMesh(ByVal DataStream As IO.Stream, ByVal Dev As Direct3D.Device, _
    '                        ByVal Textures As DX9Tools.Collections.SimpleTextureCollection, _
    '                        Optional ByVal MeshFlags As Direct3D.MeshFlags = Direct3D.MeshFlags.Managed, _
    '                        Optional ByVal TextureCallback As LoadTextureFromFile = Nothing, Optional ByVal TexturePath As String = Nothing) As MeshEx
    '    Dim Mats() As Direct3D.ExtendedMaterial
    '    Dim Mesh As MeshEx

    '    Mesh = New DX9Tools.MeshEx()
    '    Mesh.Mesh = Direct3D.Mesh.FromStream(DataStream, MeshFlags, Dev, Mats)
    '    SetMeshMaterials(Mesh, Dev, Mats, Textures, TextureCallback, TexturePath)

    '    Return Mesh
    'End Function

    '''' <summary>
    '''' Loads a mesh from a stream.
    '''' </summary>
    '''' <param name="DataStream">The stream to load the mesh from.</param>
    '''' <param name="Dev">A reference to a Direct3D.Device type.</param>
    '''' <param name="Textures">A reference to a DX9Tools.SimpleTextureCollection type that any textures will be added to.</param>
    '''' <param name="MeshFlags">Defines flags used to specify creation options for a mesh.</param>
    '''' <param name="TextureCallback">Provides a reference to a LoadTextureFromFile delagate. If this is NOT 'nothing' then the LoadMesh method will rely on it to load the texture.</param>
    '''' <param name="TexturePath">Specifies the path to the location of the images that will be loaded.</param>
    '''' <returns>Returns a reference to a new MeshEx type.</returns>
    '''' <remarks>If a texture path is specified in the TexturePath parameter any</remarks>
    'Public Function LoadMesh(ByVal DataStream As IO.Stream, ByVal Dev As Direct3D.Device, _
    '                        ByVal Textures As DX9Tools.Collections.TextureCollection, _
    '                        Optional ByVal MeshFlags As Direct3D.MeshFlags = Direct3D.MeshFlags.Managed, _
    '                        Optional ByVal TextureCallback As LoadTextureFromFile = Nothing, Optional ByVal TexturePath As String = Nothing) As MeshEx
    '    Dim Mats() As Direct3D.ExtendedMaterial
    '    Dim Mesh As MeshEx

    '    Mesh = New DX9Tools.MeshEx()
    '    Mesh.Mesh = Direct3D.Mesh.FromStream(DataStream, MeshFlags, Dev, Mats)
    '    SetMeshMaterials(Mesh, Dev, Mats, Textures, TextureCallback, TexturePath)

    '    Return Mesh
    'End Function
#End Region

    '#Region "  SetMeshMaterials overloads ... "
    '    ''' <summary>
    '    ''' Adds materals to a MexhEx object, loading textures if nessary.
    '    ''' </summary>
    '    ''' <param name="Mesh">A reference to a MeshEx type that the materails will be added to.</param>
    '    ''' <param name="Dev">A reference to a  Direct3D.Device type.</param>
    '    ''' <param name="Materials">The array of materials to be added to the Mesh.</param>
    '    ''' <param name="Textures">A reference to a texture collection. Any texture that is loaded will be added to this collection</param>
    '    ''' <param name="TextureCallback">Provides a reference to a LoadTexture delagate. If this is NOT 'nothing' then the SetMeshMaterials method will rely on it to load the texture.</param>
    '    ''' <param name="TexturePath">Specifies the path to the location of the images that will be loaded.</param>
    '    ''' <exception cref="DX9ToolsD3DDeviceIsNothing">
    '    '''     Will throw a DX9ToolsD3DDeviceIsNothing Exception if the Dev parameter is nothing and it is needed to load a texture.
    '    ''' </exception>
    '    Public Shared Sub SetMeshMaterials(ByVal Mesh As MeshEx, ByVal Dev As Direct3D.Device, _
    ' ByVal Materials() As Direct3D.ExtendedMaterial, ByVal Textures As DX9Tools.Collections.SimpleTextureCollection, _
    ' Optional ByVal TextureCallback As LoadTextureFromFile = Nothing, Optional ByVal TexturePath As String = Nothing)
    '        Dim TmpMat As MaterialTextureCombo
    '        Dim idx As Integer

    '        If Not Materials Is Nothing Then

    '            For idx = 0 To Materials.Length - 1
    '                ' create a new material
    '                TmpMat = New MaterialTextureCombo()
    '                TmpMat.Material = Materials(idx).Material3D

    '                ' check if the material
    '                If Not Materials(idx).TextureFilename Is Nothing Then
    '                    If Materials(idx).TextureFilename.Length > 0 And Not Textures Is Nothing Then
    '                        Dim Tex As Direct3D.Texture
    '                        Dim Path As String

    '                        ' build the path to the texture that will be loaded
    '                        If TexturePath Is Nothing Then
    '                            Path = Materials(idx).TextureFilename
    '                        Else
    '                            Path = IO.Path.Combine(TexturePath, IO.Path.GetFileName(Materials(idx).TextureFilename))
    '                        End If

    '                        ' load the texture
    '                        If TextureCallback Is Nothing Then
    '                            If Dev Is Nothing Then Throw New Exceptions.DX9ToolsD3DDeviceIsNothing("Cannot load texture without a valid reference to a Direct3D.Device type.")
    '                            Tex = Direct3D.TextureLoader.FromFile(Dev, Path)
    '                        Else
    '                            Tex = TextureCallback(Path)
    '                        End If

    '                        ' add the texture to the texture collection
    '                        TmpMat.Textures(TextureStage.Base) = Textures.Add(Tex)
    '                    Else
    '                        TmpMat.Textures(TextureStage.Base) = -1
    '                    End If
    '                End If

    '                ' add the material to the list
    '                Mesh.AddMaterial(TmpMat.Material, TextureStage.Base, TmpMat.Textures(TextureStage.Base))
    '            Next ' idx loop
    '        End If ' mats is not nothing
    '    End Sub

    '    ''' <summary>
    '    ''' Adds materals to a MexhEx object, loading textures if nessary.
    '    ''' </summary>
    '    ''' <param name="Mesh">A reference to a MeshEx type that the materails will be added to.</param>
    '    ''' <param name="Dev">A reference to a  Direct3D.Device type.</param>
    '    ''' <param name="Materials">The array of materials to be added to the Mesh.</param>
    '    ''' <param name="Textures">A reference to a texture collection. Any texture that is loaded will be added to this collection</param>
    '    ''' <param name="TextureCallback">Provides a reference to a LoadTexture delagate. If this is NOT 'nothing' then the SetMeshMaterials method will rely on it to load the texture.</param>
    '    ''' <param name="TexturePath">Specifies the path to the location of the images that will be loaded.</param>
    '    ''' <exception cref="DX9ToolsD3DDeviceIsNothing">
    '    '''     Will throw a DX9ToolsD3DDeviceIsNothing Exception if the Dev parameter is nothing and it is needed to load a texture.
    '    ''' </exception>
    '    Public Shared Sub SetMeshMaterials(ByVal Mesh As MeshEx, ByVal Dev As Direct3D.Device, _
    ' ByVal Materials() As Direct3D.ExtendedMaterial, ByVal Textures As DX9Tools.Collections.TextureCollection, _
    ' Optional ByVal TextureCallback As LoadTextureFromFile = Nothing, Optional ByVal TexturePath As String = Nothing)
    '        Dim TmpMat As MaterialTextureCombo
    '        Dim idx As Integer

    '        If Not Materials Is Nothing Then

    '            For idx = 0 To Materials.Length - 1
    '                ' create a new material
    '                TmpMat = New MaterialTextureCombo()
    '                TmpMat.Material = Materials(idx).Material3D

    '                ' check if the material
    '                If Not Materials(idx).TextureFilename Is Nothing Then
    '                    If Materials(idx).TextureFilename.Length > 0 And Not Textures Is Nothing Then
    '                        Dim Tex As Direct3D.Texture
    '                        Dim Path As String

    '                        ' build the path to the texture that will be loaded
    '                        If TexturePath Is Nothing Then
    '                            Path = Materials(idx).TextureFilename
    '                        Else
    '                            Path = IO.Path.Combine(TexturePath, IO.Path.GetFileName(Materials(idx).TextureFilename))
    '                        End If

    '                        ' load the texture
    '                        If TextureCallback Is Nothing Then
    '                            If Dev Is Nothing Then Throw New Exceptions.DX9ToolsD3DDeviceIsNothing("Cannot load texture without a valid reference to a Direct3D.Device type.")
    '                            Tex = Direct3D.TextureLoader.FromFile(Dev, Path)
    '                        Else
    '                            Tex = TextureCallback(Path)
    '                        End If

    '                        ' add the texture to the texture collection
    '                        TmpMat.Textures(TextureStage.Base) = Textures.Add(Tex)
    '                    Else
    '                        TmpMat.Textures(TextureStage.Base) = -1
    '                    End If
    '                End If

    '                ' add the material to the list
    '                Mesh.AddMaterial(TmpMat.Material, TextureStage.Base, TmpMat.Textures(TextureStage.Base))
    '            Next ' idx loop
    '        End If ' mats is not nothing
    '    End Sub
    '#End Region

#Region " GenerateEllipsePoints overloads ... "
    '''' <summary>
    '''' Generates an array of Point types that represent an ellipse.
    '''' </summary>
    '''' <param name="R">The rectangle that specifies the bounds of the ellipse.</param>
    '''' <param name="Pts">The varible that will be used to store the array of Point types that represent the ellipse.</param>
    'Public Function GenerateEllipsePoints(ByVal R As RectangleF) As Point()
    '    Dim TmpPts() As Vector2
    '    Dim idx As Integer
    '    Dim Pts() As Point

    '    GenerateEllipsePoints(R, 359, DegreeToRadian(1), DegreeToRadian(0), TmpPts)

    '    ReDim Pts(359)
    '    For idx = 0 To 359
    '        Pts(idx) = New Point(CInt(TmpPts(idx).X), CInt(TmpPts(idx).Y))
    '    Next

    '    Return Pts
    'End Function

    '''' <summary>
    '''' Generates an array of Point types that represent an ellipse.
    '''' </summary>
    '''' <param name="R">The rectangle that specifies the bounds of the ellipse.</param>
    '''' <param name="Pts">The varible that will be used to store the array of PointF types that represent the ellipse.</param>
    'Public Sub GenerateEllipsePoints(ByVal R As RectangleF, ByRef Pts() As PointF)
    '    Dim TmpPts() As Vector2
    '    Dim idx As Integer

    '    GenerateEllipsePoints(R, 359, DegreeToRadian(1), DegreeToRadian(0), TmpPts)

    '    ReDim Pts(359)
    '    For idx = 0 To 359
    '        Pts(idx) = New PointF(TmpPts(idx).X, TmpPts(idx).Y)
    '    Next
    'End Sub

    ''' <summary>
    ''' Generates an array of Point types that represent an ellipse.
    ''' </summary>
    ''' <param name="R">The rectangle that specifies the bounds of the ellipse.</param>
    ''' <param name="Pts">The varible that will be used to store the array of Vector2 types that represent the ellipse.</param>
    Public Function GenerateEllipsePoints(ByVal R As RectangleF) As Vector2()
        Return GenerateEllipsePoints(R, 359, DegreeToRadian(1), DegreeToRadian(0))
    End Function

    ''' <summary>
    ''' Generates an array of Point types that represent an ellipse.
    ''' </summary>
    ''' <param name="R">The rectangle that specifies the bounds of the ellipse.</param>
    ''' <param name="NumPoints">The number of points in the ellipse.</param>
    ''' <param name="DegPerPoint">The number of degress to use between each point.</param>
    ''' <param name="StartDeg">The start degree.</param>
    ''' <param name="Pts">The varible that will be used to store the array of Vector2 types that represent the ellipse.</param>
    Public Function GenerateEllipsePoints(ByVal R As RectangleF, ByVal NumPoints As Integer, _
                                          ByVal DegPerPoint As Single, ByVal StartDeg As Single) As Vector2()
        Dim idx As Integer
        Dim X, Y, Radian As Single
        Dim Pts() As Vector2

        ReDim Pts(NumPoints)

        Radian = StartDeg
        For idx = 0 To NumPoints
            Pts(idx) = New Vector2(CSng(R.Left + System.Math.Cos(Radian) * R.Right), _
                                   CSng(R.Top + System.Math.Sin(Radian) * R.Bottom))
            Radian += DegPerPoint
        Next

        Return Pts
    End Function

    '''' <summary>
    '''' Generates an array of Point types that represent an ellipse.
    '''' </summary>
    '''' <param name="R">The rectangle that specifies the bounds of the ellipse.</param>
    '''' <param name="Pts">The varible that will be used to store the array of Vector3 types that represent the ellipse.</param>
    'Public Sub GenerateEllipsePoints(ByVal R As RectangleF, ByRef Pts() As Vector3)
    '    GenerateEllipsePoints(R, Orientation.Xy, 359, DegreeToRadian(1), DegreeToRadian(0), Pts)
    'End Sub

    '''' <summary>
    '''' Generates an array of Point types that represent an ellipse.
    '''' </summary>
    '''' <param name="R">The rectangle that specifies the bounds of the ellipse.</param>
    '''' <param name="Orient">Specifies the orientation axis.</param>
    '''' <param name="Pts">The varible that will be used to store the array of Vector3 types that represent the ellipse.</param>
    'Public Sub GenerateEllipsePoints(ByVal R As RectangleF, ByVal Orient As Orientation, ByRef Pts() As Vector3)
    '    GenerateEllipsePoints(R, Orient, 359, DegreeToRadian(1), DegreeToRadian(0), Pts)
    'End Sub

    '''' <summary>
    '''' Generates an array of Point types that represent an ellipse.
    '''' </summary>
    '''' <param name="R">The rectangle that specifies the bounds of the ellipse.</param>
    '''' <param name="Orient">Specifies the orientation axis.</param>
    '''' <param name="NumPoints">The number of points in the ellipse.</param>
    '''' <param name="DegPerPoint">The number of degress to use between each point.</param>
    '''' <param name="StartDeg">The start degree.</param>
    '''' <param name="Pts">The varible that will be used to store the array of Vector3 types that represent the ellipse.</param>
    '''' <exception cref="ArgumentOutOfRangeException">
    ''''     Will throw a ArgumentOutOfRangeException Exception if the NumPoints is less then or equal then 0.
    '''' </exception>
    'Public Sub GenerateEllipsePoints(ByVal R As RectangleF, ByVal Orient As Orientation, _
    '                                        ByVal NumPoints As Integer, _
    '                                        ByVal DegPerPoint As Single, ByVal StartDeg As Single, _
    '                                        ByRef Pts() As Vector3)
    '    Dim idx As Integer
    '    Dim X, Y, Radian As Single

    '    If NumPoints = 0 Then Throw New ArgumentOutOfRangeException("NumPoints", NumPoints, "NumPoints needs to be equal to or greater than 1.")

    '    ReDim Pts(NumPoints - 1)

    '    Radian = StartDeg
    '    For idx = 0 To Pts.Length - 1
    '        Select Case Orient
    '            Case Orientation.XY
    '                Pts(idx) = New Vector3(CSng(R.Left + System.Math.Cos(Radian) * R.Right), _
    '                                       CSng(R.Top + System.Math.Sin(Radian) * R.Bottom), _
    '                                       0)
    '            Case Orientation.XZ
    '                Pts(idx) = New Vector3(CSng(R.Left + System.Math.Cos(Radian) * R.Right), _
    '                                       0, _
    '                                       CSng(R.Top + System.Math.Sin(Radian) * R.Bottom))
    '            Case Orientation.ZY
    '                Pts(idx) = New Vector3(0, _
    '                                       CSng(R.Top + System.Math.Sin(Radian) * R.Bottom), _
    '                                       CSng(R.Left + System.Math.Cos(Radian) * R.Right))
    '        End Select

    '        Radian += DegPerPoint
    '    Next
    'End Sub

    '''' <summary>
    '''' Generates an array of Point types that represent an ellipse.
    '''' </summary>
    '''' <param name="R">The rectangle that specifies the bounnds of the ellipse.</param>
    '''' <param name="Pts">The varible that will be used to store the array of Point types that represent the ellipse.</param>
    'Public Sub GenerateEllipsePoints(ByVal R As Rectangle, ByRef Pts() As Point)
    '    GenerateEllipsePoints(New RectangleF(R.Left, R.Top, R.Width, R.Height), Pts)
    'End Sub

    '''' <summary>
    '''' Generates an array of Point types that represent an ellipse.
    '''' </summary>
    '''' <param name="R">The rectangle that specifies the bounnds of the ellipse.</param>
    '''' <param name="Pts">The varible that will be used to store the array of PointF types that represent the ellipse.</param>
    'Public Sub GenerateEllipsePoints(ByVal R As Rectangle, ByRef Pts() As PointF)
    '    GenerateEllipsePoints(New RectangleF(R.Left, R.Top, R.Width, R.Height), Pts)
    'End Sub

    ''' <summary>
    ''' Generates an array of Point types that represent an ellipse.
    ''' </summary>
    ''' <param name="R">The rectangle that specifies the bounnds of the ellipse.</param>
    ''' <param name="Pts">The varible that will be used to store the array of Vector2 types that represent the ellipse.</param>
    Public Function GenerateEllipsePoints(ByVal R As Rectangle, ByRef Pts() As Vector2) As Vector2()
        Return GenerateEllipsePoints(New RectangleF(R.Left, R.Top, R.Width, R.Height))
    End Function

    ''' <summary>
    ''' Generates an array of Point types that represent an ellipse.
    ''' </summary>
    ''' <param name="R">The rectangle that specifies the bounnds of the ellipse.</param>
    ''' <param name="NumPoints">The number of points in the ellipse.</param>
    ''' <param name="DegPerPoint">The number of degress to use between each point.</param>
    ''' <param name="StartDeg">The start degree.</param>
    ''' <param name="Pts">The varible that will be used to store the array of Vector2 types that represent the ellipse.</param>
    Public Function GenerateEllipsePoints(ByVal R As Rectangle, ByVal NumPoints As Integer, _
                                            ByVal DegPerPoint As Single, ByVal StartDeg As Single) As Vector2()
        Return GenerateEllipsePoints(New RectangleF(R.Left, R.Top, R.Width, R.Height), NumPoints, DegPerPoint, StartDeg)
    End Function

    '''' <summary>
    '''' Generates an array of Point types that represent an ellipse.
    '''' </summary>
    '''' <param name="R">The rectangle that specifies the bounnds of the ellipse.</param>
    '''' <param name="Pts">The varible that will be used to store the array of Vector3 types that represent the ellipse.</param>
    'Public Function GenerateEllipsePoints(ByVal R As Rectangle) As Vector3()
    '    Return GenerateEllipsePoints(New RectangleF(R.Left, R.Top, R.Width, R.Height))
    'End Function

    '''' <summary>
    '''' Generates an array of Point types that represent an ellipse.
    '''' </summary>
    '''' <param name="R">The rectangle that specifies the bounnds of the ellipse.</param>
    '''' <param name="Orient">Specifies the orientation axis.</param>
    '''' <param name="Pts">The varible that will be used to store the array of Vector3 types that represent the ellipse.</param>
    'Public Sub GenerateEllipsePoints(ByVal R As Rectangle, ByVal Orient As Orientation, ByRef Pts() As Vector3)
    '    GenerateEllipsePoints(New RectangleF(R.Left, R.Top, R.Width, R.Height), Orient, Pts)
    'End Sub

    '''' <summary>
    '''' Generates an array of Point types that represent an ellipse.
    '''' </summary>
    '''' <param name="R">The rectangle that specifies the bounnds of the ellipse.</param>
    '''' <param name="Orient">Specifies the orientation axis.</param>
    '''' <param name="NumPoints">The number of points in the ellipse.</param>
    '''' <param name="DegPerPoint">The number of degress to use between each point.</param>
    '''' <param name="StartDeg">The start degree.</param>
    '''' <param name="Pts">The varible that will be used to store the array of Vector3 types that represent the ellipse.</param>
    'Public Function GenerateEllipsePoints(ByVal R As Rectangle, ByVal Orient As Orientation, _
    '                                        ByVal NumPoints As Integer, _
    '                                        ByVal DegPerPoint As Single, ByVal StartDeg As Single) As Vector3()
    '    Return GenerateEllipsePoints(New RectangleF(R.Left, R.Top, R.Width, R.Height), Orient, NumPoints, DegPerPoint, StartDeg)
    'End Function
#End Region

    '#Region " VectorSwap overloads ... "
    '    ''' <summary>
    '    ''' Swaps 2 values in a an array of Vector3 structures.
    '    ''' </summary>
    '    ''' <param name="Vertexes">The reference to an array of vectors to swap the values of.</param>
    '    ''' <param name="SwapWhat">Specifies what components of the Vector3 structures to swap the values of.</param>
    '    ''' <returns>Returns a new array of Vector3 structures with the new swapped values.</returns>
    '    ''' <remarks>
    '    ''' Using VectorSwap you can rotate a vector/mesh along a different axis. For example...
    '    ''' <example>
    '    ''' <code>
    '    ''' Dim V1 As Vector3 = New Vector3(10, 5, 1)
    '    ''' Dim V2 As Vector3
    '    ''' V2 = VectorSwap(V1, Orientation.XY)
    '    ''' The values of V2 are now ...
    '    ''' ' V2.X = 5
    '    ''' ' V2.Y = 10
    '    ''' ' V2.Z = 1
    '    ''' </code>
    '    ''' </example>
    '    ''' </remarks>
    '    Public Function VectorSwap(ByVal Vertexes() As Vector3, ByVal SwapWhat As Orientation) As Vector3()
    '        Dim Verts() As Vector3

    '        Dim idx As Integer
    '        If Vertexes Is Nothing Then Return Nothing
    '        ReDim Verts(Vertexes.Length - 1)
    '        For idx = 0 To Vertexes.Length - 1
    '            Verts(idx) = VectorSwap(Vertexes(idx), SwapWhat)
    '        Next

    '        Return Verts
    '    End Function

    '    ''' <summary>
    '    ''' Swaps 2 values in a Vector3 structure.
    '    ''' </summary>
    '    ''' <param name="Vertex">The reference vector to swap the values of.</param>
    '    ''' <param name="SwapWhat">Specifies what components of the Vector3 structure to swap the values of.</param>
    '    ''' <returns>Returns a new Vector3 with the new swapped values.</returns>
    '    ''' <remarks>
    '    ''' Using VectorSwap you can rotate a vector/mesh along a different axis. For example...
    '    ''' <example>
    '    ''' <code>
    '    ''' Dim V1 As Vector3 = New Vector3(10, 5, 1)
    '    ''' Dim V2 As Vector3
    '    ''' V2 = VectorSwap(V1, Orientation.XY)
    '    ''' The values of V2 are now ...
    '    ''' ' V2.X = 5
    '    ''' ' V2.Y = 10
    '    ''' ' V2.Z = 1
    '    ''' </code>
    '    ''' </example>
    '    ''' </remarks>
    '    Public Function VectorSwap(ByVal Vertex As Vector3, ByVal SwapWhat As Orientation) As Vector3
    '        Select Case SwapWhat
    '            Case Orientation.XY
    '                Return New Vector3(Vertex.Y, Vertex.X, Vertex.Z)

    '            Case Orientation.XZ
    '                Return New Vector3(Vertex.Z, Vertex.Y, Vertex.X)

    '            Case Orientation.ZY
    '                Return New Vector3(Vertex.X, Vertex.Z, Vertex.Y)

    '        End Select
    '    End Function
    '#End Region

    '#Region " Vector3FromRectangle overloads ... "
    '    'TODO: add more Vector3FromRectangle overloads 
    '    ''' <summary>
    '    ''' Generates an array of 4 to 6 vector3 types using a rectanglef type as the source.
    '    ''' </summary>
    '    ''' <param name="R">The source rectangle from which the vectors will be built from.</param>
    '    ''' <param name="Axis">Determins on what axis the returned vectors will be aligned with.</param>
    '    ''' <returns>Returns an array of 4 to 6 Vector3 types.</returns>
    '    ''' <remarks>
    '    ''' Only accepts TriangleFan, TriangleList, and TriangleStrip otherwise returns nothing. 
    '    ''' If you specify TriangleList Vector3FromRectangle will return 6 vertexes, TriangleFan and 
    '    ''' TriangleStrip only return 4 vertexes.
    '    ''' Equivalent to calling ...
    '    ''' <example>
    '    ''' <code>
    '    ''' Vector3FromRectangle(R, Direct3D.PrimitiveType.TriangleList, 0, Axis)
    '    ''' </code>
    '    ''' </example>
    '    ''' </remarks>
    '    Public Function Vector3FromRectangle(ByVal R As RectangleF, ByVal Axis As Orientation) As Vector3()
    '        Return Vector3FromRectangle(R, Direct3D.PrimitiveType.TriangleList, 0, Axis)
    '    End Function

    '    ''' <summary>
    '    ''' Generates an array of 4 to 6 vector3 types using a rectanglef type as the source.
    '    ''' </summary>
    '    ''' <param name="R">The source rectangle from which the vectors will be built from.</param>
    '    ''' <param name="Depth">Sets the depth to use when creating the vectors.</param>
    '    ''' <returns>Returns an array of 4 to 6 Vector3 types.</returns>
    '    ''' <remarks>
    '    ''' Only accepts TriangleFan, TriangleList, and TriangleStrip otherwise returns nothing. 
    '    ''' If you specify TriangleList Vector3FromRectangle will return 6 vertexes, TriangleFan and 
    '    ''' TriangleStrip only return 4 vertexes.
    '    ''' Equivalent to calling ...
    '    ''' <example>
    '    ''' <code>
    '    ''' Vector3FromRectangle(R, Direct3D.PrimitiveType.TriangleList, Depth, Orientation.XY)
    '    ''' </code>
    '    ''' </example>
    '    ''' </remarks>
    '    Public Function Vector3FromRectangle(ByVal R As RectangleF, ByVal Depth As Single) As Vector3()
    '        Return Vector3FromRectangle(R, Direct3D.PrimitiveType.TriangleList, Depth, Orientation.XY)
    '    End Function

    '    ''' <summary>
    '    ''' Generates an array of 4 to 6 vector3 types using a rectanglef type as the source.
    '    ''' </summary>
    '    ''' <param name="R">The source rectangle from which the vectors will be built from.</param>
    '    ''' <returns>Returns an array of 4 to 6 Vector3 types.</returns>
    '    ''' <remarks>
    '    ''' Only accepts TriangleFan, TriangleList, and TriangleStrip otherwise returns nothing. 
    '    ''' If you specify TriangleList Vector3FromRectangle will return 6 vertexes, TriangleFan and 
    '    ''' TriangleStrip only return 4 vertexes.
    '    ''' Equivalent to calling ...
    '    ''' <example>
    '    ''' <code>
    '    ''' Vector3FromRectangle(R, Direct3D.PrimitiveType.TriangleList, 0, Orientation.XY)
    '    ''' </code>
    '    ''' </example>
    '    ''' </remarks>
    '    Public Function Vector3FromRectangle(ByVal R As RectangleF) As Vector3()
    '        Return Vector3FromRectangle(R, Direct3D.PrimitiveType.TriangleList, 0, Orientation.XY)
    '    End Function

    '    ''' <summary>
    '    ''' Generates an array of 4 to 6 vector3 types using a rectanglef type as the source.
    '    ''' </summary>
    '    ''' <param name="R">The source rectangle from which the vectors will be built from.</param>
    '    ''' <param name="Order">Determins how the vertexes will be stored in vertex array.</param>
    '    ''' <returns>Returns an array of 4 to 6 Vector3 types.</returns>
    '    ''' <remarks>
    '    ''' Only accepts TriangleFan, TriangleList, and TriangleStrip otherwise returns nothing. 
    '    ''' If you specify TriangleList Vector3FromRectangle will return 6 vertexes, TriangleFan and 
    '    ''' TriangleStrip only return 4 vertexes.
    '    ''' Equivalent to calling ...
    '    ''' <example>
    '    ''' <code>
    '    ''' Vector3FromRectangle(R, Order, 0, Orientation.XY)
    '    ''' </code>
    '    ''' </example>
    '    ''' </remarks>
    '    Public Function Vector3FromRectangle(ByVal R As RectangleF, ByVal Order As Direct3D.PrimitiveType) As Vector3()
    '        Return Vector3FromRectangle(R, Order, 0, Orientation.XY)
    '    End Function

    '    ''' <summary>
    '    ''' Generates an array of 4 to 6 vector3 types using a rectanglef type as the source.
    '    ''' </summary>
    '    ''' <param name="R">The source rectangle from which the vectors will be built from.</param>
    '    ''' <param name="Order">Determins how the vertexes will be stored in vertex array.</param>
    '    ''' <param name="Depth">Sets the depth to use when creating the vectors.</param>
    '    ''' <returns>Returns an array of 4 to 6 Vector3 types.</returns>
    '    ''' <remarks>
    '    ''' Only accepts TriangleFan, TriangleList, and TriangleStrip otherwise returns nothing. 
    '    ''' If you specify TriangleList Vector3FromRectangle will return 6 vertexes, TriangleFan and 
    '    ''' TriangleStrip only return 4 vertexes.
    '    ''' Equivalent to calling ...
    '    ''' <example>
    '    ''' <code>
    '    ''' Vector3FromRectangle(R, Order, Depth, Orientation.XY)
    '    ''' </code>
    '    ''' </example>
    '    ''' </remarks>
    '    Public Function Vector3FromRectangle(ByVal R As RectangleF, ByVal Order As Direct3D.PrimitiveType, ByVal Depth As Single) As Vector3()
    '        Return Vector3FromRectangle(R, Order, Depth, Orientation.XY)
    '    End Function

    '    ''' <summary>
    '    ''' Generates an array of 4 to 6 vector3 types using a rectanglef type as the source.
    '    ''' </summary>
    '    ''' <param name="R">The source rectangle from which the vectors will be built from.</param>
    '    ''' <param name="Order">Determins how the vertexes will be stored in vertex array.</param>
    '    ''' <param name="Depth">Sets the depth to use when creating the vectors.</param>
    '    ''' <param name="Axis">Determins on what axis the returned vectors will be aligned with.</param>
    '    ''' <returns>Returns an array of 4 to 6 Vector3 types.</returns>
    '    ''' <remarks>
    '    ''' Only accepts TriangleFan, TriangleList, and TriangleStrip otherwise returns nothing. 
    '    ''' If you specify TriangleList Vector3FromRectangle will return 6 vertexes, TriangleFan and 
    '    ''' TriangleStrip only return 4 vertexes.
    '    ''' </remarks>
    '    Public Function Vector3FromRectangle(ByVal R As RectangleF, ByVal Order As Direct3D.PrimitiveType, ByVal Depth As Single, ByVal Axis As Orientation) As Vector3()
    '        Dim V(3) As Vector3

    '        ' TopLeft
    '        V(0) = New Vector3(R.Left, R.Bottom, Depth)
    '        ' Topright
    '        V(1) = New Vector3(R.Right, R.Bottom, Depth)
    '        ' bottomleft
    '        V(2) = New Vector3(R.Left, R.Top, Depth)
    '        ' bottomright
    '        V(3) = New Vector3(R.Right, R.Top, Depth)

    '        ' orient vertex along proper axis
    '        If Axis <> Orientation.XY Then
    '            Dim idx As Integer
    '            For idx = 0 To 3
    '                V(idx) = VectorSwap(V(idx), Axis)
    '            Next
    '        End If

    '        ' determin order and return the arraw of vertexe points
    '        Select Case Order
    '            Case Direct3D.PrimitiveType.TriangleFan
    '                Dim VX() As Vector3 = {V(0), V(1), V(3), V(2)}
    '                Return VX

    '            Case Direct3D.PrimitiveType.TriangleList
    '                Dim VX() As Vector3 = {V(0), V(1), V(2), V(2), V(1), V(3)}
    '                Return VX

    '            Case Direct3D.PrimitiveType.TriangleStrip
    '                Dim VX() As Vector3 = {V(2), V(0), V(3), V(1)}
    '                Return VX

    '        End Select

    '        'un recognized primitive type so return nothing
    '        Return Nothing
    '    End Function
    '#End Region

    '#Region " Vector2FromRectangle overloads ... "
    '    ''' <summary>
    '    ''' Generates an array of 4 to 8 vector2 types using a rectanglef type as the source.
    '    ''' </summary>
    '    ''' <param name="R">The source rectangle from which the vectors will be built from.</param>
    '    ''' <param name="Order">Determins how the vertexes will be stored in vertex array.</param>
    '    ''' <param name="Depth">Sets the depth to use when creating the vectors.</param>
    '    ''' <param name="Axis">Determins on what axis the returned vectors will be aligned with.</param>
    '    ''' <returns>Returns an array of 4 to 8 Vector2 types.</returns>
    '    ''' <remarks>
    '    ''' Only accepts standard primitive types found in the Direct3D.PrimitiveType enum otherwise returns nothing. 
    '    ''' If you specify TriangleList Vector3FromRectangle will return 6 vertexes, TriangleFan and 
    '    ''' TriangleStrip only return 4 vertexes.
    '    ''' If you specify LineList Vector2FromRectangle will return 8 vertexes representing 4 lines starting with the top 
    '    ''' line in a clockwise direction.
    '    ''' If you specify LineStrip Vector2FromRectangle will return 5 vertexes representing 4 lines starting with the top 
    '    ''' line in a clockwise direction.
    '    ''' If you specify PointList Vector2FromRectangle will return 4 vertexes representing 4 points starting from the top left 
    '    ''' moving in a clockwise direction.
    '    ''' </remarks>
    '    Public Function Vector2FromRectangle(ByVal R As Rectangle, ByVal Order As Direct3D.PrimitiveType) As Vector2()
    '        Return Vector2FromRectangle(New RectangleF(R.Left, R.Top, R.Width, R.Height), Order)
    '    End Function

    '    ''' <summary>
    '    ''' Generates an array of 4 to 8 vector2 types using a rectanglef type as the source.
    '    ''' </summary>
    '    ''' <param name="R">The source rectangle from which the vectors will be built from.</param>
    '    ''' <param name="Order">Determins how the vertexes will be stored in vertex array.</param>
    '    ''' <param name="Depth">Sets the depth to use when creating the vectors.</param>
    '    ''' <param name="Axis">Determins on what axis the returned vectors will be aligned with.</param>
    '    ''' <returns>Returns an array of 4 to 8 Vector2 types.</returns>
    '    ''' <remarks>
    '    ''' Only accepts standard primitive types found in the Direct3D.PrimitiveType enum otherwise returns nothing. 
    '    ''' If you specify TriangleList Vector3FromRectangle will return 6 vertexes, TriangleFan and 
    '    ''' TriangleStrip only return 4 vertexes.
    '    ''' If you specify LineList Vector2FromRectangle will return 8 vertexes representing 4 lines starting with the top 
    '    ''' line in a clockwise direction.
    '    ''' If you specify LineStrip Vector2FromRectangle will return 5 vertexes representing 4 lines starting with the top 
    '    ''' line in a clockwise direction.
    '    ''' If you specify PointList Vector2FromRectangle will return 4 vertexes representing 4 points starting from the top left 
    '    ''' moving in a clockwise direction.
    '    ''' </remarks>
    '    Public Function Vector2FromRectangle(ByVal R As RectangleF, ByVal Order As Direct3D.PrimitiveType) As Vector2()
    '        Dim V(3) As Vector2

    '        ' TopLeft
    '        V(0) = New Vector2(R.Left, R.Bottom)
    '        ' Topright
    '        V(1) = New Vector2(R.Right, R.Bottom)
    '        ' bottomleft
    '        V(2) = New Vector2(R.Left, R.Top)
    '        ' bottomright
    '        V(3) = New Vector2(R.Right, R.Top)


    '        ' determin order and return the arraw of vertex points
    '        Select Case Order
    '            Case Direct3D.PrimitiveType.LineList
    '                Dim VX() As Vector2 = {V(0), V(1), V(1), V(3), V(3), V(2), V(2), V(0)}
    '                Return VX

    '            Case Direct3D.PrimitiveType.LineStrip
    '                Dim VX() As Vector2 = {V(0), V(1), V(3), V(2), V(0)}
    '                Return VX

    '            Case Direct3D.PrimitiveType.PointList
    '                Dim VX() As Vector2 = {V(0), V(1), V(3), V(2)}
    '                Return VX

    '            Case Direct3D.PrimitiveType.TriangleFan
    '                Dim VX() As Vector2 = {V(0), V(1), V(3), V(2)}
    '                Return VX

    '            Case Direct3D.PrimitiveType.TriangleList
    '                Dim VX() As Vector2 = {V(0), V(1), V(2), V(2), V(1), V(3)}
    '                Return VX

    '            Case Direct3D.PrimitiveType.TriangleStrip
    '                Dim VX() As Vector2 = {V(2), V(0), V(3), V(1)}
    '                Return VX

    '        End Select

    '        'un recognized primitive type so return nothing
    '        Return Nothing
    '    End Function
    '#End Region

#Region " Build Rectangle Strip Horiz/Vert overloads ... "
    Public Function BuildVertRectangleStrip(ByVal Rect As Rectangle, ByVal StripCount As Integer) As Vector3()
        Return BuildHorizRectangleStrip(New RectangleF(Rect.Left, Rect.Top, Rect.Width, Rect.Height), StripCount, Nothing)
    End Function

    Public Function BuildVertRectangleStrip(ByVal Rect As Rectangle, ByVal StripCount As Integer, ByRef tVerts() As Vector2) As Vector3()
        Return BuildHorizRectangleStrip(New RectangleF(Rect.Left, Rect.Top, Rect.Width, Rect.Height), StripCount, tVerts)
    End Function

    Public Function BuildVertRectangleStrip(ByVal Rect As RectangleF, ByVal StripCount As Integer) As Vector3()
        Return BuildHorizRectangleStrip(Rect, StripCount, Nothing)
    End Function

    ''' <summary>
    ''' Builds a plane using triangle strips.
    ''' </summary>
    ''' <param name="Rect">Specifies the dimentions of the plane that will be created.</param>
    ''' <param name="StripCount">Specifies the number of strips that will make up plane. 2 polys equals 1 strip.</param>
    ''' <param name="tVerts">Returns a reference to an array of texture coordanants for the plane.</param>
    ''' <returns>Returns an array of Vector3 structures representing the plane.</returns>
    ''' <remarks>
    ''' BuildVertRectangleStrip will create an array of Vector3 structures that represents a
    ''' </remarks>
    Public Function BuildVertRectangleStrip(ByVal Rect As RectangleF, ByVal StripCount As Integer, ByRef tVerts() As Vector2) As Vector3()
        Dim idx, idy As Integer
        Dim V() As Vector3
        Dim tV() As Vector2
        Dim xOff As Single

        If StripCount < 1 Then Throw New ArgumentOutOfRangeException("StripCount")

        ReDim V(StripCount * 2 + 1)
        ReDim tV(StripCount * 2 + 1)

        xOff = Rect.Width / StripCount
        idy = 0
        For idx = 0 To StripCount
            Dim tX As Single
            Dim NextIndex As Integer
            Dim Offset As Single

            tX = CSng(idx / StripCount)
            NextIndex = idy + 1
            Offset = idx * xOff

            V(idy) = New Vector3(Rect.Left + Offset, Rect.Bottom, 0)
            V(NextIndex) = New Vector3(Rect.Left + Offset, Rect.Top, 0)

            tV(idy) = New Vector2(tX, 1)
            tV(NextIndex) = New Vector2(tX, 0)
            idy += 2
        Next

        tVerts = tV
        Return V
    End Function

    Public Function BuildHorizRectangleStrip(ByVal Rect As Rectangle, ByVal StripCount As Integer) As Vector3()
        Return BuildHorizRectangleStrip(New RectangleF(Rect.Left, Rect.Top, Rect.Width, Rect.Height), StripCount, Nothing)
    End Function

    Public Function BuildHorizRectangleStrip(ByVal Rect As Rectangle, ByVal StripCount As Integer, ByRef tVerts() As Vector2) As Vector3()
        Return BuildHorizRectangleStrip(New RectangleF(Rect.Left, Rect.Top, Rect.Width, Rect.Height), StripCount, tVerts)
    End Function

    Public Function BuildHorizRectangleStrip(ByVal Rect As RectangleF, ByVal StripCount As Integer) As Vector3()
        Return BuildVertRectangleStrip(Rect, StripCount, Nothing)
    End Function

    Public Function BuildHorizRectangleStrip(ByVal Rect As RectangleF, ByVal StripCount As Integer, ByRef tVerts() As Vector2) As Vector3()
        Dim idx, idy As Integer
        Dim V() As Vector3
        Dim tV() As Vector2
        Dim yOff As Single

        If StripCount < 1 Then Throw New ArgumentOutOfRangeException("StripCount")

        ReDim V(StripCount * 2 + 1)
        ReDim tV(StripCount * 2 + 1)

        yOff = Rect.Height / StripCount
        idy = 0
        For idx = 0 To StripCount
            Dim tY As Single
            Dim NextIndex As Integer
            Dim Offset As Single

            tY = CSng(idx / StripCount)
            NextIndex = idy + 1
            Offset = idx * yOff

            V(idy) = New Vector3(Rect.Left, Rect.Top + Offset, 0)
            V(NextIndex) = New Vector3(Rect.Right, Rect.Top + Offset, 0)

            tV(idy) = New Vector2(0, tY)
            tV(NextIndex) = New Vector2(1, tY)
            idy += 2
        Next

        tVerts = tV
        Return V
    End Function
#End Region

#Region " BuildLinePoints overloads ... "
    Public Function BuildLinePoints(ByVal V1 As Vector3, ByVal V2 As Vector3, ByVal PointCount As Integer) As Vector3()
        Dim idx As Integer
        Dim V() As Vector3

        If PointCount < 2 Then Throw New ArgumentOutOfRangeException("PointCount")

        ReDim V(PointCount)

        idx = 0
        While idx / (PointCount - 1) <> 1
            V(idx) = Vector3.Lerp(V1, V2, CSng(idx / (PointCount - 1)))
            idx += 1
        End While

        V(V.Length - 1) = V2
        Return V
    End Function

    Public Function BuildLinePoints(ByVal V1 As Vector2, ByVal V2 As Vector2, ByVal PointCount As Integer) As Vector2()
        Dim idx As Integer
        Dim V() As Vector2

        If PointCount < 2 Then Throw New ArgumentOutOfRangeException("PointCount")

        ReDim V(PointCount)

        idx = 0
        While idx / (PointCount - 1) <> 1
            V(idx) = Vector2.Lerp(V1, V2, CSng(idx / (PointCount - 1)))
            idx += 1
        End While

        V(V.Length - 1) = V2
        Return V
    End Function
#End Region

#Region " Build cylindar Strip Horiz/Vert overloads ... "
    Public Function BuildCylinderStrip(ByVal Position As Vector3, ByVal Radius As Single, ByVal StripCount As Integer, ByVal Height As Single) As Vector3()
        Return BuildCylinderStrip(Position, Radius, StripCount, Height, Nothing)
    End Function

    Public Function BuildCylinderStrip(ByVal Position As Vector3, ByVal Radius As Single, ByVal StripCount As Integer, ByVal Height As Single, ByRef tVerts() As Vector2) As Vector3()
        Dim idx, idy As Integer
        Dim V() As Vector3
        Dim tV() As Vector2
        Dim RP As Vector3 = Vector3.Add(Position, New Vector3(Radius, 0, 0))

        If StripCount < 3 Then Throw New ArgumentOutOfRangeException("StripCount")

        ReDim V(StripCount * 2 + 1)
        ReDim tV(StripCount * 2 + 1)

        idy = 0
        For idx = 1 To StripCount + 1
            Dim tX As Single
            Dim NextIndex As Integer

            tX = CSng(idx / StripCount)
            NextIndex = idy + 1

            RP = MathHelpers.RotateAroundAxis(Position, RP, New Vector3(0, 0, DegreeToRadian(CSng(360 / StripCount))))
            V(idy) = RP
            V(NextIndex) = Vector3.Add(V(idy), New Vector3(0, 0, Height))

            tV(idy) = New Vector2(tX, 1)
            tV(NextIndex) = New Vector2(tX, 0)
            idy += 2
        Next

        tVerts = tV
        Return V
    End Function

    Public Function BuildCylinderStrip(ByVal Position As Vector3, ByVal Radius As Single, ByVal StripCount As Integer, ByVal StackCount As Integer, ByVal Height As Single, ByRef tVerts() As Vector2) As Vector3()
        Dim idx, idy, idz As Integer
        Dim V() As Vector3
        Dim tV() As Vector2
        Dim RP As Vector3 '= Vector3.Add(Position, New Vector3(Radius, 0, 0))

        If StripCount < 3 Then Throw New ArgumentOutOfRangeException("StripCount")
        If StackCount < 1 Then Throw New ArgumentOutOfRangeException("StripCount")

        ReDim V(StripCount * 2 + 1 * StackCount)
        ReDim tV(StripCount * 2 + 1 * StackCount)

        For idz = 0 To StackCount - 1
            RP = Vector3.Add(Position, New Vector3(Radius, 0, 0))
            idy = 0
            For idx = 1 To StripCount + 1
                Dim tX As Single
                Dim NextIndex As Integer

                tX = CSng(idx / StripCount)
                NextIndex = idy + 1

                RP = MathHelpers.RotateAroundAxis(Position, RP, New Vector3(0, 0, DegreeToRadian(CSng(360 / StripCount))))
                V(idy) = RP
                V(NextIndex) = Vector3.Add(V(idy), New Vector3(0, 0, Height))

                V(idy).Z += idz * Height
                V(NextIndex).Z += (idz * Height) ' + Height

                tV(idy) = New Vector2(tX, 1)
                tV(NextIndex) = New Vector2(tX, 0)
                idy += 2
            Next

            tVerts = tV
        Next
        Return V
    End Function
#End Region

End Module
