﻿Public Class Light
    Public Light As New Direct3D.Light
    Friend mstrName As String
    Friend mblnEnabled As Boolean = False
    'Friend mobjLights As Collections.Generic.Dictionary(Of String, Light)
    'Friend mobjActiveLights As Collections.Generic.Dictionary(Of String, Light)


    Public Function IsActive() As Boolean
        Return mblnEnabled
    End Function

    Public Function SetEnabled(ByVal Value As Boolean) As Boolean
        If mstrName Is Nothing Then Return False
        If mobjEnabledLights Is Nothing Then Return False
        If mblnEnabled = Value Then Return True


        ' check if it is already in the list
        If Value Then
            ' we just return true if already added as active light
            If mobjEnabledLights.ContainsKey(mstrName) Then
                Return True
            Else
                ' if we have not initilized the light yet return false
                If mblnHasBeenInit = False Then Return False
                ' light is not yet in the active list so check if there is any room for it
                If mobjEnabledLights.Count >= mobjDevice.DeviceCaps.MaxActiveLights Then
                    ' no room for the light
                    SpecifyError("SetAsActive", "No availible slot for light!")
                    Return False
                Else
                    ' there is room for it so add it
                    mobjEnabledLights.Add(mstrName, Me)
                End If
            End If
        Else
            ' remove from active list if already added
            If mobjEnabledLights.ContainsKey(mstrName) Then
                mobjEnabledLights.Remove(mstrName)
            End If
        End If

        mblnEnabled = Value
        Return True
    End Function

    Public Property Name() As String
        Get
            Return mstrName
        End Get
        Set(ByVal value As String)
            If value Is Nothing Then Throw New ArgumentNullException("Value")
            If value.Trim = String.Empty Then Throw New ArgumentNullException("Value")
            If mobjLights Is Nothing = False Then
                If mstrName Is Nothing = False Then
                    If mobjLights.ContainsKey(mstrName) Then mobjLights.Remove(mstrName)
                    If mblnEnabled And mobjEnabledLights.ContainsKey(mstrName) Then mobjEnabledLights.Remove(mstrName)
                End If
            End If
            mstrName = value
            If mobjLights Is Nothing = False Then
                mobjLights.Add(mstrName, Me)
                If mblnEnabled Then mobjEnabledLights.Remove(mstrName)
            End If
        End Set
    End Property
End Class

Public Module Lights

    Friend mobjLights As New Collections.Hashtable() ' .Generic.Dictionary(Of String, Light)
    Friend mobjEnabledLights As New Collections.Hashtable() ' .Generic.Dictionary(Of String, Light)
    Friend mstrCurrentLight As String = String.Empty


    Public Function EnableLighting(ByVal LightingOn As Boolean) As Boolean
        If mblnHasBeenInit = False Then Return False
        mobjDevice.RenderState.Lighting = LightingOn
        Return True
    End Function

    Public Function IsLightingEnabled() As Boolean
        If mblnHasBeenInit = False Then Return False
        Return mobjDevice.RenderState.Lighting
        Return True
    End Function

    Public Function ChangeLightName(ByRef Name As String, ByVal Value As String) As Boolean
        If mobjLights.ContainsKey(Name) Then Return False
        If mobjLights.ContainsKey(Value) Then Return False
        Dim Item As Light
        Item = DirectCast(mobjLights(Name), Light)
        mobjLights.Remove(Name)
        mobjLights.Add(Value, Item)

        Return True
    End Function

    Public Function GetLightNames(ByRef Names() As String) As Boolean
        Erase Names
        ReDim Names(mobjLights.Count - 1)
        mobjLights.Keys.CopyTo(Names, 0)

        Return True
    End Function

    Public Function ApplyEnabledLights() As Boolean
        If mblnHasBeenInit = False Then Return False
        Dim Index As Integer = 0
        Dim L As DictionaryEntry
        For Each L In mobjEnabledLights  ' With mobjEnabledLights.GetEnumerator
            Dim Tmp As Light
            Tmp = DirectCast(L.Value, Light)
            'While .MoveNext
            With mobjDevice.Lights(Index)
                .Ambient = Tmp.Light.Ambient
                .Attenuation0 = Tmp.Light.Attenuation0
                .Attenuation1 = Tmp.Light.Attenuation1
                .Attenuation2 = Tmp.Light.Attenuation2
                .Deferred = Tmp.Light.Deferred
                .Diffuse = Tmp.Light.Diffuse
                .Direction = Tmp.Light.Direction
                .Falloff = Tmp.Light.Falloff
                .InnerConeAngle = Tmp.Light.InnerConeAngle
                .OuterConeAngle = Tmp.Light.OuterConeAngle
                .Position = Tmp.Light.Position
                .Range = Tmp.Light.Range
                .Specular = Tmp.Light.Specular
                .Type = Tmp.Light.Type
                .Enabled = True
                .Commit()
            End With

            'mobjDevice.Lights(Index).FromLight(L.Light)
            '  mobjDevice.Lights(Index).Enabled = True
            '  mobjDevice.Lights(Index).Commit()
            Index += 1
            'End While
        Next
        Return True
    End Function

    Public Function SetLightEnabled(ByVal Name As String, ByVal Value As Boolean) As Boolean
        If mobjLights.ContainsKey(Name) = False Then Return False
        DirectCast(mobjLights(Name), Light).mblnEnabled = Value
        Return True
    End Function

    Public Sub RemoveAllLights(Optional ByVal RemoveEnabledLights As Boolean = False)
        mobjLights.Clear()
        If RemoveEnabledLights Then RemoveAllEnabledLights()
    End Sub

    Public Sub RemoveAllEnabledLights()
        mobjEnabledLights.Clear()
    End Sub

    Public Function GetNumberOfLights() As Integer
        Return mobjLights.Count
    End Function

    Public Function GetNumberOfEnabledLights() As Integer
        Return mobjEnabledLights.Count
    End Function

    Public Function GetCurrentLight() As Light
        If mobjLights.ContainsKey(mstrCurrentLight) = False Then Return Nothing
        Return DirectCast(mobjLights(mstrCurrentLight), Light)
    End Function

    Public Function SetAmbientLighting(ByVal Color As Color) As Boolean
        If mblnHasBeenInit = False Then Return False
        mobjDevice.RenderState.Ambient = Color
        Return True
    End Function

    Public Function GetAmbientLighting(ByRef Value As Color) As Boolean
        If mblnHasBeenInit = False Then Return False
        Value = mobjDevice.RenderState.Ambient
        Return True
    End Function

    Public Function SetCurrentLight(ByVal Name As String) As Boolean
        If Name Is Nothing Then Return False
        If mobjLights.ContainsKey(Name) = False Then Return False
        mstrCurrentLight = Name
        Return True
    End Function

    Public Function RemoveLight(ByVal Name As String) As Boolean
        If Name Is Nothing Then Return False
        If mobjLights.ContainsKey(Name) = False Then Return False
        If Name = mstrCurrentLight Then mstrCurrentLight = String.Empty
        If mobjEnabledLights.ContainsKey(Name) Then mobjEnabledLights.Remove(Name)
        '  mobjLights(Name).mobjLights = Nothing
        mobjLights.Remove(Name)
        Return True
    End Function

    Public Function AddLight(ByVal Name As String, ByVal Value As Direct3D.Light, Optional ByVal Active As Boolean = False) As Boolean
        If Name Is Nothing Then Return False
        If mobjLights.ContainsKey(Name) Then Return False
        ' Value.mobjLights = mobjLights
        'Value.mobjActiveLights = mobjActiveLights
        Dim V As New Light()
        V.mstrName = Name
        V.Light = Value
        V.mblnEnabled = Active
        If V.mblnEnabled Then mobjEnabledLights.Add(Name, V)
        mobjLights.Add(Name, V)
        'Value.mblnActive = Not Value.mblnActive
        'Return Value.SetActive(Not Value.mblnActive)
        Return True
    End Function

    Public Function AddLight(ByVal Name As String, ByVal Value As Light) As Boolean
        If Name Is Nothing Then Return False
        If mobjLights.ContainsKey(Name) Then Return False
        ' Value.mobjLights = mobjLights
        'Value.mobjActiveLights = mobjActiveLights
        Value.mstrName = Name
        If Value.mblnEnabled Then mobjEnabledLights.Add(Name, Value)
        mobjLights.Add(Name, Value)
        'Value.mblnActive = Not Value.mblnActive
        'Return Value.SetActive(Not Value.mblnActive)
        Return True
    End Function

    Public Function SetLight(ByVal Name As String, ByVal Value As Light) As Boolean
        If Name Is Nothing Then Return False
        If mobjLights.ContainsKey(Name) = False Then Return False
        mobjLights(Name) = Value
        Return True
    End Function

    Public Function GetLight(ByVal Name As String, ByRef Value As Light) As Boolean
        If Name Is Nothing Then Return False
        If mobjLights.ContainsKey(Name) = False Then Return False
        Value = DirectCast(mobjLights(Name), Light)
        Return True
    End Function
End Module
