﻿
Public NotInheritable Class Mesh
    Implements IDisposable

    Friend mstrName As String
    Protected Friend mobjMaterials As New Collections.Specialized.StringCollection()
    Protected Friend mobjTextures As New Collections.Specialized.StringCollection()
    'Friend mobjMaterialsCollectionReference As Collections.Generic.Dictionary(Of String, Mesh)
    Protected Friend mobjMesh As Direct3D.Mesh

    Public Property Mesh() As Direct3D.Mesh
        Get
            Return mobjMesh
        End Get
        Set(ByVal value As Direct3D.Mesh)
            mobjMesh = value
        End Set
    End Property

    Public ReadOnly Property Materials() As Collections.Specialized.StringCollection
        Get
            Return mobjMaterials
        End Get
    End Property

    Public ReadOnly Property Textures() As Collections.Specialized.StringCollection
        Get
            Return mobjTextures
        End Get
    End Property

    Public Property Name() As String
        Get
            Return mstrName
        End Get
        Set(ByVal value As String)
            If value Is Nothing Then Throw New ArgumentNullException("Value")
            If value.Trim = String.Empty Then Throw New ArgumentNullException("Value")
            If mobjMeshes Is Nothing = False Then
                If mstrName Is Nothing = False Then
                    If mobjMeshes.ContainsKey(mstrName) Then mobjMeshes.Remove(mstrName)
                End If
            End If
            mstrName = value
            If mobjMeshes Is Nothing = False Then
                mobjMeshes.Add(mstrName, Me)
            End If
        End Set
    End Property

#Region " IDisposable Support "
    Private disposed As Boolean = False

    ' IDisposable
    Private Overloads Sub Dispose(ByVal disposing As Boolean)
        If Not Me.disposed Then
            If disposing Then
                If Me.mobjMesh Is Nothing = False Then Me.mobjMesh.Dispose()
                Me.mobjMesh = Nothing
                Me.mobjMaterials.Clear()
                Me.mobjTextures.Clear()
                Me.mobjMaterials = Nothing
                Me.mobjTextures = Nothing
            End If

            ' TODO: put code to free unmanaged resources here
        End If
        Me.disposed = True
    End Sub

    ' This code added by Visual Basic to correctly implement the disposable pattern.
    Public Overloads Sub Dispose() Implements IDisposable.Dispose
        ' Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub

    Protected Overrides Sub Finalize()
        ' Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
        Dispose(False)
        MyBase.Finalize()
    End Sub
#End Region

End Class

Public Module Meshes

    Friend mobjMeshes As New Collections.Hashtable() ' .Generic.Dictionary(Of String, Mesh)

    Public Function ChangeMeshName(ByRef Name As String, ByVal Value As String) As Boolean
        If mobjMeshes.ContainsKey(Name) Then Return False
        If mobjMeshes.ContainsKey(Value) Then Return False
        Dim Item As Mesh
        Item = DirectCast(mobjMeshes(Name), Mesh)
        mobjMeshes.Remove(Name)
        mobjMeshes.Add(Value, Item)

        Return True
    End Function

    Public Function GetMeshNames(ByRef Names() As String) As Boolean
        Erase Names
        ReDim Names(mobjMeshes.Count - 1)
        mobjMeshes.Keys.CopyTo(Names, 0)

        Return True
    End Function

    Public Function GetNumberOfMeshs() As Integer
        Return mobjMeshes.Count
    End Function

    Public Sub RemoveAllMeshes(Optional ByVal CallDispose As Boolean = True)
        If CallDispose Then
            With mobjMeshes.GetEnumerator
                While .MoveNext
                    DirectCast(.Value, Mesh).Dispose()
                End While
            End With
        End If
        mobjMeshes.Clear()
    End Sub

    Public Function RemoveMesh(ByVal Name As String) As Boolean
        If mobjMeshes.ContainsKey(Name) = False Then Return False
        ' mobjMeshes(Name).mobjMaterialsCollectionReference = Nothing
        mobjMeshes.Remove(Name)
        Return True
    End Function

    Public Function AddMesh(ByVal Name As String, ByVal Value As Mesh) As Boolean
        If mobjMeshes.ContainsKey(Name) Then Return False
        ' Value.mobjMaterialsCollectionReference = mobjMeshes
        Value.mstrName = Name
        mobjMeshes.Add(Name, Value)
        Return True
    End Function

    Public Function GetMesh(ByVal Name As String, ByRef Value As Mesh) As Boolean
        If mobjMeshes.ContainsKey(Name) = False Then Return False
        Value = DirectCast(mobjMeshes(Name), Mesh)
        Return True
    End Function

    Public Function GetMeshMaterial(ByVal Name As String, ByVal Index As Integer, ByRef Value As String) As Boolean
        If mobjMeshes.ContainsKey(Name) = False Then Return False
        With DirectCast(mobjMeshes(Name), Mesh)
            Dim Mat As String
            Try
                Mat = .Materials(Index)
            Catch
                Return False
            End Try
            Value = .Materials(Index)
        End With
        Return True
    End Function

    Public Function SetMeshMaterial(ByVal Name As String, ByVal Index As Integer, ByVal Value As String) As Boolean
        If mobjMeshes.ContainsKey(Name) = False Then Return False
        With DirectCast(mobjMeshes(Name), Mesh)
            Dim Mat As String
            Try
                Mat = .Materials(Index)
            Catch
                Return False
            End Try
            .Materials(Index) = Value
        End With
        Return True
    End Function

    Public Function GetMeshTexture(ByVal Name As String, ByVal Index As Integer, ByRef Value As String) As Boolean
        If mobjMeshes.ContainsKey(Name) = False Then Return False
        With DirectCast(mobjMeshes(Name), Mesh)
            Dim Mat As String
            Try
                Mat = .Textures(Index)
            Catch
                Return False
            End Try
            Value = .Textures(Index)
        End With
        Return True
    End Function

    Public Function SetMeshTexture(ByVal Name As String, ByVal Index As Integer, ByVal Value As String) As Boolean
        If mobjMeshes.ContainsKey(Name) = False Then Return False
        With DirectCast(mobjMeshes(Name), Mesh)
            Dim Mat As String
            Try
                Mat = .Textures(Index)
            Catch
                Return False
            End Try
            .Textures(Index) = Value
        End With
        Return True
    End Function
End Module

