﻿Public Module Textures

    Friend mobjTextures As New Collections.Hashtable() ' .Generic.Dictionary(Of String, Direct3D.Texture)


    Public Function ApplyTexture(ByVal Stage As Integer, ByVal Name As String) As Boolean
        If mblnHasBeenInit = False Then Return False
        If Name = NOTEXTURE Or Name Is Nothing Then
            mobjDevice.SetTexture(Stage, Nothing)
            Return True
        End If
        If mobjTextures.ContainsKey(Name) = False Then Return False

        mobjDevice.SetTexture(Stage, DirectCast(mobjTextures(Name), Direct3D.Texture))
        Return True
    End Function

    Public Function ChangeTextureName(ByRef Name As String, ByVal Value As String) As Boolean
        If mobjTextures.ContainsKey(Name) Then Return False
        If mobjTextures.ContainsKey(Value) Then Return False
        Dim Item As Direct3D.Texture
        Item = DirectCast(mobjTextures(Name), Direct3D.Texture)
        mobjTextures.Remove(Name)
        mobjTextures.Add(Value, Item)

        Return True
    End Function

    Public Function GetTextureNames(ByRef Names() As String) As Boolean
        Erase Names
        ReDim Names(mobjTextures.Count - 1)
        mobjTextures.Keys.CopyTo(Names, 0)

        Return True
    End Function

#Region " LoadTexture overloads ... "
    Public Function LoadTexture(ByVal Device As Direct3D.Device, ByVal File As String) As Direct3D.Texture
        Return Direct3D.TextureLoader.FromFile(Device, File, 0, 0, 0, 0, _
                                               Direct3D.Format.Unknown, Direct3D.Pool.Managed, _
                                               Direct3D.Filter.None, Direct3D.Filter.None, _
                                               Color.Black.ToArgb)
    End Function

    Public Function LoadTexture(ByVal Device As Direct3D.Device, ByVal File As String, ByVal ColorKey As Color) As Direct3D.Texture
        Return Direct3D.TextureLoader.FromFile(Device, File, 0, 0, 0, 0, _
                                               Direct3D.Format.Unknown, Direct3D.Pool.Managed, _
                                               Direct3D.Filter.None, Direct3D.Filter.None, _
                                               ColorKey.ToArgb)
    End Function

    Public Function LoadTexture(ByVal Device As Direct3D.Device, ByVal File As String, ByVal Filter As Direct3D.Filter) As Direct3D.Texture
        Return Direct3D.TextureLoader.FromFile(Device, File, 0, 0, 0, 0, _
                                               Direct3D.Format.Unknown, Direct3D.Pool.Managed, _
                                               Filter, Filter, _
                                               Color.Black.ToArgb)
    End Function

    Public Function LoadTexture(ByVal Device As Direct3D.Device, ByVal File As String, ByVal Filter As Direct3D.Filter, ByVal ColorKey As Color) As Direct3D.Texture
        Return Direct3D.TextureLoader.FromFile(Device, File, 0, 0, 0, 0, _
                                               Direct3D.Format.Unknown, Direct3D.Pool.Managed, _
                                               Filter, Filter, _
                                               Color.Black.ToArgb)
    End Function

    Public Function LoadTexture(ByVal Device As Direct3D.Device, ByVal File As String, ByRef Info As Direct3D.ImageInformation) As Direct3D.Texture
        Return Direct3D.TextureLoader.FromFile(Device, File, 0, 0, 0, 0, _
                                               Direct3D.Format.Unknown, Direct3D.Pool.Managed, _
                                               Direct3D.Filter.None, Direct3D.Filter.None, _
                                               Color.Black.ToArgb)
        Info = Direct3D.TextureLoader.ImageInformationFromFile(File)
    End Function

    Public Function LoadTexture(ByVal Device As Direct3D.Device, ByVal File As String, ByVal ColorKey As Color, ByRef Info As Direct3D.ImageInformation) As Direct3D.Texture
        Return Direct3D.TextureLoader.FromFile(Device, File, 0, 0, 0, 0, _
                                               Direct3D.Format.Unknown, Direct3D.Pool.Managed, _
                                               Direct3D.Filter.None, Direct3D.Filter.None, _
                                               ColorKey.ToArgb)
        Info = Direct3D.TextureLoader.ImageInformationFromFile(File)
    End Function

    Public Function LoadTexture(ByVal Device As Direct3D.Device, ByVal File As String, ByVal Filter As Direct3D.Filter, ByRef Info As Direct3D.ImageInformation) As Direct3D.Texture
        Return Direct3D.TextureLoader.FromFile(Device, File, 0, 0, 0, 0, _
                                               Direct3D.Format.Unknown, Direct3D.Pool.Managed, _
                                               Filter, Filter, _
                                               Color.Black.ToArgb)
        Info = Direct3D.TextureLoader.ImageInformationFromFile(File)
    End Function

    Public Function LoadTexture(ByVal Device As Direct3D.Device, ByVal File As String, ByVal Filter As Direct3D.Filter, ByVal ColorKey As Color, ByRef Info As Direct3D.ImageInformation) As Direct3D.Texture
        Return Direct3D.TextureLoader.FromFile(Device, File, 0, 0, 0, 0, _
                                               Direct3D.Format.Unknown, Direct3D.Pool.Managed, _
                                               Filter, Filter, _
                                               Color.Black.ToArgb)
        Info = Direct3D.TextureLoader.ImageInformationFromFile(File)
    End Function

    Public Function LoadTexture(ByVal Filename As String, Optional ByVal MipLevels As Integer = 0, _
                                Optional ByVal Filter As Direct3D.Filter = Direct3D.Filter.Linear, _
                                Optional ByVal MipFilter As Direct3D.Filter = Direct3D.Filter.Linear, _
                                Optional ByVal ColorKey As Integer = 0) As Direct3D.Texture

        Dim Tex As Direct3D.Texture
        Try
            Dim fInfo As Direct3D.ImageInformation
            fInfo = Direct3D.TextureLoader.ImageInformationFromFile(Filename)
            Tex = Direct3D.TextureLoader.FromFile(mobjDevice, Filename, fInfo.Width, fInfo.Height, MipLevels, 0, fInfo.Format, _
                                                  Direct3D.Pool.Managed, Filter, MipFilter, ColorKey)
        Catch
            Return Nothing
        End Try

        Return Tex
    End Function

    Public Function LoadTexture(ByVal Stream As IO.Stream, Optional ByVal MipLevels As Integer = 0, _
                                Optional ByVal Filter As Direct3D.Filter = Direct3D.Filter.Linear, _
                                Optional ByVal MipFilter As Direct3D.Filter = Direct3D.Filter.Linear, _
                                Optional ByVal ColorKey As Integer = 0) As Direct3D.Texture

        Dim Tex As Direct3D.Texture
        Try
            Tex = Direct3D.TextureLoader.FromStream(mobjDevice, Stream, 0, 0, MipLevels, 0, Direct3D.Format.Unknown, _
                                                  Direct3D.Pool.Managed, Filter, MipFilter, ColorKey)
        Catch
            Return Nothing
        End Try

        Return Tex
    End Function
#End Region

    Public Function GetNumberOfTextures() As Integer
        Return mobjTextures.Count
    End Function

    Public Sub RemoveAllTextures(Optional ByVal CallDispose As Boolean = True)
        If CallDispose Then
            With mobjTextures.GetEnumerator
                ' .Reset()
                While .MoveNext
                    If .Value Is Nothing = False Then
                        DirectCast(.Value, Direct3D.Texture).Dispose()
                    End If
                End While
            End With

        End If
        mobjTextures.Clear()
    End Sub

    Public Function RemoveTexture(ByVal Name As String) As Boolean
        If mobjTextures.ContainsKey(Name) = False Then Return False
        mobjTextures.Remove(Name)
        Return True
    End Function

    Public Function AddTexture(ByVal Name As String, ByVal Value As Direct3D.Texture) As Boolean
        If Name = NOTEXTURE Then Return False
        If mobjTextures.ContainsKey(Name) Then Return False
        mobjTextures.Add(Name, Value)
        Return True
    End Function

    Public Function CreateRenderToSurfaceTexture(ByVal Name As String, ByVal Width As Integer, ByVal Height As Integer, _
    Optional ByVal NumLevels As Integer = 0) As Boolean
        If mblnHasBeenInit = False Then Return False
        If NumLevels < 0 Then Return False
        If Name = NOTEXTURE Then Return False
        If mobjTextures.ContainsKey(Name) Then Return False

        Dim Value As Direct3D.Texture
        Try
            Value = New Direct3D.Texture(mobjDevice, Width, Height, NumLevels, Direct3D.Usage.RenderTarget, _
                                         mobjDevice.DisplayMode.Format, Direct3D.Pool.Default)
        Catch ex As Exception
            SpecifyError(ex.Source, ex.Message)
            Return True
        End Try
        mobjTextures.Add(Name, Value)
        Return True
    End Function

    Public Function TextureExists(ByVal Name As String) As Boolean
        Return mobjTextures.ContainsKey(Name)
    End Function

    Public Function GetTexture(ByVal Name As String, ByRef Value As Direct3D.Texture) As Boolean
        If mobjTextures.ContainsKey(Name) = False Then Return False
        Value = DirectCast(mobjTextures(Name), Direct3D.Texture)
        Return True
    End Function
End Module
