'Created by: X
'Copyright: Created by: X
'E-Mail: mailto:createdbyx@gmail.com
'Web: http://www.createdbyx.com/
'Date: Monday, January 24, 2005
'=============================================

'High resolution timer references ...
'http://support.microsoft.com/default.aspx?scid=http://support.microsoft.com:80/support/kb/articles/Q172/3/38.asp&NoWebContent=1
'http://msdn.microsoft.com/library/default.asp?url=/library/en-us/winui/WinUI/WindowsUserInterface/Windowing/Timers/TimerReference/TimerFunctions/QueryPerformanceCounter.asp
'http://msdn.microsoft.com/library/default.asp?url=/library/en-us/winui/WinUI/WindowsUserInterface/Windowing/Timers/TimerReference/TimerFunctions/QueryPerformanceFrequency.asp
'=============================================

Imports System.ComponentModel


''' <summary>
''' The GameTimer class is used to represent time.
''' </summary>
Public Class HiResolutionTimer
    Inherits Timer


    Private Declare Auto Function QueryPerformanceCounter Lib "Kernel32.dll" (ByRef lpPerformanceCount As Long) As Boolean
    Private Declare Auto Function QueryPerformanceFrequency Lib "Kernel32.dll" (ByRef lpFrequency As Long) As Boolean


    ''' <summary>
    ''' Creates a new instance of the class.
    ''' </summary>
    Public Sub New()
        Me.CaptureTime()
        Me.mlngStartTime = mlngCapturedTime
        Me.mlngLastCapturedTime = Me.mlngStartTime
        Me.mlngFrequency = 0
        If QueryPerformanceFrequency(mlngFrequency) = False Then
            Throw New Win32Exception(0, "High speed timer not availible on this system!") ' // timer not supported
        End If
    End Sub

    Public Overrides Function CaptureTime() As Long
        mlngLastCapturedTime = mlngCapturedTime
        QueryPerformanceCounter(mlngCapturedTime)
        Return mlngCapturedTime
    End Function

    Public Overrides Function GetTimeInfo(ByVal Info As TimeInformation) As Long
        Select Case Info
            Case TimeInformation.CapturedTime
                Return mlngCapturedTime

            Case TimeInformation.Duration
                Dim T As Long
                QueryPerformanceCounter(T)
                Return T - mlngCapturedTime

            Case TimeInformation.DurationSincePreviousCapture
                Dim T As Long
                QueryPerformanceCounter(T)
                Return T - mlngLastCapturedTime

            Case TimeInformation.Frequency
                Return mlngFrequency

            Case TimeInformation.PreviousCapturedTime
                Return Me.mlngLastCapturedTime

            Case TimeInformation.SystemTime
                Dim T As Long
                QueryPerformanceCounter(T)
                Return T

            Case TimeInformation.StartTime
                Return mlngStartTime

            Case TimeInformation.DurationSinceStart
                Dim T As Long
                QueryPerformanceCounter(T)
                Return T - mlngStartTime
        End Select
    End Function

    Public Overrides Function Clone() As Object
        Dim Tmp As New HiResolutionTimer()
        Tmp.mlngCapturedTime = Me.mlngCapturedTime
        Tmp.mlngFrequency = Me.mlngFrequency
        Tmp.mlngLastCapturedTime = Me.mlngLastCapturedTime
        Tmp.mlngStartTime = Me.mlngStartTime
        Return Tmp
    End Function
End Class
