﻿Public Class TimedEvents
    Implements IDisposable

    Private Structure IntervalStruct
        Public PreviousTime As Long
        Public Interval As Single
        Public ID As Guid
    End Structure

    Public Event IntervalEllapsed(ByVal ID As Guid)

    Protected Friend mintCapturedTime As Long
    Protected Friend mobjTimer As ITimer
    Private mobjItems As New Collections.ArrayList()

    Public Property Timer() As ITimer
        Get
            Return mobjTimer
        End Get
        Set(ByVal Value As ITimer)
            mobjTimer = Value
        End Set
    End Property

    Public Function AddInterval(ByVal Interval As Single) As Guid
        If Me.disposed Then Exit Function
        Dim Tmp As New IntervalStruct()
        Tmp.PreviousTime = Me.mintCapturedTime
        Tmp.Interval = Interval
        Tmp.ID = Guid.NewGuid
        Me.mobjItems.Add(Tmp)
        Return Tmp.ID
    End Function

    Public Function RemoveInterval(ByVal ID As Guid) As Boolean
        If Me.disposed Then Exit Function
        Dim T As IntervalStruct
        For Each T In Me.mobjItems
            If Guid.Equals(T.ID, ID) Then
                Dim Index As Integer
                Index = Me.mobjItems.IndexOf(T)
                Me.mobjItems.RemoveAt(Index)
                Return True
            End If
        Next
        Return False
    End Function

    Public Sub Clear()
        If Me.disposed Then Exit Sub
        Me.mobjItems.Clear()
    End Sub

    Public Overridable Sub Update()
        If Me.disposed Then Exit Sub
        ' get the currwnt time
        Me.mintCapturedTime = mobjTimer.GetTimeInfo(TimeInformation.SystemTime)

        ' cycle through each item check if time to fire event
        Dim T As IntervalStruct
        For Each T In Me.mobjItems
            If Me.mintCapturedTime > T.PreviousTime + (Me.mobjTimer.GetTimeInfo(TimeInformation.Frequency) / T.Interval) Then
                Try
                    RaiseEvent IntervalEllapsed(T.ID)
                Catch ex As Exception

                End Try
                T.PreviousTime = Me.mintCapturedTime
            End If
        Next
    End Sub

    Public ReadOnly Property CapturedTime() As Long
        Get
            Return mintCapturedTime
        End Get
    End Property

    Public Sub New(ByVal Timer As ITimer)
        Me.mobjTimer = Timer
    End Sub

    Public Sub New()
        Me.mobjTimer = New Timer()
    End Sub

#Region " IDisposable Support "
    Private disposed As Boolean = False

    ' IDisposable
    Private Overloads Sub Dispose(ByVal disposing As Boolean)
        If Not Me.disposed Then
            If disposing Then
                Me.mobjTimer = Nothing
                Me.mobjItems.Clear()
                Me.mobjItems = Nothing
            End If

            ' TODO: put code to free unmanaged resources here
        End If
        Me.disposed = True
    End Sub

    ' This code added by Visual Basic to correctly implement the disposable pattern.
    Public Overloads Sub Dispose() Implements IDisposable.Dispose
        ' Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub

    Protected Overrides Sub Finalize()
        ' Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
        Dispose(False)
        MyBase.Finalize()
    End Sub
#End Region
End Class
