﻿Public NotInheritable Class VertexBuffer
    Implements IDisposable

    Friend mstrName As String
    Public Buffer As Direct3D.VertexBuffer
    Public PrimitiveType As Direct3D.PrimitiveType = Direct3D.PrimitiveType.TriangleList
    Public PrimitiveCount As Integer = 0
    Public StartVertex As Integer = 0
    Public StreamIndex As Integer = 0
    Public StreamOffset As Integer = 0

    ' Friend mobjVBCollectionReference As Collections.Generic.Dictionary(Of String, VertexBuffer)

    Public Property Name() As String
        Get
            Return mstrName
        End Get
        Set(ByVal value As String)
            If value Is Nothing Then Throw New ArgumentNullException("Value")
            If value.Trim = String.Empty Then Throw New ArgumentNullException("Value")
            If mobjVertexBuffers Is Nothing = False Then
                If mstrName Is Nothing = False Then
                    If mobjVertexBuffers.ContainsKey(mstrName) Then mobjVertexBuffers.Remove(mstrName)
                End If
            End If
            mstrName = value
            If mobjVertexBuffers Is Nothing = False Then
                mobjVertexBuffers.Add(mstrName, Me)
            End If
        End Set
    End Property

#Region " IDisposable Support "
    Private disposed As Boolean = False

    ' IDisposable
    Private Overloads Sub Dispose(ByVal disposing As Boolean)
        If Not Me.disposed Then
            If disposing Then
                If Me.Buffer Is Nothing = False Then Me.Buffer.Dispose()
                Me.Buffer = Nothing
            End If

            ' TODO: put code to free unmanaged resources here
        End If
        Me.disposed = True
    End Sub

    ' This code added by Visual Basic to correctly implement the disposable pattern.
    Public Overloads Sub Dispose() Implements IDisposable.Dispose
        ' Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub

    Protected Overrides Sub Finalize()
        ' Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
        Dispose(False)
        MyBase.Finalize()
    End Sub
#End Region

End Class

Public Module VertexBuffers

    Friend mobjVertexBuffers As New Collections.Hashtable() ' .Generic.Dictionary(Of String, VertexBuffer)

    Public Function SetVertexBufferPrimitiveType(ByRef Name As String, ByVal Value As Direct3D.PrimitiveType) As Boolean
        If mobjVertexBuffers.ContainsKey(Name) Then Return False
        Dim Item As VertexBuffer
        Item = DirectCast(mobjVertexBuffers(Name), VertexBuffer)
        Item.PrimitiveType = Value

        Return True
    End Function

    Public Function SetVertexBufferStreamOffset(ByRef Name As String, ByVal Value As Integer) As Boolean
        If mobjVertexBuffers.ContainsKey(Name) Then Return False
        Dim Item As VertexBuffer
        Item = DirectCast(mobjVertexBuffers(Name), VertexBuffer)
        Item.StreamOffset = Value

        Return True
    End Function

    Public Function SetVertexBufferStreamIndex(ByRef Name As String, ByVal Value As Integer) As Boolean
        If mobjVertexBuffers.ContainsKey(Name) Then Return False
        Dim Item As VertexBuffer
        Item = DirectCast(mobjVertexBuffers(Name), VertexBuffer)
        Item.StreamIndex = Value

        Return True
    End Function

    Public Function SetVertexBufferPrimitiveCount(ByRef Name As String, ByVal Value As Integer) As Boolean
        If mobjVertexBuffers.ContainsKey(Name) Then Return False
        Dim Item As VertexBuffer
        Item = DirectCast(mobjVertexBuffers(Name), VertexBuffer)
        Item.PrimitiveCount = Value

        Return True
    End Function

    Public Function SetVertexBufferStartVertex(ByRef Name As String, ByVal Value As Integer) As Boolean
        If mobjVertexBuffers.ContainsKey(Name) Then Return False
        Dim Item As VertexBuffer
        Item = DirectCast(mobjVertexBuffers(Name), VertexBuffer)
        Item.StartVertex = Value

        Return True
    End Function

    Public Function ChangeVertexBufferName(ByRef Name As String, ByVal Value As String) As Boolean
        If mobjVertexBuffers.ContainsKey(Name) Then Return False
        If mobjVertexBuffers.ContainsKey(Value) Then Return False
        Dim Item As VertexBuffer
        Item = DirectCast(mobjVertexBuffers(Name), VertexBuffer)
        mobjVertexBuffers.Remove(Name)
        mobjVertexBuffers.Add(Value, Item)

        Return True
    End Function

    Public Function GetVertexBufferNames(ByRef Names() As String) As Boolean
        Erase Names
        ReDim Names(mobjVertexBuffers.Count - 1)
        mobjVertexBuffers.Keys.CopyTo(Names, 0)

        Return True
    End Function

    Public Function GetNumberOfVertexBuffers() As Integer
        Return mobjVertexBuffers.Count
    End Function

    Public Sub RemoveAllVertexBuffers(Optional ByVal CallDispose As Boolean = True)
        If CallDispose Then
            With mobjVertexBuffers.GetEnumerator
                While .MoveNext
                    DirectCast(.Value, VertexBuffer).Dispose()
                End While
            End With
        End If
        mobjVertexBuffers.Clear()
    End Sub

    Public Function RemoveVertexBuffer(ByVal Name As String) As Boolean
        If Name Is Nothing Then Return False
        If mobjVertexBuffers.ContainsKey(Name) = False Then Return False
        'mobjVertexBuffers(Name).mobjVBCollectionReference = Nothing
        mobjVertexBuffers.Remove(Name)
        Return True
    End Function

    Public Function AddVertexBuffer(ByVal Name As String, ByVal Value As VertexBuffer) As Boolean
        If Name Is Nothing Then Return False
        If mobjVertexBuffers.ContainsKey(Name) Then Return False
        ' Value.mobjVBCollectionReference = mobjVertexBuffers
        Value.mstrName = Name
        mobjVertexBuffers.Add(Name, Value)
        Return True
    End Function

    Public Function VertexBufferExists(ByVal Name As String) As Boolean
         Return mobjVertexBuffers.ContainsKey(Name)
    End Function

    Public Function ApplyVertexBuffer(ByVal Name As String, ByVal Stream As Integer, ByVal Value As VertexBuffer, Optional ByVal OffsetInBytes As Integer = 0) As Boolean
        If mobjVertexBuffers.ContainsKey(Name) = False Then Return False
        mobjDevice.SetStreamSource(Stream, Value.Buffer, OffsetInBytes)
        Return True
    End Function

    Public Function ApplyVertexBuffer(ByVal Name As String) As Boolean
        If mobjVertexBuffers.ContainsKey(Name) = False Then Return False
        Dim Item As VertexBuffer
        Item = DirectCast(mobjVertexBuffers(Name), VertexBuffer)
        Try
            mobjDevice.SetStreamSource(Item.StreamIndex, Item.Buffer, Item.StreamOffset)
        Catch
            Return False
        End Try
        Return True
    End Function

    Public Function SetVertexBuffer(ByVal Name As String, ByVal Value As VertexBuffer) As Boolean
        If mobjVertexBuffers.ContainsKey(Name) = False Then Return False
        mobjVertexBuffers(Name) = Value
        Return True
    End Function

    Public Function GetVertexBuffer(ByVal Name As String, ByRef Value As VertexBuffer) As Boolean
        If mobjVertexBuffers.ContainsKey(Name) = False Then Return False
        Value = DirectCast(mobjVertexBuffers(Name), VertexBuffer)
        Return True
    End Function
End Module
