﻿Public Class Viewport
    Public Viewport As Direct3D.Viewport
    Public BackgroundColor As Color = Color.Black
    Friend mstrName As String

    ' Friend mobjVPCollectionReference As Collections.Generic.Dictionary(Of String, Viewport)

    Public Property Name() As String
        Get
            Return mstrName
        End Get
        Set(ByVal value As String)
            If mstrName = DEFAULTVIEWPORT Then Exit Property
            If value Is Nothing Then Throw New ArgumentNullException("Value")
            If value.Trim = String.Empty Then Throw New ArgumentNullException("Value")
            If mobjViewports Is Nothing = False Then
                If mstrName Is Nothing = False Then
                    If mobjViewports.ContainsKey(mstrName) Then mobjViewports.Remove(mstrName)
                End If
            End If
            mstrName = value
            If mobjViewports Is Nothing = False Then
                mobjViewports.Add(mstrName, Me)
            End If
        End Set
    End Property
End Class

Public Module Viewports

    Friend mobjViewports As New Collections.Hashtable() ' .Generic.Dictionary(Of String, Viewport)
    Friend mstrCurrentViewport As String = String.Empty
    Public Const DEFAULTVIEWPORT As String = "{58A6DD2B-BFBF-4542-B931-9FF95A82F0B9}"
    Public Const CURRENTVIEWPORT As String = "{88CCE316-DCB1-4bc7-81C8-54649F8755A4}"


    Public Function ChangeViewportName(ByRef Name As String, ByVal Value As String) As Boolean
        If Name = CURRENTVIEWPORT Then Name = mstrCurrentViewport
        If Name = DEFAULTVIEWPORT Then Return False
        If mobjViewports.ContainsKey(Name) Then Return False
        If mobjViewports.ContainsKey(Value) Then Return False
        Dim Item As Viewport
        Item = DirectCast(mobjViewports(Name), Viewport)
        mobjViewports.Remove(Name)
        mobjViewports.Add(Value, Item)

        Return True
    End Function

    Public Function GetViewportNames(ByRef Names() As String) As Boolean
        Erase Names
        ReDim Names(mobjViewports.Count - 1)
        mobjViewports.Keys.CopyTo(Names, 0)

        Return True
    End Function

    Public Sub RemoveAllViewports()
        If mblnHasBeenInit = False Then Exit Sub
        Dim VP As Viewport = Nothing
        If mobjViewports.ContainsKey(DEFAULTVIEWPORT) Then VP = DirectCast(mobjViewports(DEFAULTVIEWPORT), Viewport)
        mobjViewports.Clear()
        If VP Is Nothing = False Then mobjViewports.Add(DEFAULTVIEWPORT, VP)
        mstrCurrentViewport = DEFAULTVIEWPORT
    End Sub

    Public Function GetNumberOfViewports() As Integer
        Return mobjViewports.Count
    End Function

    Public Function GetCurrentViewport() As Viewport
        If mobjViewports.ContainsKey(mstrCurrentViewport) = False Then Return Nothing
        Return DirectCast(mobjViewports(mstrCurrentViewport), Viewport)
    End Function

    Public Function SetCurrentViewport(ByVal Name As String, Optional ByVal ApplyViewport As Boolean = True) As Boolean
        If Name = CURRENTVIEWPORT Then Name = mstrCurrentViewport
        If mobjViewports.ContainsKey(Name) = False Then Return False
        If ApplyViewport And mblnHasBeenInit = False Then Return False
        If ApplyViewport Then mobjDevice.Viewport = DirectCast(mobjViewports(Name), Viewport).Viewport
        mstrCurrentViewport = Name

        Return True
    End Function

    Public Function RemoveViewport(ByVal Name As String) As Boolean
        If Name = DEFAULTVIEWPORT Then Return False
        If Name = CURRENTVIEWPORT Then Name = mstrCurrentViewport
        If mobjViewports.ContainsKey(Name) = False Then Return False
        If Name = mstrCurrentViewport Then mstrCurrentViewport = DEFAULTVIEWPORT
        ' mobjViewports(Name).mobjVPCollectionReference = Nothing
        mobjViewports.Remove(Name)
        Return True
    End Function

    Public Function AddViewport(ByVal Name As String, ByVal Value As Viewport) As Boolean
        If Name = CURRENTVIEWPORT Then Return False
        If Name = DEFAULTVIEWPORT Then Return False
        If mobjViewports.ContainsKey(Name) Then Return False
        ' Value.mobjVPCollectionReference = mobjViewports
        Value.mstrName = Name
        mobjViewports.Add(Name, Value)
        Return True
    End Function

    Public Function SetViewport(ByVal Name As String, ByVal Value As Viewport) As Boolean
        If Name = CURRENTVIEWPORT Then Name = mstrCurrentViewport
        If mobjViewports.ContainsKey(Name) = False Then Return False
        mobjViewports(Name) = Value
        Return True
    End Function

    Public Function ApplyViewport(ByVal Name As String) As Boolean
        If Name = CURRENTVIEWPORT Then Name = mstrCurrentViewport
        If mobjViewports.ContainsKey(Name) = False Then Return False
        ' If mblnBeginScene = False Then Return False
        mobjDevice.Viewport = DirectCast(mobjViewports(Name), Viewport).Viewport
        Return True
    End Function

    Public Function GetViewport(ByVal Name As String, ByRef Value As Viewport) As Boolean
        If Name = CURRENTVIEWPORT Then Name = mstrCurrentViewport
            If mobjViewports.ContainsKey(Name) = False Then Return False
            Value = DirectCast(mobjViewports(Name), Viewport)
        Return True
    End Function

    Public Function SetViewportBackgroundColor(ByVal Name As String, ByVal Value As Color) As Boolean
        If Name = CURRENTVIEWPORT Then Name = mstrCurrentViewport
        If mobjViewports.ContainsKey(Name) = False Then Return False
        CType(mobjViewports(Name), Viewport).BackgroundColor = Value
        Return True
    End Function

    Public Function GetViewportBackgroundColor(ByVal Name As String, ByRef Value As Color) As Boolean
        If Name = CURRENTVIEWPORT Then Name = mstrCurrentViewport
        If mobjViewports.ContainsKey(Name) = False Then Return False
        Value = CType(mobjViewports(Name), Viewport).BackgroundColor
        Return True
    End Function

    Public Function SetViewportWidth(ByVal Name As String, ByVal Value As Integer) As Boolean
        If Name = CURRENTVIEWPORT Then Name = mstrCurrentViewport
        If mobjViewports.ContainsKey(Name) = False Then Return False
        CType(mobjViewports(Name), Viewport).Viewport.Width = Value
        Return True
    End Function

    Public Function GetViewportWidth(ByVal Name As String, ByRef Value As Integer) As Boolean
        If Name = CURRENTVIEWPORT Then Name = mstrCurrentViewport
        If mobjViewports.ContainsKey(Name) = False Then Return False
        Value = CType(mobjViewports(Name), Viewport).Viewport.Width
        Return True
    End Function

    Public Function SetViewportHeight(ByVal Name As String, ByVal Value As Integer) As Boolean
        If Name = CURRENTVIEWPORT Then Name = mstrCurrentViewport
        If mobjViewports.ContainsKey(Name) = False Then Return False
        CType(mobjViewports(Name), Viewport).Viewport.Height = Value
        Return True
    End Function

    Public Function GetViewportHeight(ByVal Name As String, ByRef Value As Integer) As Boolean
        If Name = CURRENTVIEWPORT Then Name = mstrCurrentViewport
        If mobjViewports.ContainsKey(Name) = False Then Return False
        Value = CType(mobjViewports(Name), Viewport).Viewport.Height
        Return True
    End Function

End Module
